/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DomConfig.java,v 1.15 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.config;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.http.ServletUtil;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.logging.*;

import org.enhydra.barracuda.config.events.*;

/**
 * This class provides the methods needed to configure the dom
 * screen
 */
public class DomConfig extends DefaultEventGateway {

    protected static BCategory logger = BCategory.getBInstance(DomConfig.class.getName());

    //form id (must be unique)
    public static final String FORM = DomConfig.class.getName()+".Form";

    //model name
    public static final String MODEL_NAME = "Dom";

    //model elements (these are the data items supported)
    public static final String DEFAULT_DOM_LOADER_DL = "DefaultDOMLoader_DebugLevel";
    public static final String DEFAULT_DOM_WRITER_DL = "DefaultDOMWriter_DebugLevel";
    public static final String DEFAULT_DOM_WRITER_DPP = "DefaultDOMWriter_DefaultPrintPretty";
    public static final String ERROR_MESSAGE = "ErrorMessage";
    public static final String SUCCESS_MESSAGE = "SuccessMessage";
    public static final String UPDATE_BUTTON = "UpdateButton";

    //private vars
    private ListenerFactory updateConfigFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new UpdateConfigHandler();}        public String getListenerID() {return getID(UpdateConfigHandler.class);}};

    public DomConfig() {
        //specify generic interest
        specifyLocalEventInterests(updateConfigFactory);
    }

    //------------------------------------------------------------
    //                       Data Models
    //------------------------------------------------------------
    /**
     * define the template model that backs this screen
     */
    class DomModel extends AbstractTemplateModel {

        //register the model by name
        public String getName() {return MODEL_NAME;}
        
        //provide items by key
        public Object getItem(String key) {
            if (logger.isDebugEnabled()) logger.debug("Asking for key:"+key);    
            ViewContext vc = getViewContext();

            //Handle requests for data
            if (key.equals(DEFAULT_DOM_LOADER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultDOMLoader.class);
            } else if (key.equals(DEFAULT_DOM_WRITER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultDOMWriter.class);
            } else if (key.equals(DEFAULT_DOM_WRITER_DPP)) {
                return ScreenUtil.getToggleButton(vc, DEFAULT_DOM_WRITER_DPP, "true", DefaultDOMWriter.defaultPrintPretty);
            } else if (key.equals(ERROR_MESSAGE)) {
                return ScreenUtil.getErrMsg(vc, FORM, ERROR_MESSAGE);
            } else if (key.equals(SUCCESS_MESSAGE)) {
                return ScreenUtil.getSuccessMsg(vc, FORM, SUCCESS_MESSAGE);
            } else if (key.equals(UPDATE_BUTTON)) {
                return ScreenUtil.getUpdateButtonComp(vc, updateConfigFactory);
            } else return super.getItem(key);

        }
    }

    //------------------------------------------------------------
    //                  Form Mappings, Validators
    //------------------------------------------------------------
    /**
     * define the form map that backs the model
     */
    class DomForm extends DefaultFormMap {
        public DomForm() {
            //define the elements (note: these don't need any validators). Note
            //also that we set the defaults to the current values. This allows
            //us just to reset all current values down below without checking to
            //see if the elements actually got passed in from the form.
            if (logger.isDebugEnabled()) logger.debug("Defining form elements");            
            this.defineElement(new DefaultFormElement(DEFAULT_DOM_LOADER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(DefaultDOMLoader.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_DOM_WRITER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(DefaultDOMWriter.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_DOM_WRITER_DPP, FormType.BOOLEAN, new Boolean(false), null, false));    //if a checkbox element is not present its equiv to being false
        }
    }

    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * UpdateConfigHandler - handle the request to update the config 
     * screen.
     */
    class UpdateConfigHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //figure out our target locale and get the appropriate screen
            Locale locale = context.getViewCapabilities().getClientLocale();
            MasterScreen screen = new MasterScreenFactory().getInstance(DomConfig.this, context, locale);
            if (logger.isDebugEnabled()) ServletUtil.showParams(context.getRequest(), logger);            

            //create the login form
            ValidationException ve = null;
            DomForm fm = new DomForm();
            try {
                //map/validate the form
                fm.map(context.getRequest()).validate(true);
                
                //If there were no errors, update the data. Note that we just
                //assume that all the data is here. The reason we can do this is
                //because we prepopulated the form up above with the default values.
                //The more proper way would be to check each value, see if it's set,
                //and then only update it if the value has actually changed. Lot more
                //code to do that though, and since we're just setting statics, the
                //cost of doing it this way is minimal.
                ScreenUtil.setPriority(DefaultDOMLoader.class, fm.getIntegerVal(DEFAULT_DOM_LOADER_DL).intValue());
                ScreenUtil.setPriority(DefaultDOMWriter.class, fm.getIntegerVal(DEFAULT_DOM_WRITER_DL).intValue());
                DefaultDOMWriter.defaultPrintPretty = fm.getBooleanVal(DEFAULT_DOM_WRITER_DPP).booleanValue();

                //remember our success
                fm.putState(SUCCESS_MESSAGE, new Boolean(true));

            } catch (ValidationException e) {
                ve = e;
            }

            //store the error message in the form and then put the form in the
            //the event context
            fm.putState(ERROR_MESSAGE, ve);
            context.putState(FORM, fm);

            //fire an update so that the screen will redraw
            ((DomModel) screen.domModel).fireModelChanged();

            //redirect to the get screen again
            throw new ClientSideRedirectException(new GetBConfig());
        }
    }

    //------------------------------------------------------------
    //                         Utility Methods
    //------------------------------------------------------------
    public TemplateModel getModel() {
        return new DomModel();
    }
}

