/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: BAction.java,v 1.12 2001/10/03 21:25:20 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.events.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.logging.*;


/**
 * A BAction component is a component that you can use to catch
 * client side events.
 */
public class BAction extends BComponent {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(BAction.class.getName());
    
    //private vars

    /**
     * @clientCardinality 1
     */
    protected ControlEvent actionEvent = null;
    protected String actionUrl = null;
    protected List listeners = null;
    private Map params = null;
    protected boolean disableBackButton = false;

    /**
     * @clientCardinality 0..*
     */
    private ListenerFactory lnkListenerFactory;
    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BAction() {}
    
    /**
     * Public constructor which creates the component and
     * binds it to a view
     *
     * @param view the View the component should be bound to
     */
    public BAction(View view) {
        if (view!=null) this.addView(view);
    }

    /**
     * Public constructor which creates the component and binds it
     * to a view. Also allows you to specify that a custom url be fired.
     *
     * <p>Null values may be passed in for any parameters,
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually
     * rendering the component
     *
     * @param actionUrl the action url to be fired
     * @param view the View the component should be bound to
     */
    public BAction(String actionUrl, View view) {
        if (actionUrl!=null) this.setAction(actionUrl);
        if (view!=null) this.addView(view);
    }
    
    /**
     * Public constructor which creates the component and binds it
     * to a view. Also allows you to specify that a custom event be fired.
     *
     * <p>Null values may be passed in for any parameters,
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually
     * rendering the component
     *
     * @param actionEvent the action event to be fired
     * @param view the View the component should be bound to
     */
    public BAction(ControlEvent actionEvent, View view) {
        if (actionEvent!=null) this.setAction(actionEvent);
        if (view!=null) this.addView(view);
    }
    


    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BAction.class, HTMLElement.class);
    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLActionRenderer();}
    }




    //--------------- BAction ------------------------------------
    /**
     * Set the action to be fired by this component.
     *
     * @param url the URL representing the action to be fired
     */
    public void setAction(String iactionUrl) {
        actionUrl = iactionUrl;
        invalidate();
    }
    
    /**
     * Set the action to be fired by this component.
     *
     * @param actionEvent the event to be fired by this component
     */
    public void setAction(ControlEvent iactionEvent) {
        actionEvent = iactionEvent;
        invalidate();
    }
    
    /**
     * Get the action to be fired by this component. The action
     * will either be the URL or the Event that backs this component
     *
     * @vc the ViewContext that determines the context of the action
     * @return a string representing the action to be fired by this component
     */
    public String getAction(ViewContext vc) {
        StringBuffer sb = new StringBuffer(200);
        String sep = "?";

        //build the base action
        if (actionUrl!=null) {
            //from the url
//            sb.append(actionUrl);
            sb.append(URLRewriter.encodeURL(vc, actionUrl));    //take into account the need for URL rewriting
            
        } else {
            //from the event
            if (actionEvent==null) actionEvent = new ActionEvent();
            // RTL20010719 sb.append(actionEvent.getEventID()+actionEvent.getEventExtension());
//            sb.append(actionEvent.getEventIDWithExtension());
            sb.append(URLRewriter.encodeURL(vc, actionEvent.getEventIDWithExtension()));    //take into account the need for URL rewriting

            //now, see if we have event listeners
            if (listeners!=null && listeners.size()>0) {
                Iterator it = listeners.iterator();
                while (it.hasNext()) {
                    ListenerFactory lf = (ListenerFactory) it.next();
                    sb.append(sep+BaseEvent.EVENT_ID+"="+lf.getListenerID());
                    sep = "&";
                }
            }                
        }

        //finally add any custom params
        if (params!=null && params.size()>0) {
            Iterator it = params.keySet().iterator();
            while (it.hasNext()) {
                Object key = it.next();
                Object val = params.get(key);
//csc_100301.2_start - we really need to be encoding these paramters
//                if (key!=null && val!=null) sb.append(sep+key+"="+val);
                if (key!=null && val!=null) {
                    sb.append(sep);
                    sb.append(URLEncoder.encode(key.toString()));
                    sb.append("=");
                    sb.append(URLEncoder.encode(val.toString()));
                }
//csc_100301.2_end
                sep = "&";
            }
        }

        return sb.toString();
    }
    
    /**
     * Set any associated params
     */
    public void setParam(String key, String val) {
        if (params==null) params = new HashMap(10);
        params.put(key, val);
    }

    /**
     * Get any associated params
     */
    public Map getParams() {
        return params;
    }

    /**
     * Set disable back button (only works if your client supports
     * Javascript)
     *
     * @param disable true if we want the back button disabled 
     */
    public void setDisableBackButton(boolean idisableBackButton) {
        disableBackButton = idisableBackButton;
    }

    /**
     * Get disable back button
     *
     * @return true if we want the back button disabled 
     */
    public boolean getDisableBackButton() {
        return disableBackButton;
    }

    /**
     * Add an event listener to this component.
     *
     * @param lf the event listener to be added
     */
    public void addEventListener(ListenerFactory lf) {
        if (lf==null) return;
        if (listeners==null) listeners = new ArrayList(5);
        listeners.add(lf);    
        invalidate();
    }
    
    /**
     * Remove an event listener from this component
     *
     * @param lf the event listener to be removed
     */
    public void removeEventListener(ListenerFactory lf) {
        if (lf==null) return;
        if (listeners==null) return;
        listeners.remove(lf);    
        invalidate();
    }

    

    //--------------- BComponent ---------------------------------
    /**
     * Render a specific view for the component.
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View is not supported
     * @param list a List of all the views for this component
     */
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {
        if (logger.isInfoEnabled()) logger.info(depth, "rendering view: "+view);

        //actually render the view according to known interfaces
        try {
            Renderer r = getRenderer(view);
            r.renderComponent(this, view, vc);
            
        } catch (DOMException e) {
            logger.warn(depth+2, "DOM Error:", e);
            throw new DOMAccessException("Error rendering component in view:"+e, e);
        }
    }
}