/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BComponent.java,v 1.16 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.util.logging.*;


/**
 *
 */
public class BComponent extends AbstractBComponent {
    //constants
    public static final String VISIBILITY_MARKER = "visdom";
    public static final String CACHING_MARKER = "cachdom";

    //private vars
//NOTE: the name must remain null unless specifically set; setting the name
//will cause the name attribute to be set in the markup to which the component
//is bound (meaning, if the markup specifies a form name that will get overwritten
//if you specify a name)
//    protected String name = this.getClass().getName();
    protected String name = null;
    protected boolean visible = true;
    protected boolean enabled = true;
    protected Map attrs = null;

    /**
     * @clientCardinality 0..* 
     */
    private View lnkView;
    
    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BComponent.class, HTMLElement.class);
        installRendererFactory(rfHTML, BComponent.class, HTMLDocument.class);

        XMLRendererFactory rfXML = new XMLRendererFactory();
        installRendererFactory(rfXML, BComponent.class, Node.class);
    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLComponentRenderer();}
    }

    /**
     * XML RendererFactory
     */
    static class XMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new XMLComponentRenderer();}
    }



    //--------------- BComponent ---------------------------------
    /**
     * Set the name for this component. 
     *
     * <p>Note that for several types of views (HTMLAnchorElement, HTMLAppletElement, 
     * HTMLButtonElement, HTMLFormElement, HTMLFrameElement, HTMLIFrameElement, 
     * HTMLInputElement, HTMLMapElement, HTMLMetaElement, HTMLObjectElement, 
     * HTMLParamElement, HTMLSelectElement, and HTMLTextAreaElement) this property 
     * will be used in rendering if it is actually set. This means that if you set 
     * the component name, and it is bound to a view that is backed by one of these
     * nodes, then the name attribute in that node will be overridden. So be careful!!!
     *
     * @param name the name for this component
     */
    public void setName(String iname) {
        name = iname;
    }
    
    /**
     * Get the name for this component
     *
     * @return the name for this component
     */
    public String getName() {
        return name;
    }
        
    /**
     * Set the component visibility
     *
     * @param true if the component should be visible
     */
    public void setVisible(boolean val) {
        setVisible(val, false);
    }
    
    /**
     * Set the component visibility recursively
     *
     * @param true if the component should be visible
     * @param true if we want to set this value recursively
     */
    public void setVisible(boolean val, boolean recurse) {
        visible = val;
        if (recurse) {
            Iterator it = children.iterator();
            while (it.hasNext()) {
                BContainer child = (BContainer) it.next();
                if (child!=null && child instanceof BComponent) {
                    BComponent wcomp = (BComponent) child;
                    wcomp.setVisible(val, recurse);
                }
            }
        }
        invalidate();
    }
    
    /**
     * Get the component visibility
     *
     * @return true if the component is visible
     */
    public boolean isVisible() {
        return visible;    
    }
    
    /**
     * Enable/disable the component 
     *
     * @param true if the component should be enabled
     */
    public void setEnabled(boolean val) {
        setEnabled(val, false);
    }
    
    /**
     * Enable/disable the component recursively
     *
     * @param true if the component should be enabled
     * @param true if we want to set this value recursively
     */
    public void setEnabled(boolean val, boolean recurse) {
        enabled = val;
        if (recurse) {
            Iterator it = children.iterator();
            while (it.hasNext()) {
                BContainer child = (BContainer) it.next();
                if (child!=null && child instanceof BComponent) {
                    BComponent wcomp = (BComponent) child;
                    wcomp.setEnabled(val, recurse);
                }
            }
        }
        invalidate();
    }

    /**
     * See if the component is enabled
     *
     * @return true if the component is enabled
     */
    public boolean isEnabled() {
        return enabled;    
    }
    
    /**
     * Set the components primary view. This method effectively
     * removes any other views and binds the component to the 
     * newly specified view.
     *
     * @param view the view to which this component is bound
     */
    public void setView(View view) {
        removeAllViews();
        addView(view);
    }
    
    /**
     * Bind a component to a view
     *
     * @param view the view to which this component is bound
     */
    public void addView(View view) {
        if (view==null || views.contains(view)) return;
        views.add(view);
        invalidate();
    }
    
    /**
     * Remove a view from this component
     *
     * @return true if we were able to remove the view from the component
     */
    public boolean removeView(View view) {
        if (view==null) return false;
        invalidate();
        return views.remove(view);    
    }
    
    /**
     * Remove all views from this component
     */
    public void removeAllViews() {
        if (views!=null) views.clear();
        invalidate();
    }
    
    /**
     * Get a list of all the views for this component. This returns
     * a copy of the underlying view list.
     *
     * @return a List of all the views for this component
     */
    public List getViews() {
        return new ArrayList(views);
    }
    
    protected View getFirstView() {
        //see if we have any views
        if (views!=null && views.size()>0) {
            return (View) views.get(0);
        }
        
        //if not, see if the children have any views
        View firstView = null;
        if (children!=null) {
            Iterator it = children.iterator();
            while (it.hasNext()) {
                Object o = it.next();
                if (o instanceof BComponent) {
                    firstView = ((BComponent) o).getFirstView();
                    if (firstView!=null) break;
                }
            }
        }
        return firstView;
    }


    
    /**
     * set an attribute for this particular component. When the component
     * is rendered, component attributes will be shown as element attributes
     * in the elements that back each of the views associated with this component.
     * This means that if you set an attribute for the component, it will 
     * affect all views associated with the component.If you wish to set an 
     * attribute for a specific view alone, then you should get the view, find
     * the node that backs it, and tehn set the attribute manually that way.
     *
     * @param attr the attribute name
     * @param val the attribute value
     */    
    public void setAttr(Object attr, Object val) {
        if (attrs==null) attrs = new HashMap();
        attrs.put(attr,val);
        invalidate();
    }
    
    /**
     * get an attribute associated with this particular component. Note that
     * the attribute map that backs this method only keeps tracks of specific
     * attributes you have added to the component. It does not look at attributes
     * that are physically associated with the underlying elements that back each
     * of the views associated with this component. What this means is that if
     * the template that backs a view has some attribute "foo" and you try to
     * see the value of that attribute using this method, you will not be able 
     * to find it unless you have actually associated an attribute named "foo" 
     * with the specific component.
     *
     * @param attr the attribute name
     * @return the value for the given attribute (may be null)
     */    
    public Object getAttr(Object attr) {
        if (attrs==null) return null;
        return attrs.get(attr);
    }
    
    /**
     * get a copy of the underlying component attribute Map
     *
     * @return a copy of the underlying component attribute Map
     */    
    public Map getAttrMap() {
        if (attrs==null) return null;
        return new HashMap(attrs);    
    }

    /**
     * Render the component for a view with the specified 
     * ViewContext. You shouldn't override this method unless
     * you really know what you're doing...
     *
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View cannot be rendered
     */
    public void render(ViewContext vc) throws RenderException {
        render(vc, 0);
    }
    
    protected void render(ViewContext vc, int depth) throws RenderException {
    
        //if the component is not already validated, re-render it
        Iterator it = null;
        if (!validated) {
            try {
                //prepare for rendering 
                if (logger.isInfoEnabled()) logger.info(depth, "preparing to render comp "+this.toRef());
//                preRender(vc, 0);
                preRender(vc, depth);

                //add in any temp views
                if (logger.isInfoEnabled()) logger.info(depth, "adding in temp views "+this.toRef());
                if (tempViews!=null) views.addAll(tempViews);
            
                //render our own views
                if (logger.isDebugEnabled()) logger.debug(depth, "rendering our own view in comp "+this.toRef());
                View view = null;
                it = views.iterator();
                while (it.hasNext()) {
                    view = (View) it.next();
                    if (view==null) continue;
                    try {
                        if (view.getNode() instanceof Element) {
                            Element el = (Element) view.getNode();

                            //set visibility
                            if (isVisible()) {
                                el.removeAttribute(VISIBILITY_MARKER);
                            } else {
                                el.setAttribute(VISIBILITY_MARKER, "false");
                                return; //if the component is not visible, there is no need to go any further!!!
                            }
                        
                            //set any attributes
                            if (attrs!=null) {
                                Iterator it2 = attrs.keySet().iterator();
                                while (it2.hasNext()) {
                                    Object key = it2.next();
                                    Object val = attrs.get(key);
                                    if (val==null) {
                                        el.removeAttribute(key.toString());
                                    } else {
                                        el.setAttribute(key.toString(), val.toString());                            
                                    }
                                }
                            }
                        }
                    
                        //render the view
                        vc.putState(ViewContext.ELEMENT_FACTORY, view.getElementFactory());
                        renderView(view, vc, depth+1);
                    } catch (RenderException e) {
                        //for now just consume the exceptions
                        logger.warn(depth, "RenderException:"+e+" for View:"+view, e);
                    }
                }
            
                //now invoke render in all of our children
                if (logger.isDebugEnabled()) logger.debug(depth, "rendering our children in comp "+this.toRef()+(this.hasChildren() ? "" : "(n/a)"));
                BContainer child = null;
                it = children.iterator();
                while (it.hasNext()) {
                    child = (BContainer) it.next();
                    if (child==null) continue;
                    try {
                        if (child instanceof BComponent) {
                            BComponent wcomp = (BComponent) child;
                            if (wcomp.supports(vc)) wcomp.render(vc, depth+1);
                        }
                    } catch (RenderException e) {
                        //for now just consume the exceptions
                        logger.warn(depth, "RenderException:"+e+" for Child:"+child, e);
                    }
                }
            } finally {
                //allow for any cleanup after render
                if (logger.isDebugEnabled()) logger.debug(depth, "cleaning up after render in comp "+this.toRef()+(this.hasChildren() ? "" : "(n/a)"));
//               postRender(vc, 0);
                postRender(vc, depth);

                //cleanup any step children
                removeAllStepChildren();

                //cleanup any temp views
                if (tempViews!=null) 
                if (tempViews!=null) {
                    it = tempViews.iterator();
                    while (it.hasNext()) {
                        View tempView = (View) it.next();
                        this.removeView(tempView);
                    }
                    tempViews = null;
                }
            
                //now consider ourselves validated
                validated = true;
            }
        }
                
        //if we are the root component, check to see if we need to prevent 
        //caching. If so, we need to get a reference to the doc, find the first
        //element, and set the caching marker. The DOM writer will strip this 
        //out (which is why we ALWAYS need to do this, even if we're validated)
        if (parent==null && getPreventCaching()) {
            if (logger.isDebugEnabled()) logger.debug(depth, "checking for caching marker");
            View v = getFirstView();
            if (v!=null) {
                Node n = v.getNode();
                Document doc = n.getOwnerDocument();
                Element el = doc.getDocumentElement();
                el.setAttribute(CACHING_MARKER, "true");
            }
        }
        
        if (logger.isInfoEnabled()) logger.info(depth, "rendering complete in comp "+this.toRef());
    }
    
    /**
     * Prep phase before rendering. This is typically where you would put
     * any pre-rendering specific logic.
     */
//    protected final void preRender(ViewContext vc) {
//        preRender(vc, 0);
//    }

    protected void preRender(ViewContext vc, int depth) {
        //this is actually the method you should override when implementing
        //pre-render logic
        
        //(--n/a--)
    }

    /**
     * Render a specific view for the component. 
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View cannot be rendered
     * @param list a List of all the views for this component
     */
    protected final void renderView (View view, ViewContext vc) throws RenderException {
        renderView(view, vc, 0);
    } 
    
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {

        //This is where we would put custom rendering 
        //code (think of this method as the equivalent 
        //of AWT's paint()...
        //
        //The base BComponent class is headless (no visual
        //element, used primarily for containment) so we'll
        //never have anything to render here.
    }

    /**
     * Cleanup after rendering. This method is guaranteed to be invoked, even 
     * if there is an error during rendering. This is typically where you would
     * put any custom post-rendering cleanup.
     */
//    protected final void postRender(ViewContext vc) {
//        postRender(vc, 0);
//  }

    protected void postRender(ViewContext vc, int depth) {
        //this is actually the method you should override when implementing
        //post-render logic
        
        //(--n/a--)
    }

    /**
     * Determine if a specific ViewContext is supported
     * by this component
     *     
     * @param vc ViewContext for the client view
     * @returns true if the specified ViewCapabilites are supported
     */
    public boolean supports(ViewContext vc) {
        return true;
    }

}