/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BInput.java,v 1.12 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 *
 * Q's:
 */
public class BInput extends BComponent {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(BInput.class.getName());

    //valid types
    public static final String TEXT = "text";
    public static final String PASSWORD = "password";
    public static final String SUBMIT = "submit";
    public static final String RESET = "reset";
    public static final String FILE = "file";
    public static final String HIDDEN = "hidden";
    public static final String IMAGE = "image";
    public static final String BUTTON = "button";
    public static final String RADIO = "radio";
    public static final String CHECKBOX = "checkbox";
    
    //private vars
    protected List listeners = null;
    protected String type = null;
    protected String value = null;
    protected boolean disableBackButton = false;

    /**
     * @clientCardinality 0..* 
     */
    private ListenerFactory lnkListenerFactory;
    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BInput() {}
    
    /**
     * Public constructor which creates the component and
     * binds it to a specific view
     *
     * @param view the View the component should be bound to
     */
    public BInput(View iview) {
        this(null, null, null, iview, null);
    }
    
    /**
     * Public constructor which creates the component and
     * sets the initial data. The component is also
     * bound to the specified view.
     *
     * <p>Null values may be passed in for any parameters, 
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually 
     * rendering the component
     *
     * @param type valid input type. May be null (indicating don't render 
     *        this attribute)
     * @param name the name of the button, or null (indicating 
     *         don't render this attribute)
     * @param value a String value for the input. May be null (indicating 
     *        don't render this attribute)
     * @param view the View the component should be bound to
     * @param dvc the default ViewContext (opt--its presence allows the 
     *         component to be rendered as markup in toString())
     */
    public BInput(String itype, String iname, String ivalue, View iview, ViewContext idvc) {
        if (idvc!=null) setDefaultViewContext(idvc);
        if (itype!=null) setType(itype);
        if (iname!=null) setName(iname);
        if (ivalue!=null) setValue(ivalue);
        if (iview!=null) setView(iview);
    }
    



    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BInput.class, HTMLElement.class);
        installRendererFactory(rfHTML, BInput.class, HTMLDocument.class);

    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLInputRenderer();}
    }




    //--------------- BInput --------------------------------------
    /**
     * Set the input type (TEXT, PASSWORD, SUBMIT, RESET, FILE, HIDDEN,
     * IMAGE, BUTTON, RADIO, or CHECKBOX). If this value remains null, 
     * the type will default to whatever is specified in the underlying 
     * markup. If you set this value, then the type will be overridden 
     * in all the views associated with this component.
     *
     * @param type an string value representing the size.
     */
    public void setType(String itype) {
        itype = itype.toLowerCase();
        if (itype!=null && !itype.equals(TEXT) && !itype.equals(PASSWORD) &&
                           !itype.equals(SUBMIT) && !itype.equals(RESET) &&
                           !itype.equals(FILE) && !itype.equals(HIDDEN) &&
                           !itype.equals(IMAGE) && !itype.equals(BUTTON) &&
                           !itype.equals(RADIO) && !itype.equals(CHECKBOX)) {
            itype = TEXT;
        }
        type = itype;
        invalidate();
    }
    
    /**
     * Get the type of input. May return a null if the type has not been 
     * manually specified. 
     *
     * @return the component type
     */
    public String getType() {
        return type;
    }

    /**
     * Set the value for this input. In most cases, the value object will 
     * simply be rendered as the 'value' attribute. If this value remains null, 
     * the type will default to whatever is specified in the underlying 
     * markup. If you set this value, then the type will be overridden in all 
     * the views associated with this component.
     *
     * @param value the value object
     */
    public void setValue(String ivalue) {
        value = ivalue;
        invalidate();
    }
    
    /**
     * Get the value for this input. May return a null if the type has not been 
     * manually specified. 
     *
     * @return the value for this input
     */
    public String getValue() {
        return value;
    }

    /**
     * Add an event listener to this component. 
     *
     * @param lf the event listener to be added
     */
    public void addEventListener(ListenerFactory lf) {
        addEventListener(lf, false);
    }
    
    /**
     * Add an event listener to this component. 
     *
     * @param lf the event listener to be added
     * @param disableBackButton true if the back button should be 
     *      disabled when the action occurs
     */
    public void addEventListener(ListenerFactory lf, boolean idisableBackButton) {
        if (lf==null) return;
        disableBackButton = idisableBackButton;
        if (listeners==null) listeners = new ArrayList(5);
        listeners.add(lf);    
        invalidate();
    }
    
    /**
     * Remove an event listener from this component
     *
     * @param lf the event listener to be removed
     */
    public void removeEventListener(ListenerFactory lf) {
        if (lf==null) return;
        if (listeners==null) return;
        listeners.remove(lf);    
        invalidate();
    }

    /**
     * Here in the pre-render phase we actually add
     * BAction step children for any of the listeners
     * that might have been added to this component
     */
    protected void preRender(ViewContext vc, int depth) {
        //we want to actually add proxy components as step children
        //for any event listeners we might have to support
        if (listeners!=null) {
            //run through our list of listeners
            Iterator it = listeners.iterator();
            while (it.hasNext()) {
                ListenerFactory lf = (ListenerFactory) it.next();
            
                //run through our list of views
                Iterator it2 = getViews().iterator();
                while (it2.hasNext()) {
                    View v = (View) it2.next();
                    BAction baComp = new BAction(v);
                    baComp.setDisableBackButton(disableBackButton);
                    baComp.addEventListener(lf);
                    this.addStepChild(baComp, true);
                }
            }
        }
    }

    /**
     * Render a specific view for the component. 
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View is not supported
     */
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {
        if (logger.isInfoEnabled()) logger.info(depth, "rendering comp:"+this.toRef()+" view:"+view);

        //actually render the view according to known interfaces
        try {
            Renderer r = getRenderer(view);
            r.renderComponent(this, view, vc);
        } catch (DOMException e) {
            logger.warn(depth+2, "DOM Error:", e);
            throw new DOMAccessException("Error rendering component in view:"+e, e);
        }
    }

}