/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BLink.java,v 1.13 2001/09/27 21:16:43 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.events.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.util.logging.*;


/**
 * This class creates a link component that represents a URL. If you
 * do not pass in a previously created view, you must invoke 
 * setDefaultViewContext or the component will not be rendered 
 * properly because it is not bound to an underlying node. You will
 * not need to invoke this method if you provide your own view or if
 * you use the contructor that takes a ViewContext object.
 */
public class BLink extends BAction {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(BLink.class.getName());
    
    //private vars
    protected String text = null;
    protected String href = null;
    protected String target = null;
    protected boolean allowMarkupInText = false;    //csc_092701.1

    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BLink() {}
    
    /**
     * Public constructor which creates the component and binds it to a 
     * view. 
     *
     * @param view the View the component should be bound to
     */
    public BLink(View iview) {
        if (iview!=null) addView(iview);
    }

    /**
     * Public constructor which creates the component and sets the text, 
     * and target values. This link will fire the default action event (unless
     * you manually specify an action). This constructor requires a ViewContext 
     * object which will be used to create a default view for the component
     *
     * @param text the text string that backs this component
     * @param dvc the default ViewContext (opt--its presence allows the 
     *         component to be rendered as markup in toString())
     */
    public BLink(String itext, ViewContext idvc) {
        if (idvc!=null) setDefaultViewContext(idvc);
        if (itext!=null) setText(itext);
    }

    /**
     * Public constructor which creates the component and sets the text, 
     * action, and target values. This constructor requires a ViewContext 
     * object which will be used to create a default view for the component
     *
     * @param text the text string that backs this component
     * @param actionUrl the action url to be fired
     * @param dvc the default ViewContext (opt--its presence allows the 
     *         component to be rendered as markup in toString())
     */
    public BLink(String itext, String iactionUrl, ViewContext idvc) {
        if (idvc!=null) setDefaultViewContext(idvc);
        if (itext!=null) setText(itext);
        if (iactionUrl!=null) setAction(iactionUrl);
    }

    /**
     * Public constructor which creates the component and sets the text, 
     * action, and target values. This constructor requires a ViewContext 
     * object which will be used to create a default view for the component
     *
     * @param text the text string that backs this component
     * @param actionEvent the action event to be fired (if null, the default
     *        ActionEvent will be fired)
     * @param dvc the default ViewContext (opt--its presence allows the 
     *         component to be rendered as markup in toString())
     */
    public BLink(String itext, ControlEvent iactionEvent, ViewContext idvc) {
        if (idvc!=null) setDefaultViewContext(idvc);
        if (itext!=null) setText(itext);
        if (iactionEvent!=null) setAction(iactionEvent);
    }



    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BLink.class, HTMLElement.class);
    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLLinkRenderer();}
    }



    //--------------- BLink --------------------------------------
    /**
     * Set the text for this particular component
     *
     * @param text the text representation of this component
     */
    public void setText(String itext) {
        text = itext;
        invalidate();
    }
    
    /**
     * Get the text for this particular component
     *
     * @return the text for this particular component
     */
    public String getText() {
        return text;    
    }
    
    /**
     * Set the target for this particular component
     *
     * @param target the ext representation of the target
     */
    public void setTarget(String itarget) {
        target = itarget;
        invalidate();
    }
    
    /**
     * Get the target for this particular component
     *
     * @return the target for this particular component
     */
    public String getTarget() {
        return target;    
    }

    /**
     * Do we wish to allow markup in this text (defaults to false)
     *
     * @param val true if we wish to allow markup in the text
     */
    public void setAllowMarkupInText(boolean val) {
        allowMarkupInText = val;
        invalidate();
    }
    
    /**
     * See if we allow markup in the text
     *
     * @return true if we wish to allow markup in the text
     */
    public boolean allowMarkupInText() {
        return allowMarkupInText;    
    }
    
    
}