/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BTemplate.java,v 1.8 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.model.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 *
 * Q's:
 */
public class BTemplate extends BComponent {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(BTemplate.class.getName());

    
    //private vars
    protected HashMap templateModels = new HashMap();
    private LocalModelListener callback = new LocalModelListener();
    protected Node templateNode = null;
    protected Stack itStack = null;

    /**
     * @clientCardinality 1..* 
     */
    private TemplateModel lnkTemplateModel;


    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BTemplate() {
        this (null, null);
    }
    
    /**
     * Public constructor which creates the component and
     * binds it to a specific model.
     *
     * @param view the View the component should be bound to
     */
    public BTemplate(TemplateView iview) {
        this (null, iview);
    }
    
    /**
     * Public constructor which creates the component and
     * binds it to a specific model. The component is also
     * bound to the specified view. 
     * 
     * <p>Null values may be passed in for any parameters, 
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually 
     * rendering the component
     *
     * @param model the specific model to back this component
     * @param view the View the component should be bound to
     */
    public BTemplate(TemplateModel imodel, TemplateView iview) {
        if (imodel!=null) addModel(imodel);
        if (iview!=null) this.addView(iview);    
    }
    



    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BTemplate.class, HTMLElement.class);
        installRendererFactory(rfHTML, BTemplate.class, HTMLDocument.class);

        XMLRendererFactory rfXML = new XMLRendererFactory();
        installRendererFactory(rfXML, BTemplate.class, Node.class);
    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLTemplateRenderer();}
    }

    /**
     * XML RendererFactory
     */
    static class XMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new XMLTemplateRenderer();}
    }



    //--------------- BTemplate ---------------------------
    /**
     * Add a model to the component. Unlike other components, the template
     * component can have ant number of models...
     *
     * @param model a model that backs the template
     */
    public void addModel(TemplateModel imodel) {
        //eliminate the obvious
        if (imodel==null) return;
    
        //add the model    and set the default if necess.
        templateModels.put(imodel.getName(), imodel);
        invalidate();
        
        //reregister if possible
        imodel.addModelListener(callback);
    }
    
    /**
     * Add a whole list of models to the component. 
     *
     * @param list a list of TemplateModels to back the component
     */
    public void addModels(List ilist) {
        //eliminate the obvious
        if (ilist==null) return;
    
        //iterate through the list, adding any instances of TemplateModel
        Iterator it = ilist.iterator();
        while (it.hasNext()) {
            Object o = it.next();
            if (o instanceof TemplateModel) addModel((TemplateModel) o);
        }
    }
    
    /**
     * Remove a model from the component. 
     *
     * @param model a model that backs the template
     */
    public void removeModel(TemplateModel imodel) {
        //eliminate the obvious
        if (imodel==null) return;
    
        //add the model    and set the default if necess.
        templateModels.remove(imodel.getName());
        invalidate();
        
        //deregister if possible
        imodel.removeModelListener(callback);
    }

    /**
     * Remove a model from the component by model name
     *
     * @param name the name of the model to be removed
     */
    public void removeModel(String modelName) {
        removeModel(getModel(modelName));
    }

    /**
     * Get the model that backs the table
     *
     * @param name the name of the model we're interested in
     * @return the model that backs the table
     */
    public TemplateModel getModel(String modelName) {
        return (TemplateModel) templateModels.get(modelName);    
    }
    
    /**
     * Get a list of models associated with this view
     *
     * @return a list of models associated with this view
     */
    public List getModels() {
        return new ArrayList(templateModels.values());    
    }
    
    /**
     * Render a specific view for the component. 
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View is not supported
     * @param list a List of all the views for this component
     */
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {
        if (logger.isInfoEnabled()) logger.info(depth, "rendering view: "+view);

        //actually render the view according to known interfaces
        try {
            Renderer r = getRenderer(view);
            r.renderComponent(this, view, vc);
        } catch (DOMException e) {
            logger.warn(depth+2, "DOM Error:", e);
            throw new DOMAccessException("Error rendering component in view:"+e, e);
        }
    }




    //--------------- Lifecycle ----------------------------------
    /**
     * Destroy cycle. The component should use this cycle to
     * perform any special cleanup.
     */
    public void destroyCycle() {
        //default destroy
        super.destroyCycle();
    
        //we set the model to null so that the component can be 
        //garbage collected. If we don't do this, the model retains
        //a reference back to the component and so the component 
        //will never be freed up...
        templateModels = null;        
    }




    //--------------- Utility ------------------------------------
/*
    class LocalTemplateModelListener implements TemplateModelListener {
        //get notified when one of the underlying models changes
        public void modelChanged(TemplateModelEvent e) {
            invalidate();
        }
    }
*/    
    class LocalModelListener implements ModelListener {
        //get notified when one of the underlying models changes
        public void modelChanged(Model m) {
            invalidate();
        }
    }
}