/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BText.java,v 1.7 2001/09/27 21:16:43 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.util.logging.*;


/**
 *
 */
public class BText extends BComponent {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(BText.class.getName());
    
    //private vars
    protected String text = null;
    protected boolean allowMarkupInText = false;    //csc_092701.1

    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BText() {}
    
    /**
     * Public constructor which creates the component and
     * binds it to a view, and sets the text
     *
     * <p>Null values may be passed in for any parameters, 
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually 
     * rendering the component
     *
     * @param text the text string that backs this component
     * @param view the View the component should be bound to
     */
    public BText(String text, View view) {
        if (text!=null) this.setText(text);
        if (view!=null) this.addView(view);
    }



    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BText.class, HTMLElement.class);
        installRendererFactory(rfHTML, BText.class, HTMLDocument.class);
/*        
        WMLRendererFactory rfWML = new WMLRendererFactory();
        installRendererFactory(rfWML, BText.class, HTMLElement.class);
        installRendererFactory(rfWML, BText.class, HTMLDocument.class);
*/        
        XMLRendererFactory rfXML = new XMLRendererFactory();
        installRendererFactory(rfXML, BText.class, Node.class);
    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLTextRenderer();}
    }

    /**
     * XML RendererFactory
     */
    static class XMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new XMLTextRenderer();}
    }


    //--------------- BComponent ---------------------------------
    /**
     * Set the text for this particular component
     *
     * @param text the text representation of this component
     */
    public void setText(String itext) {
        text = itext;
        invalidate();
    }
    
    /**
     * Get the text for this particular component
     *
     * @return the text for this particular component
     */
    public String getText() {
        return text;    
    }
    
    /**
     * Do we wish to allow markup in this text (defaults to false)
     *
     * @param val true if we wish to allow markup in the text
     */
    public void setAllowMarkupInText(boolean val) {
        allowMarkupInText = val;
        invalidate();
    }
    
    /**
     * See if we allow markup in the text
     *
     * @return true if we wish to allow markup in the text
     */
    public boolean allowMarkupInText() {
        return allowMarkupInText;    
    }
    
    /**
     * Render a specific view for the component. 
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View is not supported
     * @param list a List of all the views for this component
     */
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {
        if (logger.isInfoEnabled()) logger.info(depth, "rendering view: "+view);

        //actually render the view according to known interfaces
        try {
            Renderer r = getRenderer(view);
            r.renderComponent(this, view, vc);
        } catch (DOMException e) {
            logger.warn(depth+2, "DOM Error:", e);
            throw new DOMAccessException("Error rendering component in view:"+e, e);
        }
    }

    /**
     * Get a String representation of the component
     */
    public String toString() {
        return text;
    }

}