/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BToggleButton.java,v 1.8 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 *
 * Q's:
 */
public class BToggleButton extends BInput {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(BToggleButton.class.getName());

    //private vars
    protected boolean selected = false;

    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BToggleButton() {}
    
    /**
     * Public constructor which creates the component and
     * binds it to a specific view
     *
     * @param view the View the component should be bound to
     */
    public BToggleButton(View iview) {
        this(null, null, null, false, iview, null);
    }
    
    /**
     * Public constructor which creates the component and
     * sets the initial data. The component is also
     * bound to the specified view.
     *
     * <p>Null values may be passed in for any parameters, 
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually 
     * rendering the component
     *
     * @param type BInput.RADIO, BInput.CHECKBOX, or null (indicating 
     *         don't render this attribute)
     * @param name the name of the button, or null (indicating 
     *         don't render this attribute)
     * @param value the value of the button, or null (indicating 
     *         don't render this attribute)
     * @param selected true if the button should be selected
     * @param view the View the component should be bound to
     * @param dvc the default ViewContext (opt--its presence allows the 
     *         component to be rendered as markup in toString())
     */
    public BToggleButton(String itype, String iname, String ivalue, boolean iselected, View iview, ViewContext idvc) {
        if (idvc!=null) setDefaultViewContext(idvc);
        if (itype!=null) setType(itype);
        if (iname!=null) setName(iname);
        if (ivalue!=null) setValue(ivalue);
        setSelected(iselected);
        if (iview!=null) setView(iview);
    }
    



    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BToggleButton.class, HTMLElement.class);
        installRendererFactory(rfHTML, BToggleButton.class, HTMLDocument.class);

    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLToggleRenderer();}
    }




    //--------------- BToggleButton ------------------------------
    /**
     * Set the input type (RADIO, or CHECKBOX). If this value remains null, 
     * the type will default to whatever is specified in the underlying 
     * markup. If you set this value, then the type will be overridden 
     * in all the views associated with this component.
     *
     * @param type an string value representing the size.
     */
    public void setType(String itype) {
        itype = itype.toLowerCase();
        if (itype!=null && !itype.equals(RADIO) && !itype.equals(CHECKBOX)) {
            itype = CHECKBOX;
        }
        super.setType(itype);
    }
    
    /**
     * Specify whether or not the button is selected.
     *
     * @param selected true if the button is selected
     */
    public void setSelected(boolean iselected) {
        selected = iselected;
        invalidate();
    }
    
    /**
     * Return true if the button is selected.
     *
     * @return true if the button is selected.
     */
    public boolean isSelected() {
        return selected;
    }

    /**
     * Render a specific view for the component. 
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View is not supported
     * @param list a List of all the views for this component
     */
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {
        if (logger.isInfoEnabled()) logger.info(depth, "rendering comp:"+this.toRef()+" view:"+view);

        //actually render the view according to known interfaces
        try {
            Renderer r = getRenderer(view);
            r.renderComponent(this, view, vc);
            
        } catch (DOMException e) {
            logger.warn(depth+2, "DOM Error:", e);
            throw new DOMAccessException("Error rendering component in view:"+e, e);
        }
    }
}