/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultTableView.java,v 1.5 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * This class provides the default implementation for 
 * a TableView. It provides a View for components to 
 * render themselves in. Unlike other views, the TableView
 * provides convenience methods to access the header, body,
 * and footer elements of the view.
 */
public class DefaultTableView extends DefaultView implements TableView {

    protected static BCategory logger = BCategory.getBInstance(DefaultTableView.class.getName());

    protected Element header = null;
    protected Element body = null;
    protected Element footer = null;

    /**
     * Public noargs constructor. This creates a view which is not bound 
     * to any particular node. You must bind the view to a node before you 
     * can actually use it for anything
     */
    public DefaultTableView() {
        this(null);
    }

    /**
     * Create a view and bind it to a node.
     *
     * @param node the node the view should be bound to
     */
    public DefaultTableView(Node node) {
        if (node!=null) setNode(node);
    }

    /**
     * Get the header element from the particular Node that
     * backs this view.
     *
     * @returns the header element (may be null if there is 
     *      no recognized header)
     */
    public Element getHeaderElement() {
        return header;
    }
    
    /**
     * Get the body element from the particular Node that
     * backs this view. There will always be a body element.
     * It will either correspond to the <tbody> element or
     * to the entire table (ie. the whole table is body)
     *
     * @returns the body element
     */
    public Element getBodyElement() {
        return (body!=null ? body : (Element) node);
    }
    
    /**
     * Get the footer element from the particular Node that
     * backs this view.
     *
     * @returns the footer element (may be null if there is 
     *      no recognized footer)
     */
    public Element getFooterElement() {
        return footer;
    }

    /**
     * Here we are going to look for custom header, footer, and
     * body elements
     */
    protected void customSearchForTemplates(Node curnode) {
        //if we already have the header, body, and footer just return
        if (header!=null && body!=null && footer!=null) return;

        //make sure its an element
        if (!(curnode instanceof Element)) return;
        Element el = (Element) curnode;
        if (logger.isDebugEnabled()) {
            logger.debug(1, "node [tag="+el.getTagName()+" id="+DOMUtil.getID(el)+"] implements the following interfaces:");
            Iterator it = Classes.getAllInterfaces(curnode).iterator();
            while (it.hasNext()) {
                Object o = it.next();
                logger.debug(2, o.toString());
            }
        }
                
        //see if its an instance of HTMLTableElement
        if (curnode instanceof HTMLTableSectionElement) {
        
            String tagName = el.getTagName().toUpperCase();
            if (tagName.equals("THEAD")) {
                if (header==null) header = el;
            } else if (tagName.equals("TBODY")) {
                if (body==null) body = el;
            } else if (tagName.equals("TFOOT")) {
                if (footer==null) footer = el;
            }        
        }
    }


    public String toString() {
        return "TableView:"+getName()+" (bound to Node:"+DOMUtil.getID(node)+")";
    }
    
}