/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLActionRenderer.java,v 1.21 2001/10/03 21:25:20 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;

/**
 * This class handles the default rendering of actions into an HTML view.
 */
public class HTMLActionRenderer extends HTMLComponentRenderer {

    protected static BCategory logger = BCategory.getBInstance(HTMLActionRenderer.class.getName());

    protected EnabledHelper eh = new EnabledHelper();
        
    /**
     * The purpose of this method is to create a default Node to be used when
     * the component is not bound to any specific view. In the case of BAction,
     * it will attempt to use the default template node if possible (ie. if
     * it is an <form>, <input>, <button>, or <select> element); if anything else, 
     * it will defer to the super() method
     */
    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException {
        //ask the renderer to create the default Node
        Node templateNode = vc.getTemplateNode();
        Node defaultNode = null;
        if (templateNode instanceof HTMLAnchorElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLFormElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLInputElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLButtonElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLSelectElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else {
            return super.createDefaultNode(doc, vc);
        }
        if (logger.isInfoEnabled()) logger.info("Creating default node:"+defaultNode);
        return defaultNode;
    }

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a text component
        if (!(comp instanceof BAction)) throw new NoSuitableRendererException("This renderer can only render BAction components");
        BAction actionComp = (BAction) comp;
        Node node = view.getNode();
        ViewCapabilities vcap = vc.getViewCapabilities();

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //..HTMLAnchorElement - set the "href" attribute
        if (node instanceof HTMLAnchorElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLAnchorElement interface...");
            
            //set the href
            ((HTMLAnchorElement) node).setHref(actionComp.getAction(vc));

            //if we need to disable the back button, and the client supports 
            //Javascript, we can do so by setting the onclick parameter
            if (actionComp.getDisableBackButton() && 
                vcap.getScriptingType() instanceof ScriptingType.JavaScript1x &&
                node instanceof Element) {
                    StringBuffer sb = new StringBuffer(200);
                    sb.append("location.replace(this.href); ");
                    sb.append("return false; ");
                    ((Element) node).setAttribute("onclick", sb.toString());                            
            }

            //make sure we prevent caching (we do this for NN in particular)
            actionComp.setPreventCaching(true);
            
        //..HTMLFormElement - set the "action" attribute
        } else if (node instanceof HTMLFormElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLFormElement interface...");
            HTMLFormElement fel = (HTMLFormElement) node;

            //if we need to disable the back button, and the client supports 
            //Javascript, we can do so by setting the action
            if (actionComp.getDisableBackButton() && 
                vcap.getScriptingType() instanceof ScriptingType.JavaScript1x) {

                //redirect the form to param gateway servlet
//csc_100301.2_start
//                String target = actionComp.getAction(vc);
//                fel.setAction(ParamGateway.PARAM_TARGET+target+ParamGateway.PARAM_EXT);
                LightweightURL lu = new LightweightURL(actionComp.getAction(vc));
                fel.setAction(ParamGateway.PARAM_TARGET+lu.getBaseStr()+ParamGateway.PARAM_EXT+lu.getParamStr());
//csc_100301.2_end
                
                //set the onsubmit attribute
                StringBuffer sb = new StringBuffer(200);
                sb.append("submitAndReplace(document."+NamingHelper.getName(fel)+"); "); //csc_100201.2
                sb.append("return false; ");
                fel.setAttribute("onsubmit",sb.toString());
                
                //make sure we include the necessary script library
                String scr = ResourceGateway.EXT_RESOURCE_ID+"/org/enhydra/barracuda/core/scripts/jsrsClientServerHTTPLib.js";
                BScript scrComp = new BScript(scr, new DefaultView(node));
                actionComp.addStepChild(scrComp, true);
                
                //make sure we prevent caching (we do this for NN in particular)
                actionComp.setPreventCaching(true);

            //else just set the form action
            } else {
                fel.setAction(actionComp.getAction(vc));
            }

        //..HTMLInputElement - 
        } else if (node instanceof HTMLInputElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLInputElement interface...");
            HTMLInputElement iel = (HTMLInputElement) node;
            HTMLFormElement fel = iel.getForm();
//TODO:need to handle elements that are not part of a form (ie. fel==null)

            //if the client supports Javascript...
//            if (false) {
            if (vcap.getScriptingType() instanceof ScriptingType.JavaScript1x) {
                //build the action String
                StringBuffer sb = new StringBuffer(200);
                StringBuffer sbAction = new StringBuffer(100);
                boolean disable = actionComp.getDisableBackButton();

                //csc_100201.1
                //make sure we disable the input element, thus preventing duplicate
                //submits by double clicking (note that this won't work in NN4)
                sb.append("this.disabled=true; ");
            
                //adjust the documents action (use Base64 encoding on the target
                //so that if the action is a URL with params it won't muck up 
                //the new URL we are forming)
//csc_100301.2_start
//                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
//                String target = actionComp.getAction(vc);
//                sbAction.append(target);
//                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
                LightweightURL lu = new LightweightURL(actionComp.getAction(vc));
                sbAction.append(lu.getBaseStr());
                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
                sbAction.append(lu.getParamStr());
//csc_100301.2_end
                sb.append("document."+NamingHelper.getName(fel)+".action='"+sbAction.toString()+"';");  //csc_100201.2

                //adjust the onclick value to invoke submit
                if (disable) sb.append("submitAndReplace(document."+NamingHelper.getName(fel)+"); return false;");  //csc_100201.2
                else sb.append("document."+NamingHelper.getName(fel)+".submit(); return false;");  //csc_100201.2
                iel.setAttribute("onclick", sb.toString());
            
                //make sure we include the necessary script library
                if (disable) {
                    String scr = ResourceGateway.EXT_RESOURCE_ID+"/org/enhydra/barracuda/core/scripts/jsrsClientServerHTTPLib.js";
                    BScript scrComp = new BScript(scr, new DefaultView(node));
                    actionComp.addStepChild(scrComp, true);
                }
                
                //make sure we prevent caching (we do this for NN in particular)
                actionComp.setPreventCaching(true);
                                
            //else do it the manual way    
            } else {
                //make sure the button type is submit
                if (!iel.getType().toLowerCase().equals("submit")) {
                    String errmsg = "Cannot render Select action listener; input type is not 'submit': "+node;
                    logger.warn(errmsg);
                    throw new NoSuitableRendererException(errmsg);
                }
            
                //rename the button appropriately
                iel.setName(FormGateway.FORM_TARGET+actionComp.getAction(vc));            
                
                //redirect the form to event forwarding servlet
                fel.setAction(NamingHelper.getName(fel)+FormGateway.FORM_EXT);  //csc_100201.2
            }

        //..HTMLButtonElement - 
        } else if (node instanceof HTMLButtonElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLButtonElement interface...");
            HTMLButtonElement bel = (HTMLButtonElement) node;
            HTMLFormElement fel = bel.getForm();
//TODO:need to handle elements that are not part of a form (ie. fel==null)
            
            //if the client supports Javascript...
            if (vcap.getScriptingType() instanceof ScriptingType.JavaScript1x) {
                //build the action String
                StringBuffer sb = new StringBuffer(200);
                StringBuffer sbAction = new StringBuffer(100);
                boolean disable = actionComp.getDisableBackButton();

                //csc_100201.1
                //make sure we disable the input element, thus preventing duplicate
                //submits by double clicking (note that this won't work in NN4)
                sb.append("this.disabled=true; ");
            
                //adjust the documents action (use Base64 encoding on the target
                //so that if the action is a URL with params it won't muck up 
                //the new URL we are forming)
//csc_100301.2_start
//                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
//                String target = actionComp.getAction(vc);
//                sbAction.append(target);
//                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
                LightweightURL lu = new LightweightURL(actionComp.getAction(vc));
                sbAction.append(lu.getBaseStr());
                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
                sbAction.append(lu.getParamStr());
//csc_100301.2_end
                sb.append("document."+NamingHelper.getName(fel)+".action='"+sbAction.toString()+"';");  //csc_100201.2
            
                //adjust the onclick value to invoke submit
                if (disable) sb.append("submitAndReplace(document."+NamingHelper.getName(fel)+"); return false;");  //csc_100201.2
                else sb.append("document."+NamingHelper.getName(fel)+".submit(); return false;");  //csc_100201.2
                bel.setAttribute("onclick", sb.toString());
            
                //make sure we include the necessary script library
                if (disable) {
                    String scr = ResourceGateway.EXT_RESOURCE_ID+"/org/enhydra/barracuda/core/scripts/jsrsClientServerHTTPLib.js";
                    BScript scrComp = new BScript(scr, new DefaultView(node));
                    actionComp.addStepChild(scrComp, true);
                }
                
                //make sure we prevent caching (we do this for NN in particular)
                actionComp.setPreventCaching(true);

            //else do it the manual way    
            } else {
                //make sure the button type is submit
                if (!bel.getType().toLowerCase().equals("submit")) {
                    String errmsg = "Cannot render Select action listener; button type is not 'submit': "+node;
                    logger.warn(errmsg);
                    throw new NoSuitableRendererException(errmsg);
                }
            
                //rename the button appropriately
                bel.setName(FormGateway.FORM_TARGET+actionComp.getAction(vc));            
                
                //redirect the form to event forwarding servlet
                fel.setAction(NamingHelper.getName(fel)+FormGateway.FORM_EXT);  //csc_100201.2
            }
            
        //..HTMLSelectElement - 
        } else if (node instanceof HTMLSelectElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLSelectElement interface...");
            HTMLSelectElement sel = (HTMLSelectElement) node;
            HTMLFormElement fel = sel.getForm();
//TODO:need to handle elements that are not part of a form (ie. fel==null)
            
            //if the client supports Javascript...
            if (vcap.getScriptingType() instanceof ScriptingType.JavaScript1x) {
                //build the action String
                StringBuffer sb = new StringBuffer(200);
                StringBuffer sbAction = new StringBuffer(100);
                boolean disable = actionComp.getDisableBackButton();

                //csc_100201.1
                //make sure we disable the input element, thus preventing duplicate
                //submits by double clicking (note that this won't work in NN4)
//csc_100301.1 - we can't do this for selects, since it causes the value not to be 
//submitted along with the form since it is disabled.
//                sb.append("this.disabled=true;");
            
                //adjust the documents action (use Base64 encoding on the target
                //so that if the action is a URL with params it won't muck up 
                //the new URL we are forming)
//csc_100301.2_start
//                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
//                String target = actionComp.getAction(vc);
//                sbAction.append(target);
//                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
                LightweightURL lu = new LightweightURL(actionComp.getAction(vc));
                sbAction.append(lu.getBaseStr());
                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
                sbAction.append(lu.getParamStr());
//csc_100301.2_end
                sb.append("document."+NamingHelper.getName(fel)+".action='"+sbAction.toString()+"';");  //csc_100201.2
            
                //adjust the onclick value to invoke submit
                if (disable) sb.append("submitAndReplace(document."+NamingHelper.getName(fel)+"); return false;");  //csc_100201.2
                else sb.append("document."+NamingHelper.getName(fel)+".submit(); return false;");  //csc_100201.2
                sel.setAttribute("onchange", sb.toString());
            
                //make sure we include the necessary script library
                if (disable) {
                    String scr = ResourceGateway.EXT_RESOURCE_ID+"/org/enhydra/barracuda/core/scripts/jsrsClientServerHTTPLib.js";
                    BScript scrComp = new BScript(scr, new DefaultView(node));
                    actionComp.addStepChild(scrComp, true);
                }
                
                //make sure we prevent caching (we do this for NN in particular)
                actionComp.setPreventCaching(true);
                
            //else throw an error
            } else {
                String errmsg = "Cannot render Select action listener; client does not support Javascript: "+node;
                logger.warn(errmsg);
                throw new NoSuitableRendererException(errmsg);
            }

        } else {
            String errmsg = "Node does not implement HTMLAnchorElement, HTMLFormElement, HTMLInputElement or HTMLButtonElement and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
        
        //finally, make sure we reflect the components enabled/disabled status
        eh.setEnabled(node, comp.isEnabled());
    }
    
}