/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLLinkRenderer.java,v 1.10 2001/09/27 21:16:43 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * This class handles the default rendering of a link into an HTML view.
 */
public class HTMLLinkRenderer extends HTMLActionRenderer {

    protected static BCategory logger = BCategory.getBInstance(HTMLLinkRenderer.class.getName());

    /**
     * The purpose of this method is to create a default Node to be used when
     * the component is not bound to any specific view. In the case of BLink,
     * it will attempt to use the default template node if possible (ie. if
     * it is an <a>, <button>, or <input> element); if anything else, it will
     * create an <a> node from scratch and append an empty text node to it
     */
    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException {
        //ask the renderer to create the default Node
        Node templateNode = vc.getTemplateNode();
        Node defaultNode = null;
        if (templateNode instanceof HTMLAnchorElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLButtonElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLInputElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else {
            Element linkEl = (HTMLAnchorElement) doc.createElement("A");
            linkEl.appendChild(doc.createTextNode(""));
            defaultNode = linkEl;
        }
        if (logger.isInfoEnabled()) logger.info("Creating default node:"+defaultNode);
        return defaultNode;
    }

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a text component
        if (!(comp instanceof BLink)) throw new NoSuitableRendererException("This renderer can only render BLink components; comp is of type:"+comp.getClass().getName());
        BLink linkComp = (BLink) comp;
        Node node = view.getNode();

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //..HTMLAnchorElement
        if (node instanceof HTMLAnchorElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLAnchorElement interface...");

            //set the "target" attribute, if it exists
            String target = linkComp.getTarget();
            if (target!=null) ((HTMLAnchorElement) node).setTarget(target);

            //set the text value (if it exists) by delegating to a text component
            String text = linkComp.getText();
            if (text!=null) {
                BText textComp = new BText(text, new DefaultView(node));
                textComp.setAllowMarkupInText(linkComp.allowMarkupInText());    //csc_092701.1
                linkComp.addStepChild(textComp, true);
            }
            
        //..HTMLButtonElement
        } else if (node instanceof HTMLButtonElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLButtonElement interface...");

            //set the text value (if it exists) by delegating to a text component
            String text = linkComp.getText();
            if (text!=null) {
                BText textComp = new BText(text, new DefaultView(node));
                textComp.setAllowMarkupInText(linkComp.allowMarkupInText());    //csc_092701.1
                linkComp.addStepChild(textComp, true);
            }

        //..HTMLInputElement
        } else if (node instanceof HTMLInputElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLInputElement interface...");

            //set the text value (if it exists) by delegating to a text component
            String text = linkComp.getText();
            if (text!=null) {
                BText textComp = new BText(text, new DefaultView(node));
                textComp.setAllowMarkupInText(false);   ////csc_092701.1 - input elements can't contain markup, so don't bother allowing it
                linkComp.addStepChild(textComp, true);
            }

        } else {
            String errmsg = "Node does not implement HTMLAnchorElement, HTMLButtonElement, or HTMLInputElement and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
    }
    
}