/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLSelectRenderer.java,v 1.7 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * This class handles the default rendering of a list in an HTML view.
 */
public class HTMLSelectRenderer extends HTMLListRenderer {

    protected static BCategory logger = BCategory.getBInstance(HTMLSelectRenderer.class.getName());
    
    protected BSelect bscomp = null;
    protected ListModel lm = null;
    protected ListSelectionModel lsm = null;
    
    /**
     * The purpose of this method is to create a default Node to be used when
     * the component is not bound to any specific view. 
     */
    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException {
        //ask the renderer to create the default Node
        Element selectEl = (HTMLSelectElement) doc.createElement("SELECT");
        Node defaultNode = selectEl;
        if (logger.isInfoEnabled()) logger.info("Creating default node:"+defaultNode);
        return defaultNode;
    }

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a list component
        if (!(comp instanceof BSelect)) throw new NoSuitableRendererException("This renderer can only render BSelect components; comp is of type:"+comp.getClass().getName());
        bscomp = (BSelect) comp;
        lm = bscomp.getModel();
        lsm = bscomp.getSelectionModel();
        Node node = view.getNode();

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //..HTMLSelectElement - set the "multiples" and "size" attributes,
        //        and then mark items selected/unselected. Finally, set the
        //        "value" attribute for each option element.
        if (node instanceof HTMLSelectElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLSelectElement interface...");
            HTMLSelectElement el = (HTMLSelectElement) node;

            //set the size (if specified)
            if (bscomp.getViewSize()!=null) {
                el.setSize(bscomp.getViewSize().intValue());
            }

            //set allow multiples (get this from the selection model)
            el.setMultiple(lsm.getSelectionMode()!=ListSelectionModel.SINGLE_SELECTION);
            
            //now iterate through all the children (this list will 
            //already have been populated by the parent class renderer 
            //from the model)                
            HTMLCollection options = el.getOptions();
            for (int i=0, max=options.getLength(); i<max; i++) {
                HTMLOptionElement opt = (HTMLOptionElement) options.item(i);
                
                //mark them selected/unselected by looking at the selection model
                opt.setSelected(lsm.isSelectedIndex(i));
                
                //set the value: if the item in the model implements ItemMap,
                //use that value; otherwise, just set it equal to the text
//                Object item = lm.getItemAt(i, vc);
                Object item = lm.getItemAt(i);
                if (item==null) continue;
                if (item instanceof ItemMap) {
                    opt.setValue(((ItemMap) item).getKey().toString());
                } else {
                    opt.setValue(item.toString());
                }
            }
            
        } else {
            String errmsg = "Node does not implement HTMLSelectElement and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
    }
}