/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLTableRenderer.java,v 1.10 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * This class handles the default rendering of a table in an HTML view.
 */
public class HTMLTableRenderer extends HTMLComponentRenderer {

    protected static BCategory logger = BCategory.getBInstance(HTMLTableRenderer.class.getName());

    protected EnabledHelper eh = new EnabledHelper();
    
    protected BTable btable = null;
    
    /**
     * The purpose of this method is for a renderer to provide 
     * a default node (if none exists). This component currently
     * does not provide a default, so it throws an UnsupportedFormatException 
     * instead.
     */
    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException {
        throw new UnsupportedFormatException("Cannot create default node");
    }
    
    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a text component
        if (!(comp instanceof BTable)) throw new NoSuitableRendererException("This renderer can only render BTable components; comp is of type:"+comp.getClass().getName());
        btable = (BTable) comp;
        TableModel headerModel = btable.getHeaderModel();
        TableModel model = btable.getModel();
        TableModel footerModel = btable.getFooterModel();
        Node node = view.getNode();

        //make sure the View implements TableView
        if (!(view instanceof TableView)) throw new InvalidViewException ("Component is bound to an unsupported View:"+view);
        TableView tview = (TableView) view;

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);


        //HTMLElement Interface
        //---------------------
        //Supported Elements:
        //..HTMLTableElement
        //
        //Unsupported Elements:
        //..everything else
        if (node instanceof HTMLElement) {

            if (node instanceof HTMLTableElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering table component...");

                //get the nodes for the header, body, and footer sections
                Element bodyNode = tview.getBodyElement();
                Element headerNode = tview.getHeaderElement();
                Element footerNode = tview.getFooterElement();

                //now create a list component and bind it to the data section node.
                //(be sure to add the comp as a temporary child--by doing this, we 
                //ensure that the child will get rendered as well. After rendering, 
                //we'll remove the child from the hierarchy, so as to return to our 
                //original state)
                if (logger.isDebugEnabled()) logger.debug(1, "Creating table component for body...");
                DefaultListModel rowBodyModel = new DefaultListModel();
                BList rowBodyList = new BList(rowBodyModel, new DefaultView(bodyNode));
//                if (tempChildren==null) tempChildren = new ArrayList();
//                tempChildren.add(rowBodyList);
//                this.add(rowBodyList);
                btable.addStepChild(rowBodyList);

                //create a model, view, and list for the header (default it
                //to be the same objects as for the body)
                DefaultListModel rowHeaderModel = rowBodyModel;
                BList rowHeaderList = rowBodyList;
                if (headerNode!=null && headerModel!=null) {
                    if (logger.isDebugEnabled()) logger.debug(1, "Creating table component for header...");
                    rowHeaderModel = new DefaultListModel();
                    rowHeaderList = new BList(rowHeaderModel, new DefaultView(headerNode));
//                    tempChildren.add(rowHeaderList);
//                    this.add(rowHeaderList);
                    btable.addStepChild(rowHeaderList);
                }

                //do the same thing for the footer model
                DefaultListModel rowFooterModel = rowBodyModel;
                BList rowFooterList = rowBodyList;
                if (footerNode!=null && footerModel!=null) {
                    if (logger.isDebugEnabled()) logger.debug(1, "Creating table component for footer...");
                    rowFooterModel = new DefaultListModel();
                    rowFooterList = new BList(rowFooterModel, new DefaultView(footerNode));
//                    tempChildren.add(rowFooterList);
//                    this.add(rowFooterList);
                    btable.addStepChild(rowFooterList);
                }

                //now populate the rowModel
                if (headerModel!=null) {
                    if (logger.isDebugEnabled()) logger.debug(1, "Rendering header...");
                    populateTableFromModel(node, tview, vc, headerModel, rowHeaderModel, rowHeaderList);
                }
                if (model!=null) {
                    if (logger.isDebugEnabled()) logger.debug(1, "Rendering body...");
                    populateTableFromModel(node, tview, vc, model, rowBodyModel, rowBodyList);
                }
                if (footerModel!=null) {
                    if (logger.isDebugEnabled()) logger.debug(1, "Rendering footer...");
                    populateTableFromModel(node, tview, vc, footerModel, rowFooterModel, rowFooterList);
                }
            } else {
                if (logger.isDebugEnabled()) logger.debug(1, "Interface "+node.getClass().getName()+" not a supported View for a BTable component...");
                throw new InvalidNodeException ("View is bound to an unsupported Node:"+node);
            }
                
        } else {
            String errmsg = "Node does not implement HTMLElement and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
        
        //finally, make sure we reflect the components enabled/disabled status
        eh.setEnabled(node, comp.isEnabled());
    }
    
    protected void populateTableFromModel(Node node, TableView view, ViewContext vc, TableModel curModel, DefaultListModel rowModel, BList rowList) throws RenderException {

        //give the model a chance to initialize
        curModel.setViewContext(vc);
        curModel.resetModel();
//        vc.setTemplateNode(node);
        vc.putState(ViewContext.TEMPLATE_NODE, node);
                
        //now process items in the model based on model type
        ElementFactory ef = view.getElementFactory();
        for (int i=0,max=curModel.getRowCount(); i<max; i++) {

            //create a generic row element and add it to the rowModel
            Element rowNode = node.getOwnerDocument().createElement("TR");
            rowModel.add(rowNode);

            //create a list component for the cols in the row and bind it
            //to the rowNode
            DefaultListModel colModel = new DefaultListModel();
            DefaultView colView = new DefaultView(rowNode); 
            BList colList = new BList(colModel, colView);
            rowList.addChild(colList);
            
            //now populate the colModel
            for (int j=0,jmax=curModel.getColumnCount(); j<jmax; j++) {
                Object item = curModel.getItemAt(i, j);
                if (item!=null) colModel.add(item);
            }
        }
        
//        vc.setTemplateNode(null);
        vc.putState(ViewContext.TEMPLATE_NODE, null);
        curModel.setViewContext(null);
    }
}