/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: ApplicationGateway.java,v 1.23 2001/10/05 14:43:41 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.event.events.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.view.*;

/**
 * <p>The application gateway is responsible for a number of things.
 *
 * <ol>
 *   <li>It acts as a gateway servlet for all event handlers in this
 *         application.</li>
 *
 *   <li>It defines and instantiates a number of entities needed to
 *       dispatch events (EventBroker, EventPool, DispatcherFactory, etc)</li>
 *
 *      <li>It defines which event extension we are using</li>
 *
 *      <li>It registers all EventGateways, and any local Event interests</li>
 *
 *      <li>It performs the initial mapping of HTTPRequests to Events for this
 *       particular domain </li>
 * </ol>
 *
 * <p>Consequently, if you want to use the Barracuda event model, this
 * is the class that really kicks it all off. You must either extend this
 * class OR use the DefaultApplicationAssembler in order to specify that
 * this servlet handles all event requests for an application. This will
 * allow the system to convert requests to events and dispatch them through
 * the EventBroker to any listeners within EventGateways that have registered
 * interest with the broker.
 *
 * <p>This class should be the first servlet loaded in your web.xml file.
 *
 * <p>For an example of how to do this, look at
 * <strong>org.enhydra.barracuda.examples.ex1.SampleApplicationGateway</strong>
 *
 * <p>You might also want to look at the <a href="sm_barracuda_event.gif">UML Class
 * diagram of the Event model classes.</a>
 */
public class ApplicationGateway extends HttpServlet implements EventGateway {

    //public vars...eventually, these should probably be final
    public static boolean USE_EVENT_POOLING = true;            //use event pooling?
    public static boolean SHOW_APP_STRUCTURE = false;
    protected static BCategory logger = BCategory.getBInstance(ApplicationGateway.class.getName());

    public static final String TARGET_EVENT_NAME = "DefaultBaseEventListener.TargetEventName";            //(String)
    public static final String EXTERNAL_CONTEXT_OBJ_NAME = "External.Context.Obj";            //(String)

    private static final String DEBUG_VALUE = "DebugValue";
    private static final String APPLICATION_ASSEMBLER = "ApplicationAssembler";
    private static final String ASSEMBLY_DESCRIPTOR = "AssemblyDescriptor";
    private static final String SCRIPTING_DESCRIPTOR = "ScriptingDescriptor";
    private static final String SAX_PARSER = "SAXParser";

    /**
     * @clientCardinality 1
     */
    private EventBroker masterEventBroker = null;

    /**
     * @clientCardinality 1
     */
    private EventPool masterEventPool = null;

    /**
     * @supplierCardinality 1
     */
    protected DefaultEventGateway defaultEventGateway = new DefaultEventGateway();

    //private vars
    protected List gateways = null;
    protected ServletConfig config = null;

    /**
     * Public noargs constructor
     */
    public ApplicationGateway() {
        logger.info("Instantiating "+this);
    }


    //--------------- User Configurable Methods ------------------
    /**
     * <p>Perform any local initialization (this is where you should
     * add any other known EventGateways)
     *
     * @param config the ServletConfig object used to configure this servlet
     */
    public void initializeLocal(ServletConfig config) {
    }
    
    /**
     * <p>Perform any local cleanup (this is where you should
     * remove any known EventGateways)
     */
    public void destroyLocal() {
    }

    /**
     * <p>Provide an instance of the specific EventBroker we want to use.
     * Override this method if you'd like to use something other than
     * the DefaultEventBroker.
     *
     * @return a new instance of the EventBroker
     */
    public EventBroker getNewEventBrokerInstance() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultEventBroker");
        return new DefaultEventBroker(getDispatcherFactory(), getEventExtension());
    }
    
    /**
     * <p>Provide an instance of the specific EventPool we want to use.
     * Override this method if you'd like to use something other than
     * the DefaultEventPool.
     *
     * @return a new instance of the EventPool.
     */
    public EventPool getNewEventPoolInstance() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultEventPool");
        return new DefaultEventPool();
    }
    
    //csc_100501.1 - added at recommendation of Diez B. Roggisch 
    //[d.roggisch@artnology.com]
    /**
     * <p>Provide an instance of the specific DispatchQueue we want to use.
     * Override this method if you'd like to use something other than
     * the DefaultDispatchQueue.
     *
     * @return a new instance of the DispatchQueue.
     */
    public DispatchQueue getNewDispatchQueueInstance() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultDispatchQueue");
        return new DefaultDispatchQueue(true);
    }
    
    /**
     * <p>Provide an instance of the specific EventDispatcher we want to use.
     * Override this method if you'd like to use something other than
     * DefaultEventBroker.
     *
     * @return a new instance of the DispatcherFactory
     */
    public DispatcherFactory getDispatcherFactory() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultDispatcherFactory");
        return new DefaultDispatcherFactory();
    }
    
    /**
     * <p>Indicate which event extension we are handling. By default the gateway
     * handles extensions of <strong>.event</strong>. If you wish to handle
     * a different function you should override this method to return the
     * value defined in the web.xml file.
     *
     * @return a string defining the event extension handled by this servlet
     */
    public String getEventExtension() {
        return ".event";
    }
    
    /**
     * <p>Handle the default HttpRequest. It will probably be rare for
     * developers to override this.
     *
     * <p>Basically, this method receives a request and attempts to map it
     * to a valid HttpRequestEvent. If the event is invalid, or if the
     * event is NOT an instance of HttpRequestEvent, then we simply create
     * a new instance of HttpRequestEvent and dispatch that instead. This
     * is a very important feature, because it allows us to define events
     * which are not accessible to the outside world -- the gateway will
     * only dispatch HttpRequest events, so you define your publically
     * accessible API by creating an event hierarchy that extends from
     * the HttpRequestEvent object.
     *
     * <p>Once we have a valid event, we dispatch it. In this case (because this
     * is an Http gateway) we must have a response, so if we catch an
     * UnhandledEventException, we will generate a default error message and
     * return.
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefault (HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefaultExt(req, resp, null);
    }

    /**
     * <p>Handle the default HttpRequest with the ability to add the provided
     * object into the context. It will probably be rare for
     * developers to override this.
     *
     * <p>Basically, this method receives a request and attempts to map it
     * to a valid HttpRequestEvent. If the event is invalid, or if the
     * event is NOT an instance of HttpRequestEvent, then we simply create
     * a new instance of HttpRequestEvent and dispatch that instead. This
     * is a very important feature, because it allows us to define events
     * which are not accessible to the outside world -- the gateway will
     * only dispatch HttpRequest events, so you define your publically
     * accessible API by creating an event hierarchy that extends from
     * the HttpRequestEvent object.
     *
     * <p>Once we have a valid event, we dispatch it. In this case (because this
     * is an Http gateway) we must have a response, so if we catch an
     * UnhandledEventException, we will generate a default error message and
     * return.
     *
     * @param req the servlet request
     * @param req the servlet response
     * @param contextObj object to put into the context
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefaultExt (HttpServletRequest req, 
                                  HttpServletResponse resp, 
                                  Object externalContextObj) throws ServletException, IOException {
        long bmillis = 0;
        long smillis = 0;
        long emillis = 0;
        if (logger.isInfoEnabled()) bmillis = System.currentTimeMillis();
        if (logger.isInfoEnabled()) logger.info("Handling incoming HTTP request in "+this);

        BaseEvent event = null;
        
        //start by getting the event broker and pool. We get local copies so
        //that if these constant vaules change while we're mid-method, it's not a problem
        EventBroker eventBroker = getEventBroker();
        EventPool eventPool = getEventPool();
        boolean useEventPool = USE_EVENT_POOLING;
        boolean eventCameFromPool = false;
        boolean allowEventAliasing = true;    //just do it always
        String target = null;
        
        try {
            //figure out the name of the event handler being requested
            target = req.getServletPath();
            if (logger.isDebugEnabled()) logger.debug(1, "Incoming URI:"+req.getRequestURI());
            if (logger.isDebugEnabled()) logger.debug(1, "ServletPath:"+target);

            // cater for Enhydra's different behaviour if in a PO
            if (null == target || target.length() == 0) {
                target = req.getRequestURI();
                if (logger.isDebugEnabled()) logger.debug(1, "Setting target to request URI: " + target);
            }

            //..trim any path info (based on '/')
            if (target!=null) {
                int spos = target.lastIndexOf("/");
                if (spos>-1) target = target.substring(spos+1);
            }
            //..trim any path info (based on '\')
            if (target!=null) {
                int spos = target.lastIndexOf("\\");
                if (spos>-1) target = target.substring(spos+1);
            }

            //now get the actual name of the event
            String ext = eventBroker.getEventExtension();
            String eventName = target;
            if (eventName.endsWith(ext)) {
                eventName = target.substring(0, target.length()-ext.length());
                if (logger.isDebugEnabled()) logger.debug(1, "Target Event:"+eventName);
            }

            //now get the actual event name            
            if (allowEventAliasing) try {
                String fullEventName = eventBroker.matchEventClass(eventName);
                if (fullEventName!=null) {
                    eventName = fullEventName;
                    if (logger.isDebugEnabled()) logger.debug(1, "Fully qualified Event:"+eventName);
                }
            } catch (InvalidClassException e) {}

            //now instantiate an Event (here's where it would make sense to
            //use an Event pool)
            if (logger.isDebugEnabled()) logger.debug(1, "Creating Event");
            try {
                //get the class name for the event
                Class cl = Class.forName(eventName);
                
                //get it from the pool if appropriate
                if (useEventPool) try {
                    if (logger.isDebugEnabled()) logger.debug(2, "Checking out event from event pool");
                    event = eventPool.checkoutEvent(cl);
                    eventCameFromPool = true;
                } catch (NoAvailableEventsException e) {
                    if (logger.isDebugEnabled()) logger.debug(2, "No available events in pool");
                }
                
                //if that didn't work (or we're not getting it from the
                //pool in the first place) create it manually
                if (event==null) {
                    if (logger.isDebugEnabled()) logger.debug(2, "Creating a new event from scratch");
                    event = (BaseEvent) cl.newInstance();    //this requires that events being dispatched have a noargs constructor
                }
                
            } catch (Exception e) {
                //if we can't map the request to an event, we will just
                //dispatch a generic HttpRequestEvent
                logger.error(1, "Class "+eventName+" not found...Creating a generic HttpRequestEvent");
                event = new HttpRequestEvent();
            }

            //make sure the incoming event implements HttpRequestEvent...if not,
            //consider it invalid (This is VERY important: it prevents people
            //from hacking the system by directly invoking events that were
            //not intended to be invoked directly from teh outside world)
            if (!(event instanceof HttpRequestEvent)) event = new HttpRequestEvent();

            //now set any special paramters
            event.setSource(this);
            event.setHandled(false);
            
            //now figure out if the event targets any specific listener IDs, and
            //if so add these ids to the event. This info will be used by the
            //dispatcher to send the event to the specified listeners, rather
            //doing a general dispatch to all parties interested in this class of
            //event.
            String[] ids = req.getParameterValues(BaseEvent.EVENT_ID);
            if (logger.isDebugEnabled()) logger.debug(1, "Looking for event listener ids:"+ids);
            if (ids!=null) {
                for (int i=0, max=ids.length; i<max; i++) {
                    String id = ids[i];                
                    if (logger.isDebugEnabled()) logger.debug(2, "Target EventID:"+id);
                    if (allowEventAliasing) try {
                        id = eventBroker.matchListenerID(ids[i]);
                        if (logger.isDebugEnabled()) logger.debug(2, "Fully qualified EventID:"+id);
                    } catch (InvalidClassException e) {}
                    event.addListenerID(id);                
                }                
            }
            
            //finally, dispatch it to the EventBroker
            if (logger.isDebugEnabled()) logger.debug(1, "Dispatching event:"+event+" to broker:"+eventBroker);
//csc_100501.1            DefaultDispatchQueue eventQueue = new DefaultDispatchQueue(true);
            DispatchQueue eventQueue = getNewDispatchQueueInstance();    //csc_100501.1
//            ViewCapabilities vc = new ViewCapabilities();    
            ViewCapabilities vc = new ViewCapabilities(req, resp);    
            HttpResponseEvent defaultResponseEvent = new HttpResponseEvent();
            defaultResponseEvent.setSource(event);
            eventQueue.addEvent(event);
            DefaultEventContext context = new DefaultEventContext(eventQueue, vc, config, req, resp, defaultResponseEvent);
            context.putState(TARGET_EVENT_NAME, eventName);
            if (null != externalContextObj) {
                context.putState(EXTERNAL_CONTEXT_OBJ_NAME, externalContextObj);
            }
            if (logger.isInfoEnabled()) smillis = System.currentTimeMillis();
            eventBroker.dispatchEvent(context);
            if (logger.isInfoEnabled()) emillis = System.currentTimeMillis();

            if (logger.isInfoEnabled()) logger.info(0, "Dispatching complete! (handled in "+(emillis-smillis)+" of "+(emillis-bmillis)+" millis)");

        } catch (ClientSideRedirectException re){
            //if we get a redirect exception, request the browser to redirect accordingly
//            resp.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
//            resp.setHeader("Location", re.getRedirectURL());
//            String url = re.getRedirectURL();                                //this method doesn't work if cookies are disabled
            String url = URLRewriter.encodeRedirectURL(req, resp, re.getRedirectURL());
            if (logger.isInfoEnabled()) logger.info(0, "ClientSideRedirectException...redirecting to "+url);
            resp.sendRedirect(url);    
/*            
//For debugging purposes...if you need to see the actual link being generated
            resp.setContentType("text/html");
            PrintWriter out = resp.getWriter();
            StringBuffer sb = new StringBuffer(400);
            sb.append("<html>".trim());            
            sb.append("  <head><title>Manual Redirect</title></head>".trim());            
            sb.append("  <body>".trim());            
            sb.append("    <p><a href=\""+url+"\">Click here to manually finish ClientSideRedirect</a></p>".trim());            
            sb.append("  </body>".trim());            
            sb.append("</html>".trim());            
            out.println(sb.toString());
*/            
        } catch (EventException e){
            //if we get an EventException, handle it
            handleEventException(e, req, resp);
        } finally {
            //make sure we ALWAYS release the event
            if (event!=null && eventCameFromPool) {
                if (logger.isDebugEnabled()) logger.debug(1, "Releasing event");
                eventPool.releaseEvent(event);
                if (logger.isDebugEnabled()) logger.debug(1, "Released!");
            }
        }
    }

    /**
     * <p>Handle an EventException. Basically, this is where we handle the
     * really bad, unexpected type of event exceptions. Generally, as you code,
     * if you want to interrupt the dispatch and fire a new event, you should
     * throw an InterruptDispatchException. Only throw EventExceptions in
     * truly exceptional circumstances.
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleEventException (EventException e, HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //first see what the base exception is...if it's a servlet exception
        //or an IOException, rethrow it...
        Exception rootException = NestableException.getRootException(e);
        if (rootException instanceof ServletException) throw (ServletException) rootException;
        if (rootException instanceof IOException) throw (IOException) rootException;

        //if we get an EventException, log it and print the base exception
        resp.setContentType("text/html");
        PrintWriter out = resp.getWriter();
        ExceptionUtil.logExceptionAsHTML(out, e, req);
        logger.warn("Unexpected event exception: ", e);
        if (rootException!=e) {
            logger.warn("Root Exception...", rootException);
        }
    }


    //-------------------- ApplicationGateway --------------------
    /**
     * <p>Specify event gateways. You can call this method with
     * as many different gateways as you like. When the Application
     * Gateway initializes, it will load them all. When the gateway
     * is destroyed, they will be deregistered.
     *
     * @param gateway an event gateway we'd like to have registered
     */
    public final void specifyEventGateways(EventGateway gateway) {
        if (gateways==null) gateways = new ArrayList();
        gateways.add(gateway);
    }
    
    /**
     * <p>Provide a reference to the event broker.
     *
     * @return a reference to the master event broker
     */
    public final EventBroker getEventBroker() {
        if (masterEventBroker==null) {
            masterEventBroker = getNewEventBrokerInstance();
            
            //this ensures that the App-gateway can successfully route
            //generic ActionEvents (if we ever add other "well-known"
            //generic events we should pre-register them here as well
            //so that they can be used without any special registration
            //process)
            try {masterEventBroker.addEventAlias(ActionEvent.class);}
            catch (InvalidClassException e) {}
        }
        return masterEventBroker;
    }

    /**
     * <p>Provide a reference to the event pool. May return
     * null if we're not using event pooling.
     *
     * @return a reference to the master event pool
     */
    public final EventPool getEventPool() {
        if (USE_EVENT_POOLING && masterEventPool==null) masterEventPool = getNewEventPoolInstance();
        if (!USE_EVENT_POOLING) masterEventPool = null;
        return masterEventPool;
    }
    




    
    

    
    //-------------------- EventGateway --------------------------
    /**
     * <p>Set the parent gateway. Null indicates its the root.
     * By definition, an ApplicationGateway is always the root.
     *
     * @param eg the parent event gateway for this gateway
     */
    public final void setParent(EventGateway eg) {
        defaultEventGateway.setParent(null);
    }
    
    /**
     * <p>Get the parent gateway. Returns null if it's the root.
     *
     * @return the parent event gateway
     */
    public final EventGateway getParent() {
        return defaultEventGateway.getParent();
    }
    
    /**
     * <p>Add an event gateway to this one
     *
     * @param eg the event gateway to be added
     */
    public final void add(EventGateway eg) {
        defaultEventGateway.add(eg);
        eg.setParent(this);    //do this so the parent points at the ApplicationGateway, not the DefaultEventGateway we are using to actually store the values
    }
    
    /**
     * <p>Remove an event gateway from this one
     *
     * @param eg the event gateway to be removed
     */
    public final void remove(EventGateway eg) {
        defaultEventGateway.remove(eg);
    }
    
    /**
     * Get a list of child gateways. The list returned is a copy of the
     * underlying child gateway list.
     *
     * @return a list of child gateways
     */
    public List getChildren() {
        return defaultEventGateway.getChildren();
    }
    
    /**
     * <p>Ask all interested parties to register with
     * the EventBroker
     *
     * @param eb the event broker to register with
     */
    public final void register(EventBroker eb) {
        defaultEventGateway.register(eb);
    }

    /**
     * <p>Ask all interested parties to de-register with
     * the EventBroker
     *
     * @param eb the event broker to de-register with
     */
    public final void deregister(EventBroker eb) {
        defaultEventGateway.deregister(eb);
    }
    








    //-------------------- HTTPServlet ---------------------------
    /**
     * <p>By default the GET request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        handleDefault(new HttpServletRequestWrapper(req), res);
    }
    
    /**
     * <p>By default the POST request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        handleDefault(new HttpServletRequestWrapper(req), res);
    }
    
    /**
     * <p>By default the OPTIONS request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doOptions(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        handleDefault(new HttpServletRequestWrapper(req), res);
    }
    
    /**
     * <p>By default the DELETE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doDelete(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        handleDefault(new HttpServletRequestWrapper(req), res);
    }
    
    /**
     * <p>By default the PUT request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPut(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        handleDefault(new HttpServletRequestWrapper(req), res);
    }
    
    /**
     * <p>By default the TRACE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doTrace(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        handleDefault(new HttpServletRequestWrapper(req), res);
    }






    //-------------------- Servlet -------------------------------
    /**
     * <p>Here's where we initialize the servlet.
     *
     * @param config the servlet config object this servlet is
     *        being initialized with
     */
    public void init(ServletConfig iconfig) throws ServletException {
        try {
            logger.info("initializing servlet");

            //save a reference to the config object
            config = iconfig;

            //first and foremost, assign any debug value constants
            logger.debug("checking for DebugValues");
            Enumeration enum = config.getInitParameterNames();
            while (enum.hasMoreElements()) {
                String param = enum.nextElement().toString();
                if (param.startsWith(DEBUG_VALUE)) {
                    String value = config.getInitParameter(param);
                    logger.debug(param+" = "+value);
                
                    //split it into parts
                    try  {
                        int pos1 = value.lastIndexOf(".");
                        int pos2 = value.indexOf("=", pos1);
                        String clname = value.substring(0,pos1);
                        String var = value.substring(pos1+1, pos2);
                        String val = value.substring(pos2+1);
                        logger.debug("Class:"+clname+" Variable:"+var+" Value:"+val);

                        //figure out what value we're trying to set
                        Class cl = Class.forName(clname);
                        Field field = cl.getDeclaredField(var);
                        Object origval = field.get(null);
                        Class fieldType = field.getType();
                        Object targetval = null;
                        //...integer
                        if (fieldType==Integer.class || fieldType==int.class) {    
                            targetval = new Integer(val);
                        //...boolean
                        } else if (fieldType==Boolean.class || fieldType==boolean.class) {
                            targetval = new Boolean(val);                        
                        //...float
                        } else if (fieldType==Float.class || fieldType==float.class) {    
                            targetval = new Float(val);
                        //...double
                        } else if (fieldType==Double.class || fieldType==double.class) {    
                            targetval = new Double(val);
                        //...long
                        } else if (fieldType==Long.class || fieldType==long.class) {    
                            targetval = new Long(val);
                        //...short
                        } else if (fieldType==Short.class || fieldType==short.class) {    
                            targetval = new Short(val);
                        //...String (def)
                        } else {
                            targetval = val;
                        }
                        
                        //actually try setting it
                        field.set(null, targetval);
                        Object newval = field.get(null);
                        logger.debug("set "+clname+"."+var+" from "+origval+" to "+newval);

                    } catch (Exception e) {                
                        logger.error("err "+e+" setting value:"+value);
                        continue;                    
                    }
                }
            }

            //start by getting a reference to the EventBroker. Also request the
            //EventPool, just to make sure they're both instantiated
            logger.debug("creating EventBroker");
            EventBroker eb = this.getEventBroker();
            getEventPool();

            //perform any automated assembly if necessary
            String assemblerName = config.getInitParameter(APPLICATION_ASSEMBLER);
            String descriptor = config.getInitParameter(ASSEMBLY_DESCRIPTOR);
            if (assemblerName!=null && descriptor!=null) try {
                logger.debug("performing automated assembly");
                String parser = config.getInitParameter(SAX_PARSER);
                ApplicationAssembler assembler = (ApplicationAssembler) Class.forName(assemblerName).newInstance();
                if (parser!=null) assembler.assemble(this, config, descriptor, parser);
                else assembler.assemble(this, config, descriptor);
                logger.debug("automated assembly complete!");
            } catch (Exception e) {
//                e.printStackTrace();
                throw new RuntimeException ("Error invoking assembler:"+e);
            }

            //allow for local initialization
            logger.debug("initialize local");
            initializeLocal(config);

            //handle any gateways which were specified
            if (gateways!=null) {
                logger.debug("handle any local gateways");
                Iterator it = gateways.iterator();
                while (it.hasNext()) {
                    EventGateway gateway = (EventGateway) it.next();
                    logger.debug("adding specified event gateway:"+gateway);
                    this.add(gateway);
                }
            }

            //now register the event gateway (note that we have to set the value to
            //false when registering to ensure that all possible combinations get
            //registered. If we don't do this we run into problems later)
            logger.debug("invoke register");
            boolean aliasVal = DefaultBaseEvent.USE_ID_ALIASES;
            DefaultBaseEvent.USE_ID_ALIASES = false;
            register(eb);
            DefaultBaseEvent.USE_ID_ALIASES = aliasVal;
            
            //show the app structure
            if (SHOW_APP_STRUCTURE) {
                logger.debug("Printing Stack Trace...");
                defaultEventGateway.printStackTrace(0, System.out);
            }
        
            if (logger.isInfoEnabled()) logger.info("Initialization Complete!");
        } catch (RuntimeException e) {
            logger.fatal("ALERT!!!!! Fatal err initializing servlet:", e);
            throw e;
        }
    }
        
    /**
     * <p>Here's where we destroy the servlet.
     */
    public void destroy() {
        try {
            logger.info("destroying servlet");

            //start by getting a reference to the EventBroker
            EventBroker eb = this.getEventBroker();

            //now deregister the event gateway
            boolean aliasVal = DefaultBaseEvent.USE_ID_ALIASES;
            DefaultBaseEvent.USE_ID_ALIASES = false;
            deregister(eb);
            DefaultBaseEvent.USE_ID_ALIASES = aliasVal;
            
            //finally allow for any local destruction
            destroyLocal();
            
            //remove any gateways which were specified
            if (gateways!=null) {
                Iterator it = gateways.iterator();
                while (it.hasNext()) {
                    EventGateway gateway = (EventGateway) it.next();
                    this.remove(gateway);
                }
            }

            //lb_032801_start - Patch submitted by Larry Brasfield
            //[larry.brasfield@theplatform.com] so JVM can exit once
            //the servlet's destroy() method is called.  The problem
            //is that the event pool cleanup thread will not stop. The
            //following code corrects this. You can find related changes
            //by searching for lb_032801
            //
            //Allow finalizer to get rid of this pool object.
            //That way, a shutdown can actually be effected.
            if (masterEventPool!=null) {
                masterEventPool.shutdown();
                masterEventPool = null;
            }
            masterEventBroker = null;
            System.gc();
            System.runFinalization();
            //lb_032801_end
                    
        } catch (RuntimeException e) {
            logger.fatal("ALERT!!!!! Fatal err destroying servlet:", e);
            throw e;
        }
    }
    
    
}
