/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultApplicationAssembler.java,v 1.3 2001/10/01 22:45:56 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.util.xml.*;

/**
 * This class basically assembles a Barracuda system based on 
 * an XML descriptor file. NOTE: this is experimental and will 
 * probably change, so keep that in mind as you work with it!!! 
 * For a sample file look at /WEB-INF/classes/ex4_opt1a.xml
 */
public class DefaultApplicationAssembler extends DefaultHandler implements ApplicationAssembler {
    
    //public constants
    protected static BCategory logger = BCategory.getBInstance(DefaultApplicationAssembler.class.getName());
    
    //private constants
    private static final String DEFAULT_PARSER = "org.apache.xerces.parsers.SAXParser";

    //recognized tags
    private static final String CONSTANT = "constant";
    private static final String EVENT_GATEWAY = "event-gateway";
    private static final String EVENT_INTEREST = "event-interest";
    private static final String EVENT_ALIAS = "event-alias";
    private static final String SET_PROPERTY = "set-property";
    
    //recognized attributes
    private static final String CLASS = "class";
    private static final String FACTORY = "factory";
    private static final String EVENT = "event";
    private static final String NAME = "name";

    //private vars
    protected String parserClass = null;
    protected String assemblySourceFile = null;
    protected EventGateway rootGateway = null;
    protected EventGateway curGateway = null;
    protected ServletConfig servletConfig = null;
    protected Stack objStack = null;
    protected boolean needPropVal = false;
    protected String className = null;
    protected String propName = null;
    protected String propVal = null;
    protected int depth = 0;
    
    /**
     * Assemble the system, given the root EventGateway and the
     * XML assembly decriptor name. The default parser will be 
     * used.
     *
     * @param rootGateway the root EventGateway
     * @param servletConfig the ServletConfig object
     * @param assemblySourceFile the XML assembly descriptor
     */
    public void assemble(EventGateway irootGateway, ServletConfig iservletConfig, String iassemblySourceFile) {
        assemble(irootGateway, iservletConfig, iassemblySourceFile, DEFAULT_PARSER);
    }
    
    /**
     * Assemble the system, given the root EventGateway, an
     * XML assembly decriptor name, and a specific SAX parser
     * class.
     *
     * @param rootGateway the root EventGateway
     * @param servletConfig the ServletConfig object
     * @param assemblySourceFile the XML assembly descriptor
     * @param parserClass the SAX parser class
     */
    public void assemble(EventGateway irootGateway, ServletConfig iservletConfig, String iassemblySourceFile, String iparserClass) {
        rootGateway = irootGateway;
        servletConfig = iservletConfig;
        assemblySourceFile = iassemblySourceFile;
        parserClass = iparserClass;

        logger.info("Assembling system from input source");
        try {
            //instantiate the parser
            logger.info("Instantiating parser ("+parserClass+")");
            XMLReader parser = (XMLReader) Class.forName(parserClass).newInstance();
            parser.setContentHandler(this);
            parser.setErrorHandler(this);

            //csc_100101.1
            //if the root gateway is an instance of Servlet, try getting
            //the input stream with the Servlet classloader (this will allow
            //us to read from the WEB-INF directory)
            InputStream is = null;
            if (servletConfig!=null) {        
                logger.info("Getting input source from Servlet ("+assemblySourceFile+")");
                is = servletConfig.getServletContext().getResourceAsStream(assemblySourceFile);
            }

            //if we couldn't find the file there, look in the /classes directory            
            if (is==null) {
                logger.info("Getting input source from Assembler ("+assemblySourceFile+")");
                is = this.getClass().getResourceAsStream(assemblySourceFile);
            }
            
            //get the source file
            InputSource source = new InputSource(is);
            
            //parse the file            
            logger.info("Parsing the source file...");
            parser.parse(source);
            
        } catch (org.xml.sax.SAXParseException spe) {
            logger.warn("Error assembling system!", spe);
            spe.printStackTrace(System.err);
        } catch (org.xml.sax.SAXException se) {
            if (se.getException() != null) {
                logger.warn("Error assembling system!", se.getException());
                se.getException().printStackTrace(System.err);
            } else {
                logger.warn("Error assembling system!", se);
                se.printStackTrace(System.err);
            }
        } catch (Exception e) {
            logger.warn("Error assembling system!", e);
            e.printStackTrace(System.err);
        }

        logger.info("Assembly complete!");
    }
    
    public void startDocument() {
        curGateway = rootGateway;
        objStack = new Stack();
        objStack.push(curGateway);
    }

    public void startElement(String uri, String local, String raw, Attributes attrs) throws SAXException {
        String curTag = local;
        logger.debug(depth, "uri:"+uri+" local:"+local+" raw:"+raw+" attrs:"+attrs);        

        //event-gateway
        if (curTag.equals(EVENT_GATEWAY)) {
        
            //figure out what the new gateway is
            depth++;
            EventGateway eg = null;
            String cl = attrs.getValue(CLASS);
            logger.debug(depth, "Creating event gateway: "+cl);
            try {
                eg = (EventGateway) Class.forName(cl).newInstance();
                objStack.push(eg);
            } catch (Exception e) {
                String msg = "Error instantiating event gateway";
                logger.warn(depth, msg+": ", e);
                throw new SAXException(msg, e);
            }

            //add the event gateway to the current gateway and
            //then make the new gateway the current gateway
            logger.debug(depth, "Adding event gateway: "+eg.getClass().getName());
            if (curGateway!=null) curGateway.add(eg);
            curGateway = eg;
            
        //event-interest
        } else if (curTag.equals(EVENT_INTEREST)) {
        
            //make sure the curGateway is an instance of DefaultEventGateway.
            //if it's not, we can't specify an event interest
            if (! (curGateway instanceof DefaultEventGateway)) throw new SAXException("Unable to specify event-interest! The current event gateway is not an instance of DefaultEventGateway!");
            DefaultEventGateway deg = (DefaultEventGateway) curGateway;
        
            //figure out what factory we want to add
            String factoryName = attrs.getValue(FACTORY);
            
            //now see if the factory name is a property within the current Gateway
            ListenerFactory lf = null;
            try {
                Field field = curGateway.getClass().getField(factoryName);
                if (field!=null) {
                    logger.debug(depth, "Getting listener factory from gateway: "+field);
                    lf = (ListenerFactory) field.get(curGateway);
                }
            } catch (Exception e) {
                logger.debug(depth, "Failed! "+e+" (this err is not fatal)");
            }
            
            //if not, we assume it's a class name and try to instantiate it
            if (lf==null) {            
                logger.debug(depth, "Creating listener factory: "+factoryName);
                try {
                    lf = (ListenerFactory) Class.forName(factoryName).newInstance();
                    objStack.push(lf);
                } catch (Exception e) {
                    String msg = "Error instantiating listener factory";
                    logger.warn(depth, msg+": ",e);
                    throw new SAXException(msg, e);
                }
            }
            
            //see if there is a specific event to register for            
            String evcl = attrs.getValue(EVENT);
            Class ev = null;
            if (evcl!=null) {
                logger.debug(depth, "Creating event class: "+evcl);
                try {
                    ev = Class.forName(evcl);
                } catch (Exception e) {
                    String msg = "Error creating event class";
                    logger.warn(depth, msg+": ",e);
                    throw new SAXException(msg, e);
                }
            }
                        
            //now actually specify interests
            if (ev!=null) deg.specifyLocalEventInterests(lf, ev);
            else deg.specifyLocalEventInterests(lf);

        //event-alias
        } else if (curTag.equals(EVENT_ALIAS)) {
            //make sure the curGateway is an instance of DefaultEventGateway.
            //if it's not, we can't specify an event interest
            if (! (curGateway instanceof DefaultEventGateway)) throw new SAXException("Unable to specify event-interest! The current event gateway is not an instance of DefaultEventGateway!");
            DefaultEventGateway deg = (DefaultEventGateway) curGateway;
        
            //see if there is a specific event to register for            
            String evcl = attrs.getValue(EVENT);
            Class ev = null;
            logger.debug(depth, "Creating event class: "+evcl);
            try {
                ev = Class.forName(evcl);
            } catch (Exception e) {
                String msg = "Error creating event class";
                logger.warn(depth, msg+": ",e);
                throw new SAXException(msg, e);
            }
            
            //now actually specify interests
            deg.specifyLocalEventAliases(ev);

        //set-property
        } else if (curTag.equals(SET_PROPERTY)) {
            //get the property name
            propName = attrs.getValue(NAME);
            propVal = null;
            needPropVal = true;

        //constant            
        } else if(curTag.equals(CONSTANT)) {
            className = attrs.getValue(CLASS);
            propName = attrs.getValue(NAME);
            needPropVal = true;
        }
    }

    public void endElement(String uri, String local, String raw) throws SAXException {
        String curTag = local;
        
        //event-gateway
        if (curTag.equals(EVENT_GATEWAY)) {
            logger.debug(depth, "Finished w/ gateway: " + curGateway.getClass().getName());
            curGateway = curGateway.getParent();
            objStack.pop();
            depth--;
            
        //event-interest    
        } else if (curTag.equals(EVENT_INTEREST)) {
            objStack.pop();

        //set-property
        } else if (curTag.equals(SET_PROPERTY)) {
            Object parent = objStack.peek();
            try {
                String methodName = propName;
                logger.debug(depth, "methodName:" + methodName);
                Class cl = parent.getClass();
                Method methods[] = cl.getMethods();
                boolean success = false;
                int i = 0;
                for (int max=methods.length; i<max; i++) {
                    Method m = methods[i];
                    if (!m.getName().equalsIgnoreCase("set" + propName)) continue;
                    methodName = m.getName();
                    Class paramTypes[] = m.getParameterTypes();
                    if (paramTypes.length>1) continue;
                    Class paramType = paramTypes[0];
                    logger.debug(depth, "method:" + methodName + " paramType:" + paramType);
                    success = setMethod(parent, m, paramType, propVal);
                    if (!success) continue;
                    logger.debug(depth, parent + "." + methodName + "(\"" + propVal + "\") successful");
                    break;
                }

                if (!success) {
                    Field field = cl.getField(propName);
                    Class paramType = field.getType();
                    success = setField(parent, field, paramType, propVal);
                    if (success) {
                        logger.debug(depth, parent + "." + propName + "=\"" + propVal + "\" successful");
                    } else {
                        throw new SAXException("Error setting " + parent + "." + propName + "=\"" + propVal + "\"");
                    }
                }
            } catch(Exception e) {
                String msg = "Error setting " + propName + " in parent:" + parent;
                logger.warn(depth, msg+": ",e);
                throw new SAXException(msg, e);
            }
        } else if(curTag.equals(CONSTANT)) {
            Class cl = null;
            try {
                cl = Class.forName(className);
                logger.debug(depth, "class:" + className);
                Field field = cl.getField(propName);
                Class paramType = field.getType();
                logger.debug(depth, "field:" + propName);
                boolean success = setField(cl, field, paramType, propVal);
                if(success) {
                    logger.debug(depth, cl + "." + propName + "=\"" + propVal + "\" successful");
                } else {
                    throw new SAXException("Error setting " + cl + "." + propName + "=\"" + propVal + "\"");
                }
            } catch(Exception e) {
                String msg = "Error setting " + propName + " in target:" + cl;
                logger.warn(depth, msg+": ",e);
                throw new SAXException(msg, e);
            }
        }
    }

    public void characters(char ch[], int start, int length) {
        if(needPropVal) {
            propVal = XMLUtil.fromXMLUnicodeString(new String(ch, start, length));
            needPropVal = false;
        } else {
            logger.debug(depth, "[characters] " + new String(ch, start, length));
        }    
    }

    public void ignorableWhitespace(char ch[], int start, int length) {
        logger.debug(depth, "[whitespace] " + new String(ch, start, length));
    }

    public void warning(SAXParseException ex) {
        System.err.println("[Warning] " + getLocationString(ex) + ": " + ex.getMessage());
    }

    public void error(SAXParseException ex) {
        System.err.println("[Error] " + getLocationString(ex) + ": " + ex.getMessage());
    }

    public void fatalError(SAXParseException ex) throws SAXException {
        System.err.println("[Fatal Error] " + getLocationString(ex) + ": " + ex.getMessage());
    }

    private String getLocationString(SAXParseException ex) {
        StringBuffer str = new StringBuffer();
        String systemId = ex.getSystemId();
        if(systemId!=null) {
            int index = systemId.lastIndexOf(47);
            if(index!=-1) systemId = systemId.substring(index + 1);
            str.append(systemId);
        }
        str.append(':');
        str.append(ex.getLineNumber());
        str.append(':');
        str.append(ex.getColumnNumber());
        return str.toString();
    }

    public boolean setMethod(Object target, Method m, Class paramType, String propVal) throws InvocationTargetException, IllegalAccessException {
        if (paramType.equals(String.class)) {
            m.invoke(target, new Object[] {propVal});
            return true;
        } else if (paramType.equals(boolean.class) || paramType.equals(Boolean.class)) {
            m.invoke(target, new Object[] {new Boolean(propVal)});
            return true;
        } else if (paramType.equals(byte.class) || paramType.equals(Byte.class)) {
            m.invoke(target, new Object[] {new Byte(propVal)});
            return true;
        } else if (paramType.equals(char.class) || paramType.equals(Character.class)) {
            byte b[] = propVal.getBytes();
            m.invoke(target, new Object[] {new Character((char)b[0])});
            return true;
        } else if (paramType.equals(double.class) || paramType.equals(Double.class)) {
            m.invoke(target, new Object[] {new Double(propVal)});
            return true;
        } else if (paramType.equals(float.class) || paramType.equals(Float.class)) {
            m.invoke(target, new Object[] {new Float(propVal)});
            return true;
        } else if (paramType.equals(int.class) || paramType.equals(Integer.class)) {
            m.invoke(target, new Object[] {new Integer(propVal)});
            return true;
        } else if (paramType.equals(long.class) || paramType.equals(Long.class)) {
            m.invoke(target, new Object[] {new Long(propVal)});
            return true;
        } else if (paramType.equals(short.class) || paramType.equals(Short.class)) {
            m.invoke(target, new Object[] {new Short(propVal)});
            return true;
        } else {
            return false;
        }
    }

    public boolean setField(Object target, Field field, Class paramType, String propVal) throws IllegalAccessException {
        if (paramType.equals(String.class)) {
            field.set(target, propVal);
            return true;
        } else if (paramType.equals(boolean.class) || paramType.equals(Boolean.class)) {
            field.set(target, new Boolean(propVal));
            return true;
        } else if (paramType.equals(byte.class) || paramType.equals(Byte.class)) {
            field.set(target, new Byte(propVal));
            return true;
        } else if (paramType.equals(char.class) || paramType.equals(Character.class)) {
            byte b[] = propVal.getBytes();
            field.set(target, new Character((char)b[0]));
            return true;
        } else if (paramType.equals(double.class) || paramType.equals(Double.class)) {
            field.set(target, new Double(propVal));
            return true;
        } else if (paramType.equals(float.class) || paramType.equals(Float.class)) {
            field.set(target, new Float(propVal));
            return true;
        } else if (paramType.equals(int.class) || paramType.equals(Integer.class)) {
            field.set(target, new Integer(propVal));
            return true;
        } else if (paramType.equals(long.class) || paramType.equals(Long.class)) {
            field.set(target, new Long(propVal));
            return true;
        } else if (paramType.equals(short.class) || paramType.equals(Short.class)) {
            field.set(target, new Short(propVal));
            return true;
        } else {
            return false;
        }
    }

    public static void main(String[] args) {
//        new ApplicationAssembler(null, "/org/enhydra/barracuda/examples/ex4/system.xml").assemble();
        new DefaultApplicationAssembler().assemble(null, null, "/ex4_opt1a.xml");
    }
    
}
