/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultBaseEventListener.java,v 1.7 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.util.logging.*;

/**
 * <p>This class provides the default implementation for 
 * a BaseEventListener. This is an abstract class, meaning
 * you have to extend it if you intend to use it. Typically
 * you would implement it as an inner class and extend 
 * handleReqEvent, handleRespEvent, or handleOtherEvent. Or,
 * you can always just override handleEvent and do whatever
 * you want.
 */
public abstract class DefaultBaseEventListener implements BaseEventListener {

    //public vars
    protected static BCategory localLogger = BCategory.getBInstance(DefaultBaseEventListener.class.getName());

    //private vars
    protected String idStr = null;

    /**
     * Handle all events. This represents a default implementation 
     * that will route the event on to the convenience respective 
     * convenience methods. 
     *
     * @param context the EventContext for the base event we are handling
     * @throws EventException
     */
    public void handleEvent(EventContext context) throws EventException {
        //custom code here
        if (context==null) return;
        BaseEvent event = context.getEvent();
        try {
            if (localLogger.isDebugEnabled()) localLogger.debug(0, "Entering "+this.getListenerID());
            if (event instanceof ControlEvent && context instanceof ControlEventContext) {
                handleControlEvent((ControlEventContext) context);
            } else if (event instanceof ViewEvent && context instanceof ViewEventContext) {
                handleViewEvent((ViewEventContext) context);
            } else {
                handleOtherEvent(context);
            }
        } catch (ServletException e) {
            throw new EventException("Unexpected Servlet Error", e);
        } catch (IOException e) {
            throw new EventException("Unexpected IOException", e);
        } finally {
            event.setHandled(isHandled());    
            if (localLogger.isDebugEnabled()) localLogger.debug(0, "Exiting "+this.getListenerID());
        }
    }

    
    /**
     * Handle HttpRequestEvents
     *
     * @param context the ControlEventContext
     * @throws EventException
     * @throws ServletException
     * @throws IOException
     */
    public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
        //custom code here (we throw an exception so that if a developer fails 
        //to extend the proper method, they'll at least get a nice error message 
        //at runtime pointing out the problem. This should help prevent the case
        //where the developer writes handling code but extends the wrong handling 
        //method for the particular type of event being handled)
        String msg = "Invoked handleControlEvent()...this method should have been extended";
        localLogger.warn(msg);
        throw new UnhandledEventException(msg, context);
    }
    
    /**
     * Handle HttpResponseEvents
     *
     * @param context the ViewEventContext
     * @throws EventException
     * @throws ServletException
     * @throws IOException
     */
    public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
        //custom code here (we throw an exception so that if a developer fails 
        //to extend the proper method, they'll at least get a nice error message 
        //at runtime pointing out the problem. This should help prevent the case
        //where the developer writes handling code but extends the wrong handling 
        //method for the particular type of event being handled)
        String msg = "Invoked handleViewEvent()...this method should have been extended";
        localLogger.warn(msg);
        throw new UnhandledEventException(msg, context);
    }
    
    /**
     * Handle all Other events
     *
     * @param context the EventContext
     * @throws EventException
     */
    public void handleOtherEvent(EventContext context) throws EventException {
        //custom code here (we throw an exception so that if a developer fails 
        //to extend the proper method, they'll at least get a nice error message 
        //at runtime pointing out the problem. This should help prevent the case
        //where the developer writes handling code but extends the wrong handling 
        //method for the particular type of event being handled)
        String msg = "Invoked handleOtherEvent()...this method should have been extended";
        localLogger.warn(msg);
        throw new UnhandledEventException(msg, context);
    }

    /**
     * Return true if the event was handled in the handleEvent method. 
     * By default, most implementations will return true for you, so 
     * the only time you'd need to override is if the handler wanted 
     * to specifically indicate that it had NOT handled the event 
     * (ie in logging scenarios).
     *
     * @return true if the event was handled in the handleEvent method
     */
    public boolean isHandled() {
        return true;
    }

    /**
     * Get the ID that identifies this listener. This will typically be the 
     * class name.
     *
     * @return a string that uniquely identifies this listener
     */
    public String getListenerID() {
        if (idStr==null) {
            idStr = DefaultBaseEvent.getClassID(this.getClass());
        }
        return idStr;
    }
}
