/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultViewHandler.java,v 1.4 2001/09/28 21:07:25 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event.helper;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * <p>A very simple view handler that provides a default implementation
 * tailored for the use of components. In many ways, this is analagous to
 * the ComponentGateway class--a component hierarchy is created, initialized
 * and rendered automatically for you. All the developer has to do is a)
 * implement the handleViewEvent method to add any custom components to the root
 * component (ie. for rendering) and b) return the master DOM page that needs
 * to be rendered to generate the final view.
 */
public abstract class DefaultViewHandler extends DefaultBaseEventListener {

    //public vars...eventually, these should probably be final
    protected static BCategory logger = BCategory.getBInstance(DefaultViewHandler.class.getName());
    public static boolean printPretty = false;
    public static boolean preventCaching = false;
    public boolean recycleChildren = false;

    /**
     * Public noargs constructor
     */
    public DefaultViewHandler() {}


    //-------------------- DefaultViewHandler --------------------
    /**
     * <p>Handle a view event. This is the method developers will 
     * typically override. The developers shaould add any components
     * to the root component and then return the underlying DOM Document 
     * (that backs their components) so it can be rendered
     *
     * @param root the root component which will get rendered as a result 
     *         of this request
     * @param vec the ViewEventContext object describes what features the 
     *         client view is capable of supporting
     * @return the Document to be rendered
     * @throws ServletException
     * @throws IOException
     */
    public abstract Document handleViewEvent(BComponent root, ViewEventContext vec) throws EventException, ServletException, IOException;
    

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        return new DefaultDOMWriter(printPretty, preventCaching);
    }


    //-------------------- DefaultBaseEventListener --------------
    /**
     * Handle the ViewEvent
     */
    public void handleViewEvent(ViewEventContext vec) throws EventException, ServletException, IOException {
        long bmillis = 0;
        long smillis = 0;
        long emillis = 0;
        long elapsed1 = 0;
        long elapsed2 = 0;
        if (logger.isInfoEnabled()) bmillis = System.currentTimeMillis();
        if (logger.isInfoEnabled()) logger.info("Handling ViewEvent in "+this);

        try {
            //start by figuring out the ViewCapabilities
            if (logger.isDebugEnabled()) logger.debug(1, "Create the ViewContext");
            ViewContext vc = new DefaultViewContext(vec);
    
            //create our root component
            if (logger.isDebugEnabled()) logger.debug(1, "Create component root");
            BComponent broot = new BComponent();
            broot.setName("Root");
        
            //give the implementation a chance to add any components to the root
            if (logger.isDebugEnabled()) logger.debug(1, "Handling default");
    //            Document doc = handleDefault(broot, vc, req, resp);
            Document doc = handleViewEvent(broot, vec);
        
            //now init the component
            if (logger.isDebugEnabled()) logger.debug(1, "Invoking initCycle on component hierarchy");
            broot.initCycle();
        
            //now render the component
            if (logger.isDebugEnabled()) logger.debug(1, "Rendering component hierarchy");
            if (logger.isDebugEnabled()) smillis = System.currentTimeMillis();
            broot.render(vc);
            if (logger.isDebugEnabled()) elapsed1 = System.currentTimeMillis()-smillis;

            //its possible the implementor may want to recycle children...if so, remove them
            //prior to calling destroy on the root
            if (recycleChildren) {
                if (logger.isDebugEnabled()) logger.debug(1, "Recycling child components");
                List children = broot.getChildren();
                if (children!=null) {
                    for (int i=children.size()-1; i>=0; i--) {
                        broot.removeChild(i);            
                    }
                }
            }

            //now destroy the component
            if (logger.isDebugEnabled()) logger.debug(1, "Invoking destroyCycle on component hierarchy");
            broot.destroyCycle();
        
            //now render the page
            if (logger.isDebugEnabled()) logger.debug(1, "Rendering the DOM");
            if (logger.isInfoEnabled()) smillis = System.currentTimeMillis();
            this.getDOMWriter().write(doc, vec.getResponse());
            if (logger.isInfoEnabled()) {
                elapsed2 = System.currentTimeMillis()-smillis;
                emillis = System.currentTimeMillis();
            }
        } catch (RenderException e) {
            //if we get an EventException, handle it
            if (logger.isInfoEnabled()) logger.info("Unexpected RenderException:"+e);
            throw new EventException ("Unexpected RenderException:"+e, e);
            
        } finally {
            //nop
        }

        if (logger.isInfoEnabled()) logger.info("ViewEvent handled! (rendered in "+(elapsed1)+"/written in "+(elapsed2)+" of "+(emillis-bmillis)+" millis)");
    }


}
