/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultFormMap.java,v 1.11 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.util.data.DefaultStateMap;
import org.enhydra.barracuda.core.util.data.ServletRequestParameterStateMap;
import org.enhydra.barracuda.core.util.data.StateMap;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * <p>This class provides the default implementation of a FormMap.
 *
 * <p>A FormMap is used to provide a virtual representation of a
 * form. It can contain any number of unique FormElements, and
 * it can also be associated with FormValidators. The primary 
 * function of a form map is to:
 *
 * <ul>
 *         <li>define the map (with its elements and validators)</li>
 *         <li>actually populate the map (from either a ServletRequest
 *            or a StateMap)</li>
 *         <li>validate the map (by invoking all the validators associated
 *            with the form and all its elements)</li>
 *        <li>provide convenience methods to access the underlying values
 *            of the form elements contained in this map</li>
 * </ul>
 */
//public abstract class DefaultFormMap implements FormMap {
public class DefaultFormMap implements FormMap {

    //public constants
    protected static BCategory localLogger = BCategory.getBInstance(DefaultFormMap.class.getName());

    private static final String PARSE_EXCEPTION = "ParseException";

    //non-public vars
    protected Map elements = new HashMap(10);
    protected List validators = new ArrayList(5);
    protected StateMap statemap = new DefaultStateMap();

    //--------------- FormMap ------------------------------------
    /**
     * This defines an element to be mapped by this form, using 
     * the key from the FormElement. You would invoke this method 
     * for each element in the form.
     *
     * @param element a FormElement to be mapped by this form
     */
    public void defineElement(FormElement element) {
        defineElement(element.getKey(), element);
    }

    /**
     * This defines an element to be mapped by this form.
     * You would invoke this method for each element in the form.
     *
     * @param key the key which uniquely identifies this FormElement
     * @param element a FormElement to be mapped by this form
     */
    public void defineElement(String key, FormElement element) {
        if (localLogger.isDebugEnabled()) localLogger.debug("Defining FormElement: Key=["+key+"] Element=["+element+"]");
        elements.put(key, element);
    }

    /**
     * This defines a validator for the entire form. This validator 
     * will be invoked prior to validating specific form element 
     * validators. Calling this method multiple times will result 
     * in multiple form validators being added to the form (they 
     * will be invoked in the order they were added)
     *
     * @param validator a form validator to be applied to the entire form
     */
    public void defineValidator(FormValidator validator) {
        if (!validators.contains(validator)) {
            if (localLogger.isDebugEnabled()) localLogger.debug("Defining validator: Validator=["+validator+"]");
            validators.add(validator);
        }
    }


    /**
     * This is where we actually take an incoming form (in
     * the form of a ServletRequest) and map it using the 
     * definitions supplied by all the FormElements. If there 
     * are multiple parameters for a given key, the values 
     * will be mapped into List structures
     *
     * @param req the ServletRequest to map paramters from based on
     *        all defined FormElements
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws MappingException if for some reason the value cannot 
     *        be mapped successfully
     */
    public FormMap map(ServletRequest req) {
        if (localLogger.isInfoEnabled()) localLogger.info("Mapping ServletRequest to FormMap");
        if (localLogger.isDebugEnabled()) {
            localLogger.debug(1,"ServletRequest parameters");
            Map map = new ServletRequestParameterStateMap(req).getStateValues();
            Collections.printStackTrace(map, 0, localLogger, null);
            localLogger.debug(1,"Map elements");
            Collections.printStackTrace(elements, 0, localLogger, null);
        }
                    
        Iterator it = elements.values().iterator();
        while (it.hasNext()) {
            //get the next element, determine the original values
            FormElement element = (FormElement) it.next();
            if (localLogger.isDebugEnabled()) localLogger.debug(1, "Next FormElement: "+element);
            String[] origVals = null;
            if (element.allowMultiples()) origVals = req.getParameterValues(element.getKey());
            else { 
                String param = req.getParameter(element.getKey());
                if (param!=null) origVals = new String[]{param};
            }

            //now map the values. If there are multiple parameters for 
            //a given key, the values will be mapped into List structures
            Object origVal = null;
            Object val = null;
            if (origVals!=null) {
                int max = origVals.length;
                 if (max==1) {
                    //if there's only one
                    origVal = origVals[0];
                    val = mapElement(element, origVal.toString());
                 } else {
                    //if there's multiple values
                    origVal = new ArrayList();
                    val = new ArrayList();
                    List lorigVal = (List) origVal;
                    List lval = (List) val;
                    for (int i=0; i<max; i++) {
                        lorigVal.add(origVals[i]);        
                        lval.add(mapElement(element, origVals[i]));
                    }
                 }                
            } else {
                val = mapElement(element, null);
            }
            
            
            //place the values in the form element
            element.setOrigVal(origVal);
            element.setVal(val);
        }
        return this;
    }

    /**
     * This is where we actually take an incoming form (in
     * the form of a StateMap) and map it using the definitions 
     * supplied by all the FormElements.
     *
     * @param map the StateMap to map properties from based on
     *        all defined FormElements
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws MappingException if for some reason the value cannot 
     *        be mapped successfully
     */
    public FormMap map(StateMap map) {
        if (localLogger.isInfoEnabled()) localLogger.info("Mapping StateMap to FormMap");
        Iterator it = elements.values().iterator();
        while (it.hasNext()) {
            //get the next element, determine the original value
            FormElement element = (FormElement) it.next();
            Object origVal = map.getState(element.getKey());
            
            //now map the element 
            Object val = mapElement(element, origVal!=null ? origVal.toString() : null);

            //place the values in the form element
            element.setOrigVal(origVal);
            element.setVal(val);
        }
        return this;
    }
    
    protected Object mapElement(FormElement element, String origVal) {
        if (localLogger.isInfoEnabled()) localLogger.info("Mapping Element: "+element.getKey()+"="+origVal);
        Object val = null;
        FormType type = element.getType();
        
        if (origVal!=null) {                        

            //parse the value; catch any exceptions and use the default 
            //value instead
            try {
                val = type.parse(origVal);
            } catch (ParseException e) {
                if (localLogger.isDebugEnabled()) localLogger.debug(1, "ParseException:", e);
                element.setParseException(e);
            }
        
/*        
            //...Strings
            if (type==FormType.STRING) {
                val = origVal.toString();
                
            //...Booleans (we can handle "true", "on", or "yes"
            } else if (type==FormType.BOOLEAN) {
                String tval = origVal;
                if (tval!=null) {
                     if (tval.trim().toLowerCase().equals("on") ||
                         tval.trim().toLowerCase().equals("yes")) tval = "true";
                }
                val = new Boolean(tval);

            //...Integers
            } else if (type==FormType.INTEGER) {
                try {val = new Integer(origVal);} 
                catch (NumberFormatException e) {}
                
            //...Dates
            } else if (type==FormType.DATE) {
                try {val = DateFormat.getDateInstance().parse(origVal);} 
                catch (ParseException e) {}
                
            //...Longs
            } else if (type==FormType.LONG) {
                try {val = new Long(origVal);} 
                catch (NumberFormatException e) {}
            
            //...Shorts
            } else if (type==FormType.SHORT) {
                try {val = new Short(origVal);} 
                catch (NumberFormatException e) {}
                        
            //...Doubles
            } else if (type==FormType.DOUBLE) {
                try {val = new Double(origVal);} 
                catch (NumberFormatException e) {}
                        
            //...Floats
            } else if (type==FormType.FLOAT) {
                try {val = new Float(origVal);} 
                catch (NumberFormatException e) {}
            }
*/
        }
                
        //if the val is still null, try setting it to the default value
        if (val==null) {
            if (localLogger.isDebugEnabled()) localLogger.debug(1, "Using default val");
            val = element.getDefaultVal();
        }

        if (localLogger.isDebugEnabled()) localLogger.debug(1, "Result: "+val);
        return val;
    }

    /**
     * Validate the entire form (both form level and elements). 
     * We start by invoking form validators which apply to individual 
     * form elements, then we invoke any which apply to the entire form
     *
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws ValidationException if the form (or any element within it)
     *        is invalid
     */
    public FormMap validate(boolean deferExceptions) throws ValidationException {
        if (localLogger.isInfoEnabled()) localLogger.info("Validating FormMap (form & elements)");
        ValidationException ve = null;

        try {
            //validate each individual element
            try {
                validateElements(deferExceptions);
            } catch (DeferredValidationException dve) {
                if (ve==null) ve = new DeferredValidationException(dve.getSource(), "Validation err:"+dve);
                ve.addSubException(dve);
            }


            //validate the entire form
            try {
                validateForm(deferExceptions);
            } catch (DeferredValidationException dve) {
                if (ve==null) ve = new DeferredValidationException(dve.getSource(), "Validation err:"+dve);
                ve.addSubException(dve);
            }
        } catch (ValidationException e) {
            if (localLogger.isDebugEnabled()) localLogger.debug(1, "Validation err! (immediate): ", e);
            throw e;
        }
        
        //now, if we have generated a ValidationExceptions,
        //rethrow it
        if (ve!=null) throw ve;
        return this;
    }

    /**
     * Validate just the elements (not the form)
     *
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws ValidationException if the form (or any element within it)
     *        is invalid
     */
    public FormMap validateElements(boolean deferExceptions) throws ValidationException {
        if (localLogger.isInfoEnabled()) localLogger.info("Validating FormMap (elements)");
        ValidationException ve = null;

        try {
            //validate each individual element
            if (localLogger.isDebugEnabled()) localLogger.debug(1, "Validating individual elements...");
            Iterator it = elements.values().iterator();
            FormElement element = null;
            while (it.hasNext()) {
                try {
                    element = (FormElement) it.next();
                    if (localLogger.isDebugEnabled()) localLogger.debug(2, "Next FormElement: "+element);
                    FormValidator fv = element.getValidator();
                    if (fv!=null) fv.validate(element, this, deferExceptions);
                    if (localLogger.isDebugEnabled()) localLogger.debug(2, "Element Valid!");
                } catch (DeferredValidationException dve) {
                    if (localLogger.isDebugEnabled()) localLogger.debug(2, "Element Invalid! (deferred): "+dve.getMessage());
                    if (ve==null) ve = new DeferredValidationException(element, "Validation err:"+dve);
                    ve.addSubException(dve);
                }
            }
        } catch (ValidationException e) {
            if (localLogger.isDebugEnabled()) localLogger.debug(1, "Validation err! (immediate): ", e);
            throw e;
        }
        
        //now, if we have generated a ValidationExceptions,
        //rethrow it
        if (ve!=null) throw ve;
        return this;
    }
    
    /**
     * Validate just the form (not the individual elements)
     *
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws ValidationException if the form (or any element within it)
     *        is invalid
     */
    public FormMap validateForm(boolean deferExceptions) throws ValidationException {
        if (localLogger.isInfoEnabled()) localLogger.info("Validating FormMap (form)");
        ValidationException ve = null;

        try {
            //validate the entire form
            if (localLogger.isDebugEnabled()) localLogger.debug(1, "Validating entire form...");
            try {
                Iterator it = validators.iterator();
                while (it.hasNext()) {
                    Object obj = it.next();
                    if (localLogger.isDebugEnabled()) localLogger.debug(2, "Next obj:"+obj);
                    FormValidator fv = (FormValidator) obj;
                    if (localLogger.isDebugEnabled()) localLogger.debug(2, "Next FormValidator: "+fv);
                    fv.validate(null, this, deferExceptions);
                    if (localLogger.isDebugEnabled()) localLogger.debug(2, "Form Valid!");
                }
            } catch (DeferredValidationException dve) {
                if (localLogger.isDebugEnabled()) localLogger.debug(2, "Form Invalid! (deferred): ", dve);
                if (ve==null) ve = new DeferredValidationException(this, "Validation err:"+dve);
                ve.addSubException(dve);
            }
        } catch (ValidationException e) {
            if (localLogger.isDebugEnabled()) localLogger.debug(1, "Validation err! (immediate): ", e);
            throw e;
        }
        
        //now, if we have generated a ValidationExceptions,
        //rethrow it
        if (ve!=null) throw ve;
        return this;
    }
    
    

    //-------------------- StateMap ------------------------------
    /**
     * set a property in this StateMap
     *
     * @param key the state key object
     * @param val the state value object
     */    
    public void putState(Object key, Object val) {
        statemap.putState(key,val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the state key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        return statemap.getState(key);
    }
    
    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        return statemap.removeState(key);
    }
    
    /**
     * get a list of the keys for this StateMap
     *
     * @return a list the keys for this StateMap
     */    
    public List getStateKeys() {
        return statemap.getStateKeys();
    }
    
    /**
     * get a copy of the underlying Map
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateValues() {
        return statemap.getStateValues();    
    }


    //--------------- Convenience methods ------------------------
    /**
     * Return true if an element exists and its value is not null
     *
     * @param key the key which uniquely identifies this FormElement
     * @return true if an element exists (not null)
     */
    public boolean exists(String key) {
        FormElement fel = (FormElement) elements.get(key);
        if (fel==null) return false;
        return (fel.getVal()!=null);        
    }

    /**
     * Get an element by key
     *
     * @param key the key which uniquely identifies this FormElement
     * @return the FormElement for this key (may be null)
     */
    public FormElement getElement(String key) {
        return (FormElement) elements.get(key);
    }
    
    /**
     * return a map with containing all the elements in this form map
     *
     * @return a copy of the Map that backs this FormMap
     */
    public Map getElements() {
        return new HashMap(elements);
    }


    /**
     * Manually set the value of an element. If an element does not 
     * exist for this key one will be created.
     *
     * @param key the key
     * @param val the value for the key
     */
    public void setVal(String key, Object val) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) {
            el = new DefaultFormElement(key);
            defineElement(key, el);
        }
        el.setVal(val);
    }
    
    /**
     * Get the value for a given key. This is basically a convenience
     * method. You could manually grab the FormElement using getElement
     * and then retrieve the value that way as well.
     *
     * @param key the key which uniquely identifies this FormElement
     * @return the value for this key (may be null)
     */
    public Object getVal(String key) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) return null;
        else return el.getVal();
    }
    
    /**
     * Get a String value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
    public String getStringVal(String key) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) return null;
        else return (String) el.getVal();
    }

    /**
     * Get the value for a given key, defaulting accordingly if
     * the value is null. 
     *
     * @param key the key which uniquely identifies this FormElement
     * @param dflt the default value to be used if the underlying value is null
     * @return the value for this key (may be null)
     */
    public String getStringVal(String key, String dflt) {
        String val = getStringVal(key);
        return (val!=null ? val : dflt);
    }

    /**
     * Get an Boolean value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
    public Boolean getBooleanVal(String key) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) return null;
        else return (Boolean) el.getVal();
    }

    /**
     * Get the value for a given key, defaulting accordingly if
     * the value is null. 
     *
     * @param key the key which uniquely identifies this FormElement
     * @param dflt the default value to be used if the underlying value is null
     * @return the value for this key (may be null)
     */
    public Boolean getBooleanVal(String key, Boolean dflt) {
        Boolean val = getBooleanVal(key);
        return (val!=null ? val : dflt);
    }

    /**
     * Get an Integer value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
    public Integer getIntegerVal(String key) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) return null;
        else return (Integer) el.getVal();
    }

    /**
     * Get the value for a given key, defaulting accordingly if
     * the value is null. 
     *
     * @param key the key which uniquely identifies this FormElement
     * @param dflt the default value to be used if the underlying value is null
     * @return the value for this key (may be null)
     */
    public Integer getIntegerVal(String key, Integer dflt) {
        Integer val = getIntegerVal(key);
        return (val!=null ? val : dflt);
    }

    /**
     * Get an Date value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
    public Date getDateVal(String key) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) return null;
        else return (Date) el.getVal();
    }

    /**
     * Get the value for a given key, defaulting accordingly if
     * the value is null. 
     *
     * @param key the key which uniquely identifies this FormElement
     * @param dflt the default value to be used if the underlying value is null
     * @return the value for this key (may be null)
     */
    public Date getStringVal(String key, Date dflt) {
        Date val = getDateVal(key);
        return (val!=null ? val : dflt);
    }

    /**
     * Get an Long value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
    public Long getLongVal(String key) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) return null;
        else return (Long) el.getVal();
    }

    /**
     * Get the value for a given key, defaulting accordingly if
     * the value is null. 
     *
     * @param key the key which uniquely identifies this FormElement
     * @param dflt the default value to be used if the underlying value is null
     * @return the value for this key (may be null)
     */
    public Long getLongVal(String key, Long dflt) {
        Long val = getLongVal(key);
        return (val!=null ? val : dflt);
    }

    /**
     * Get an Short value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
    public Short getShortVal(String key) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) return null;
        else return (Short) el.getVal();
    }

    /**
     * Get the value for a given key, defaulting accordingly if
     * the value is null. 
     *
     * @param key the key which uniquely identifies this FormElement
     * @param dflt the default value to be used if the underlying value is null
     * @return the value for this key (may be null)
     */
    public Short getShortVal(String key, Short dflt) {
        Short val = getShortVal(key);
        return (val!=null ? val : dflt);
    }

    /**
     * Get an Double value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
    public Double getDoubleVal(String key) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) return null;
        else return (Double) el.getVal();
    }

    /**
     * Get the value for a given key, defaulting accordingly if
     * the value is null. 
     *
     * @param key the key which uniquely identifies this FormElement
     * @param dflt the default value to be used if the underlying value is null
     * @return the value for this key (may be null)
     */
    public Double getDoubleVal(String key, Double dflt) {
        Double val = getDoubleVal(key);
        return (val!=null ? val : dflt);
    }

    /**
     * Get an Float value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
    public Float getFloatVal(String key) {
        FormElement el = (FormElement) elements.get(key);
        if (el==null) return null;
        else return (Float) el.getVal();
    }
    
    /**
     * Get the value for a given key, defaulting accordingly if
     * the value is null. 
     *
     * @param key the key which uniquely identifies this FormElement
     * @param dflt the default value to be used if the underlying value is null
     * @return the value for this key (may be null)
     */
    public Float getFloatVal(String key, Float dflt) {
        Float val = getFloatVal(key);
        return (val!=null ? val : dflt);
    }

}
