/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultFormValidator.java,v 1.7 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * <p>This class provides the default implementation of FormValidator.
 * 
 * <p>A FormValidator is designed to validate one or more FormElements
 * within a FormMap, OR a FormMap itself. The real work occurs in the 
 * validate() method. A validator is also capable of containing other 
 * validators, which makes it very easy to create a complex set of 
 * validation logic by assembling a "master" validator from a set of
 * simpler validators.
 *
 * <p>When the validate method is invoked, we start by performing element
 * level validation by invoking the validateFormElement() methods for each 
 * of the elements in the form. If that succeeds, we validate the whole 
 * map by invoking validateForm() method to validate the form itself. Finally,
 * we invoke any child validators. In this way, a validator is not considered
 * validated until all it's rules plus those for all of it's children have 
 * passed muster.
 *
 * <p>When an invalid condition occurs, typically the developer will throw
 * a ValidationException, which stops the validation process immediately and
 * returns the specified error. There are times, however, where the developer
 * will want to try and validate as many of the rules as possible in order
 * to return all known errors at once. In this case, the developer should throw
 * DeferredValidationExceptions. These are caught internally and added to one 
 * master validation exception, which is thrown when the validation process 
 * completes.
 *
 * <p>In typical usage the developer would extend this class to 
 * override one of the three validateFormElement methods. 
 * FormValidators can also act as containers for other FormValidators,
 * thereby allowing the developer to dynamically assemble complex 
 * validators from simpler validator objects.
 */
public class DefaultFormValidator extends AbstractFormValidator {

    protected static BCategory localLogger = BCategory.getBInstance(AbstractFormValidator.class.getName());

    protected List validators = new ArrayList();
    
    /**
     * Public noargs constructor (errorMessage defaults to null)
     */
    public DefaultFormValidator() {
        this(null);
    }

    /**
     * Public constructor with a default errorMessage to be dispatched
     * when an invalid state occurs. The deferExceptions parameter
     * defaults to true.
     *
     * @param errorMessage the default error message for this validator
     */
    public DefaultFormValidator(String ierrorMessage) {
        setErrorMessage(ierrorMessage);
    }

    /**
     * Add a child validator
     *
     * @param validator the child validator to be added
     */
    public void addValidator(FormValidator validator) {
        validators.add(validator);
    }
    
    /**
     * Remove a child validator
     *
     * @param validator the child validator to be removed
     */
    public void removeValidator(FormValidator validator) {
        validators.remove(validator);
    }
    
    /**
     * Get a list of all child validators
     *
     * @return a list of all child validators
     */
    public List getValidators() {
        return new ArrayList(validators);
    }

    /**
     * Validate a FormElement locally and allow any child
     * validators a chance to validate as well.
     *
     * @param element the form element to be validated (null indicates
     *        we want to perform form level validation)
     * @param map the map to which the element belongs (sometimes necessary
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validate(FormElement element, FormMap map, boolean deferExceptions) throws ValidationException {
        ValidationException ve = null;
        if (localLogger.isInfoEnabled()) localLogger.info("Validating form");
        
        //perform local form validation
        try {
            //perform specific element validation (only occurs
            //when we're looking at a specific element)
            if (element!=null) {
                if (localLogger.isDebugEnabled()) localLogger.debug(1, "Validating local form elements:"+element);
                Object val = element.getVal();
                validateFormElement(val, element, deferExceptions);
                validateFormElement(val, element, map, deferExceptions);
            }
            
            //perform form level validation (only occurs
            //when we're NOT looking at a specific element)
            if (element==null) {
                if (localLogger.isDebugEnabled()) localLogger.debug(1, "Validating form:"+map);
                validateForm(map, deferExceptions);            
            }
        } catch (DeferredValidationException dve) {
            if (localLogger.isDebugEnabled()) localLogger.debug(1, "Deferring Validation Exception", dve);
            if (ve==null) ve = new DeferredValidationException();
            ve.addSubException(dve);
        }
        
        //give any child validators a chance to validate it as well
        Iterator it = validators.iterator();
        while (it.hasNext()) {
            FormValidator fv = (FormValidator) it.next();
            if (localLogger.isDebugEnabled()) localLogger.debug(1, "Validating child:"+fv);
            try {fv.validate(element, map, deferExceptions);}
            catch (DeferredValidationException dve) {
                if (localLogger.isDebugEnabled()) localLogger.debug(1, "Deferring Child Validation Exception:", dve);
                if (ve==null) ve = new DeferredValidationException();
                ve.addSubException(dve);
            }
        }
        
        //now, if we have generated a ValidationExceptions,
        //rethrow it
        if (ve!=null) throw ve;
    }
    
    /**
     * Validate an entire FormMap. This is the method developers should
     * override to provide specific validation for the entire form, as 
     * opposed to validating a specific element within the form.
     * 
     * To indicate a form is invalid, through a ValidationException, 
     * which will interrupt the validation process immediately. If you 
     * want to indicate an error, but would still like validation to 
     * continue (so that you can identify multiple errors in one 
     * validation pass) throw a DeferredValidationException instead.
     *
     * @param map the map to be validated
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateForm(FormMap map, boolean deferExceptions) throws ValidationException {
        //override this if you want to validate an entire form
    }
    
    /**
     * Validate a FormElement. This is the method developers should
     * override to provide specific validation based on the value
     * and (potentially) information contained in the FormElement.
     * 
     * To indicate an element is invalid, through a ValidationException, 
     * which will interrupt the validation process immediately. If you 
     * want to indicate an error, but would still like validation to 
     * continue (so that you can identify multiple errors in one 
     * validation pass) throw a DeferredValidationException instead.
     *
     * @param val the actual value to be validated
     * @param element the form element that contains the val
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        //override one of these two methods    to validate a specific element
    }

    /**
     * Validate a FormElement. This is the method developers should
     * override to provide specific validation based on the value
     * and (potentially) information contained in the FormElement and
     * FormMap structures.
     * 
     * To indicate an element is invalid, through a ValidationException, 
     * which will interrupt the validation process immediately. If you 
     * want to indicate an error, but would still like validation to 
     * continue (so that you can identify multiple errors in one 
     * validation pass) throw a DeferredValidationException instead.
     *
     * @param val the actual value to be validated
     * @param element the form element that contains the val
     * @param map the map to which the element belongs (sometimes necessary
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, FormMap map, boolean deferExceptions) throws ValidationException {
        //override one of these two methods    to validate a specific element
    }

    
}
