/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: FormMap.java,v 1.7 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.util.data.StateMap;

/**
 * <p>A FormMap is used to provide a virtual representation of a
 * form. It can contain any number of unique FormElements, and
 * it can also be associated with FormValidators. The primary 
 * function of a form map is to:
 *
 * <ul>
 *         <li>define the map (with its elements and validators)</li>
 *         <li>actually populate the map (from either a ServletRequest
 *            or a StateMap)</li>
 *         <li>validate the map (by invoking all the validators associated
 *            with the form and all its elements)</li>
 *        <li>provide convenience methods to access the underlying values
 *            of the form elements contained in this map</li>
 * </ul>
 */
public interface FormMap extends StateMap {

    /**
     * This defines an element to be mapped by this form, using 
     * the key from the FormElement. You would invoke this method 
     * for each element in the form.
     *
     * @param element a FormElement to be mapped by this form
     */
    public void defineElement(FormElement element);

    /**
     * This defines an element to be mapped by this form.
     * You would invoke this method for each element in the form.
     *
     * @param key the key which uniquely identifies this FormElement
     * @param element a FormElement to be mapped by this form
     */
    public void defineElement(String key, FormElement element);

    /**
     * This defines a validator for the entire form. This validator 
     * will be invoked prior to validating specific form element 
     * validators. Calling this method multiple times will result 
     * in multiple form validators being added to the form (they 
     * will be invoked in the order they were added)
     *
     * @param validator a form validator to be applied to the entire form
     */
    public void defineValidator(FormValidator validator);

    /**
     * This is where we actually take an incoming form (in
     * the form of a ServletRequest) and map it using the 
     * definitions supplied by all the FormElements. If there 
     * are multiple parameters for a given key, the values 
     * will be mapped into List structures
     *
     * @param req the ServletRequest to map paramters from based on
     *        all defined FormElements
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws MappingException if for some reason the value cannot 
     *        be mapped successfully
     */
    public FormMap map(ServletRequest req);

    /**
     * This is where we actually take an incoming form (in
     * the form of a StateMap) and map it using the definitions 
     * supplied by all the FormElements.
     *
     * @param map the StateMap to map properties from based on
     *        all defined FormElements
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws MappingException if for some reason the value cannot 
     *        be mapped successfully
     */
    public FormMap map(StateMap map);

    /**
     * Validate the entire form (both form level and elements). 
     * We start by invoking form validators which apply to individual 
     * form elements, then we invoke any which apply to the entire form
     *
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws ValidationException if the form (or any element within it)
     *        is invalid
     */
    public FormMap validate(boolean deferExceptions) throws ValidationException;

    /**
     * Validate just the elements (not the form)
     *
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws ValidationException if the form (or any element within it)
     *        is invalid
     */
    public FormMap validateElements(boolean deferExceptions) throws ValidationException;
    
    /**
     * Validate just the form (not the individual elements)
     *
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @return a reference to the FormMap (we do this so you can inline
     *        map/validate requests if you desire: form.map(req).validate())
     * @throws ValidationException if the form (or any element within it)
     *        is invalid
     */
    public FormMap validateForm(boolean deferExceptions) throws ValidationException;

    /**
     * Return true if an element exists (not null)
     *
     * @param key the key which uniquely identifies this FormElement
     * @return true if an element exists (not null)
     */
    public boolean exists(String key);
    
    /**
     * Get an element by key
     *
     * @param key the key which uniquely identifies this FormElement
     * @return the FormElement for this key (may be null)
     */
    public FormElement getElement(String key);
    
    /**
     * return a map with containing all the elements in this form map
     *
     * @return a copy of the Map that backs this FormMap
     */
    public Map getElements();

    /**
     * Manually set the value of an element. If an element does not 
     * exist for this key one will be created.
     *
     * @param key the key
     * @param val the value for the key
     */
    public void setVal(String key, Object val);

    /**
     * Get the value for a given key. This is basically a convenience
     * method. You could manually grab the FormElement using getElement
     * and then retrieve the value that way as well.
     *
     * @param key the key which uniquely identifies this FormElement
     * @return the value for this key (may be null)
     */
    public Object getVal(String key);

    /**
     * Get a String value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
//    public String getStringVal(String key);

    /**
     * Get the value for a given key, , defaulting accordingly if
     * the value is null. This is basically a convenience method. 
     * You could manually grab the FormElement using getElement
     * and then retrieve the value that way as well.
     *
     * @param key the key which uniquely identifies this FormElement
     * @param dflt the default value to be used if the underlying value is null
     * @return the value for this key (may be null)
     */
//    public String getStringVal(String key, String dflt);

    /**
     * Get an Boolean value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
//    public Boolean getBooleanVal(String key);

    /**
     * Get an Integer value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
//    public Integer getIntegerVal(String key);

    /**
     * Get an Date value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
//    public Date getDateVal(String key);

    /**
     * Get an Long value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
//    public Long getLongVal(String key);

    /**
     * Get an Short value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
//    public Short getShortVal(String key);

    /**
     * Get an Double value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
//    public Double getDoubleVal(String key);

    /**
     * Get an Float value from the map
     *
     * @param key the form element key
     * @return the value for the given key (may be null
     *        if the value is not set or the key does not match 
     *        a known form element)
     */
//    public Float getFloatVal(String key);

    /**
     * @clientCardinality 0..* 
     */
    /*#FormElement lnkFormElement;*/
}
