/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: FormType.java,v 1.12 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;
import java.text.*;

/**
 * This class defines all valid FormTypes. Currently we support:
 *
 * <ul>
 *        <li>String</li>
 *        <li>Boolean</li>
 *        <li>Integer</li>
 *        <li>Date</li>
 *        <li>Long</li>
 *        <li>Short</li>
 *        <li>Double</li>
 *        <li>Float</li>
 * </ul>
 */
public final class FormType {

    //public vars
    public static FormType STRING = new FormType(String.class);
    public static FormType BOOLEAN = new FormType(Boolean.class);
    public static FormType INTEGER = new FormType(Integer.class);
    public static FormType DATE = new FormType(java.util.Date.class);
    public static FormType LONG = new FormType(Long.class);
    public static FormType SHORT = new FormType(Short.class);
    public static FormType DOUBLE = new FormType(Double.class);
    public static FormType FLOAT = new FormType(Float.class);

    //private vars
    private Class formClass = null;

    /**
     * Private constructor to prevent external instantiation
     */
    private FormType(Class iformClass) {
        formClass = iformClass;
    }

    /**
     * Get the class associated with this particular
     * FormType
     */
    public Class getFormClass() {
        return formClass;
    }
    
    /**
     * Attempt to parse an object based on the specific form type.
     */
    public Object parse(String origVal) throws ParseException {
        //eliminate the obvious
        if (origVal==null) return null;

        //try and parse the data    
        Object val = null;
        try {
            //...Strings
            if (this==FormType.STRING) {
                val = origVal;
                
            //...Booleans (we can handle "true", "on", or "yes"
            } else if (this==FormType.BOOLEAN) {
                String tval = origVal.trim().toLowerCase();
                if (tval.equals("on") ||
                    tval.equals("yes") ||
                    tval.equals("true")) {
                    origVal = "true";
                } else if (tval.equals("off") ||
                    tval.equals("no") ||
                    tval.equals("false")) {
                    origVal = "false";
                } else {
                    throw new ParseException(origVal);
                }
                val = new Boolean(origVal);
            //...Integers
            } else if (this==FormType.INTEGER) {
                try {
                    val = new Integer(origVal);
                } catch (NumberFormatException e) {
                    //this basically handles the case where the user
                    //typed in an integer value like 123.00...we convert
                    //that value to a double, then to an int, and then form
                    //a new double from that. If the values are equal, we
                    //know no roundoff occurred, meaning the decimal places
                    //were all zeros, and we're in business. Otherwise, throw
                    //the exception
                    Double d1 = new Double(origVal); 
                    int d1val = d1.intValue();
                    Double d2 = new Double(d1val);
                    if (d1.equals(d2)) {
                        val = new Integer(d1val);
                    } else  {
                        throw e;
                    }
                }
            //...Dates
            } else if (this==FormType.DATE) {

/*
//rtl20010822 - start
                DateFormat df = DateFormat.getDateInstance();
                df.setLenient(false);
                val = df.parse(origVal);
//rtl20010822 - end
*/

                //rtl20010822 - start new
                // try all the various forms of date format to cover more bases
                try {
                    DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
                    df.setLenient(false);
                    val = df.parse(origVal);
                } catch (java.text.ParseException e1) {
                    try {
                        DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM);
                        df.setLenient(false);
                        val = df.parse(origVal);
                    } catch (java.text.ParseException e2) {
                        try {
                            DateFormat df = DateFormat.getDateInstance(DateFormat.LONG);
                            df.setLenient(false);
                            val = df.parse(origVal);
                        } catch (java.text.ParseException e3) {
                            DateFormat df = DateFormat.getDateInstance(DateFormat.FULL);
                            df.setLenient(false);
                            val = df.parse(origVal);
                        }
                    }
                    //rtl20010822 - end new

                }
            //...Longs
            } else if (this==FormType.LONG) {
                val = new Long(origVal);
            //...Shorts
            } else if (this==FormType.SHORT) {
                val = new Short(origVal);
            //...Doubles
            } else if (this==FormType.DOUBLE) {
                val = new Double(origVal);
            //...Floats
            } else if (this==FormType.FLOAT) {
                val = new Float(origVal);
            }
        } catch (java.text.ParseException e) {
            throw new ParseException(e, "Locale is " + Locale.getDefault().getCountry());
        } catch (NumberFormatException e) {
            throw new ParseException(e);
        }

        return val;            
    }
    
    public String toString() {
        return formClass.getName();    
    }
}
