/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Or.java,v 1.6 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator ensures that at least one of two validators is valid,
 * (effectively acting as an "OR" or "||") or a ValidationException will
 * be generated
 */
public class Or extends AbstractFormValidator {

    protected FormValidator fv1 = null;
    protected FormValidator fv2 = null;
    
    /**
     * Public constructor.
     *
     * @param fv1 the first validator
     * @param fv2 the second validator
     */
    public Or(FormValidator ifv1, FormValidator ifv2) {
        this(ifv1, ifv2, null);
    }

    /**
     * Public constructor.
     *
     * @param fv1 the first validator
     * @param fv2 the second validator
     * @param errmsg the message associated with this error
     */
    public Or(FormValidator ifv1, FormValidator ifv2, String ierrmsg) {
        setErrorMessage(ierrmsg);
        fv1 = ifv1;
        fv2 = ifv2;
    }

    /**
     * Rather than calling all the sub validate methods, we make sure
     * at least one of the sub-validators is valid
     *
     * @param element the form element to be validated (null indicates
     *        we want to perform form level validation)
     * @param map the map to which the element belongs (sometimes necessary
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validate(FormElement element, FormMap map, boolean deferExceptions) throws ValidationException {
        if (localLogger.isDebugEnabled()) localLogger.debug("Making sure "+fv1+" and "+fv2+" are both valid");

        //if both are null
        if (fv1==null && fv2==null) {
            throw this.generateException(element, deferExceptions, "Validators 1 & 2 were both null");
        }

        //validate fv1
        ValidationException ve1 = null;
        try {
            fv1.validate(element, map, deferExceptions);
        } catch (ValidationException ve) {
            ve1 = ve;
        }

        //validate fv2
        ValidationException ve2 = null;
        try {
            fv2.validate(element, map, deferExceptions);
        } catch (ValidationException ve) {
            ve2 = ve;
        }

        //now evaluate
        if (ve1!=null && ve2!=null) {
            ValidationException veNew = this.generateException(element, deferExceptions, "Both Validators 1 & 2 were invalid");
            veNew.addSubException(ve1);
            veNew.addSubException(ve2);
            throw veNew;
        }
    }
}
