/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DateRangeValidator.java,v 1.1 2001/08/16 23:27:19 robertl Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator ensures that a value is within a given date range
 */
public class DateRangeValidator extends DefaultFormValidator {

	protected Date myStartDate = null;
	protected Date myEndDate = null;

	/**
	 * Public no-args constructor.
	 */
	public DateRangeValidator() {
		this(null, null, null);
	}

	/**
	 * Public constructor.
	 *
	 * @param theStartDate the low end of the range
	 * @param theEndDate the high end of the range
	 */
	public DateRangeValidator(Date theStartDate, Date theEndDate) {
		this(theStartDate, theEndDate, null);
	}

	/**
	 * Public constructor.
	 *
	 * @param theStartDate the low end of the range
	 * @param theEndDate the high end of the range
	 * @param theErrMsg the message associated with this error
	 */
	public DateRangeValidator(Date theStartDate, Date theEndDate, String theErrMsg) {
		super(theErrMsg);
		myStartDate = theStartDate;
		myEndDate = theEndDate;
	}

	/**
	 * Validate a FormElement to make see if the element equals() a
	 * given object
	 *
	 * @param element the form element to be validated
	 * @param element the form element that contains the val
	 *		to validate elements by comparing them with other elements)
	 * @param deferExceptions do we want to deferValidation exceptions
	 *		and attempt to validate all elements so that we can process
	 *		all the exceptions at once
	 * @throws ValidationException if the element is not valid
	 */
	public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
		//eliminate the obvious
		if (myStartDate==null && myEndDate==null) {
			throw this.generateException(element,
																	 deferExceptions,
																	 "Invalid range:"+myStartDate+" to "+myEndDate);			
		}
		// cannot validate a null or empty string value, use NotNullvalidator if you want to disallow nulls
		if ( val==null || val.toString().length() == 0 ) return;

		//figure out what type of element we're dealing with
		if (localLogger.isInfoEnabled()) localLogger.info("validating val="+val+" is in range between "+myStartDate+" and "+myEndDate);
		FormType formType = element.getType();

		//if element is null, or not form type=STRING or DATE, err because this should
		//not be considered a valid option (note that these exceptions are always
		//immediate since they typically represent a programming error)
		if ( element==null ) throw new ValidationException(val, "Object val:"+val+" is associated with a null FormElement");
		if ( !(formType.equals(FormType.DATE) || formType.equals(FormType.STRING)) ) {
			throw new ValidationException(val, "Unsupported validation: "+
																		val+" is of FormType " +
																		formType.toString() +
																		" and cannot be validated by this validator");
		}

		Date dateVal = null;

		if ( formType.equals(FormType.DATE) ) {
			try {
				dateVal = (Date)val;
			} catch ( ClassCastException ex ) {
				throw this.generateException(element,
																		 deferExceptions,
																		 val.toString() +  " is not a valid date class");
			}
		} else {
			// attempt to convert the string to a date
			try {
				dateVal = (Date)(FormType.DATE.parse( val.toString() ));

			} catch ( ParseException ex ) {
				throw this.generateException(
					element,
          deferExceptions,
          val.toString() +  " is not a valid date string");
			}
		}

		String defaultErrMsg = null;
		if (myEndDate==null) {
			defaultErrMsg = "Value: "+val+ " is not after "+this.myStartDate;
		} else if (myStartDate==null) {
			defaultErrMsg = "Value: "+val+ " is not before "+this.myEndDate;
		} else {
			defaultErrMsg = "Value: "+val+ " is not in the range between "+this.myStartDate+" and "+this.myEndDate;
		}

		if ( null != this.myStartDate && dateVal.before(this.myStartDate) ) {
			throw this.generateException(element, deferExceptions, defaultErrMsg);
		}

		if ( null != this.myEndDate && dateVal.after(this.myEndDate) ) {
			throw this.generateException(element, deferExceptions, defaultErrMsg);
		}

	}
	
}
