package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator ensures that the value contains only digits tested against
 * the field size length (len). If the value's length is not equal to the length
 * or a character other than the numbers 0-9 is found, an exception will be throw.
 *
 * <p>This class can be used 3 ways
 * <p>1) As a digit validator, confirming that only digits exist for the value. The field value
 * "is not required", and will only be tested for digits when the field value "is not null".
 * Use default constructor() or constructor(errorMessage) for this case.
 * <p>2) As a digit/length validator where phoneNumber will default to false. This can be used
 * to test set lengths of digits (credit cards, quantities, soc sec numbers etc.) The field value "is required".
 * Use constructor(len) for this case.
 * <p>3) As a phone number validator (domestic/international) where certain phone number rules apply. Use
 * constructor(len, isDomesticPhone, errMessage) for this case.
 *
 * <p>Note: There is no current support for  '.' or '-' in the value for phone numbers.
 *
 * <p>This class works very nicely where the input fields are separated into their respective
 * types (i.e Separating a Domestic Phone Number into 3 'input type=text' boxes -- Area Code len=3,
 * Prefix len = 3, and Number len =4) will permit easier implementation of the rules as more are
 * added.
 *
 * <p><p>Rules:<br>
 * --domesticPhone = true<br>
 * if length = 3, then the value may not start with a zero (0)
 *
 * <br>
 * --domesticPhone = false (international)<br>
 * none
 *
 * @author Paul G. Markovich [pmarkovich@acpinteractive.com]
 */
public class DigitValidator extends DefaultFormValidator {

    protected int len = 0;
    protected boolean requireLength = false;
    protected boolean phoneNumber = false;
    protected boolean domesticPhone = false;

    /**
     * Public constructor.
     */
    public DigitValidator() {
        this(null);
    }

    /**
     * Public constructor.
     *
     * @param errmsg the message associated with this validation error
     */
    public DigitValidator(String ierrmsg) {
        super(ierrmsg);
    }

    /**
     * Public constructor.
     *
     * Utilizing this constructor, the field value will be required
     *
     * @param len the length of the field to validate all digits
     */
    public DigitValidator(int ilen) {
        //pgm changed from: this(ilen,false,null);
        this(ilen, null);
        requireLength = true;
    }

    /**
     * Public constructor.
     *
     * @param len the length of the field to validate all digits
     * @param errmsg the message associated with this validation error
     */
    public DigitValidator(int ilen, String ierrmsg) {
        //pgm changed from: this(ilen,false,errmsg);
        super(ierrmsg);
        len = ilen;
        requireLength = true;
    }

    /**
     * Public constructor.
     *
     * @param len the length of the field to validate all digits
     * @param domesticPhone when true will validate based on domestic
     *        rules. At this time the only rule is for a length
     *        of 3 (area codes/prefixes) that the value can't start with 0
     * @param errmsg the message associated with this validation error
     */
    public DigitValidator(int ilen, boolean idomesticPhone, String ierrmsg) {
        super(ierrmsg);
        len = ilen;
        domesticPhone = idomesticPhone;
        phoneNumber = true;
        requireLength = true;
    }

    /**
     * Validate a FormElement to make see if the element equals() a
     * given object
     *
     * @param element the form element to be validated
     * @param element the form element that contains the val
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        //pgm moved the following code from here to the else condition below
        //test to see if field value is null, if so no further tests required
        //if (val==null || val.toString().trim().length()<1) {
            //if (errmsg==null) errmsg = "Null value fails to meet minimum length of "+len;
            //throw this.generateException(element, deferExceptions);
        //}
        
        String s = null;
        char c[] = null;
        if (val!=null) {
            s = val.toString().trim();
            c = s.toCharArray();
        }
        
        //Field Value is not required:
        //  only test when field value is not null, confirm that field value contains 
        //  only digits 0-9, from: contructor()/constructor(errorMessage)
        if (!requireLength) { 
            if (s!=null && s.length()>0) {
                //validate that val is made up of only characters 0-9
                if (localLogger.isInfoEnabled()) localLogger.info("validating to see if val {"+s+"} contains only the numbers 0-9");
                for (int i=0; i<c.length; i++) {
                    if ((c[i] < '0') || (c[i] > '9'))  {
                        throw this.generateException(element, deferExceptions, "Val {"+val+"} contains characters other than 0123456789");
                    }
                }
            }

        //Field Value is required:
        //  field value tests for null/length of field matches requested length/ and/or 
        //  phone number requirement tests
        } else {
            //test to see if field value is null, if so no further tests required
            if (s==null || s.length()<1) {
                throw this.generateException(element, deferExceptions, "Null value fails to meet minimum length of "+len);
            }

            //validate that the length of val = len
            if (localLogger.isInfoEnabled()) localLogger.info("validating to see if length of val {"+s+"} = "+len);
            if (s.length()!=len) {
                throw this.generateException(element, deferExceptions, "Length of val {"+val+"} is not equal to the field length of "+len);
            }

            //validate that val is made up of only characters 0-9
            if (localLogger.isInfoEnabled()) localLogger.info("validating to see if val {"+s+"} contains only the numbers 0-9");
            for (int i=0; i<c.length; i++) {
                if ((c[i] < '0') || (c[i] > '9'))  {
                    throw this.generateException(element, deferExceptions, "Val {"+val+"} contains characters other than 0123456789");
                }
            }

            //phone number validations
            if (phoneNumber) {
                //Domestic 
                if (domesticPhone) {
                    if (localLogger.isInfoEnabled()) localLogger.info("Validating to see if val {"+s+"} for domestic phone requirments");
                    if ((c.length==3) && (c[0]=='0'))  {
                        throw this.generateException(element, deferExceptions, "Val {"+val+"} starts with a zero for a length of 3");
                    }
                //International
                } else {
                    if (localLogger.isInfoEnabled()) localLogger.info("Validating val {"+s+"} for international phone requirments");
                    //todo - we don't do anything special for international numbers now
                }
            }
        }
    }
}
