/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: MaxLengthValidator.java,v 1.7 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator ensures that the length of a value is <= a maximum length
 */
public class MaxLengthValidator extends DefaultFormValidator {

    protected int max = 0;

    /**
     * Public no-args constructor.
     */
    public MaxLengthValidator() {
        this(0, null);
    }

    /**
     * Public constructor.
     *
     * @param max the max length
     */
    public MaxLengthValidator(int imax) {
        this(imax, null);
    }

    /**
     * Public constructor.
     *
     * @param max the max length
     * @param errmsg the message associated with this error
     */
    public MaxLengthValidator(int imax, String ierrmsg) {
        super(ierrmsg);
        max = imax;
    }

    /**
     * Validate a FormElement to make sure the length of element does not
     * exceed a Max length. Validation is not supported for FormType.BOOLEAN
     * and FormType.DATE
     *
     * @param element the form element to be validated
     * @param element the form element that contains the val
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        // a null value will always be less than the maximum length
        if (val==null) return;
        
        //if element is null, or form type=BOOLEAN or DATE, err because this should
        //not be considered a valid option (note that these exceptions are always
        //immediate since they typically represent a programming error)
        if (element==null) throw new ValidationException(val, "Object val:"+val+" is associated with a null FormElement");
        if (element.getType().equals(FormType.BOOLEAN)) throw new ValidationException(val, "Unsupported validation: "+val+" is of FormType.BOOLEAN and cannot be validated by this validator");
        if (element.getType().equals(FormType.DATE)) throw new ValidationException(val, "Unsupported validation: "+val+" is of FormType.DATE and cannot be validated by this validator");

        //validate the length <= max
        String s = val.toString();
        if (localLogger.isInfoEnabled()) localLogger.info("validating to see if length of val {"+s+"} <= "+max);
        if (s.length()>max) {
            throw this.generateException(element, deferExceptions, "Length of val {"+val+"} exceeds maximum length of "+max);
        }
    }
    
}
