/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ResourceGateway.java,v 1.6 2001/10/02 21:12:13 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.helper.servlet;

import java.io.*;
import java.util.*;
import java.lang.ref.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

//import org.enhydra.barracuda.core.util.dom.*;
//import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.helper.state.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;

/**
 * <p>The purpose of this servlet is to look for a static resource
 * on the classpath and return it to the client
 */
public class ResourceGateway extends HttpServlet {

    //public vars...eventually, these should probably be final
    protected static BCategory logger = BCategory.getBInstance(ResourceGateway.class.getName());

    //these are intentionally non-final, so that you can programatically
    //change them if need be
    public static String EXT_RESOURCE_ID = "xlib";
    public static final String RESOURCE_NOT_FOUND = "ResourceNotFound";
    
    protected ServletConfig config = null;

    //-------------------- ResourceGateway -----------------------    
    /**
     * <p>Handle the default HttpRequest. 
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefault(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //figure out what resource we're looking for
        String resourceName = "/"+EXT_RESOURCE_ID + req.getPathInfo();
        if (logger.isDebugEnabled()) logger.debug("Looking for static resource:"+resourceName);

        //get the object from cache
        ReferenceFactory rf = new LocalReferenceFactory(resourceName);
        Object resource = ContextServices.getObjectFromCache(config.getServletContext(), resourceName, rf);

        //check the results
        //...byte array
        if (resource instanceof LocalResource) {

            //set the content type
            if (logger.isDebugEnabled()) logger.debug("Got resource from cache");
            LocalResource lr = (LocalResource) resource;
            resp.setContentType(lr.contentType);
            if (logger.isDebugEnabled()) logger.debug("Set content-type: "+lr.contentType);

            //now return the requested document
            OutputStream out = resp.getOutputStream();
//            byte[] bytes = lr.contentData.getBytes();
            byte[] bytes = lr.contentData;
            out.write(bytes);
            out.flush();
            resp.flushBuffer();
            if (logger.isDebugEnabled()) logger.debug("Wrote data: "+bytes.length+" bytes");
                        
        //...resource not found
        } else {
            if (logger.isDebugEnabled()) logger.debug("Resource not found");
            resp.sendError(HttpServletResponse.SC_NOT_FOUND, "Resource Not Found: "+resourceName);
        }
    }

    /**
     * Used to create a SoftReference to a specified resource
     */
    class LocalReferenceFactory implements ReferenceFactory {
        String resourceName = null;

        public LocalReferenceFactory(String iresourceName) {
            resourceName = iresourceName;
        }
        
        public Reference getObjectReference() {
            if (logger.isDebugEnabled()) logger.debug("Resource not in cache...will load from classpath");
            
            //figure out the content type (see: http://www.isi.edu/in-notes/iana/assignments/media-types/media-types for source list)
            String contentType = config.getServletContext().getMimeType(resourceName);
            if (contentType==null) contentType = "text/plain";
        
            //get the resource
            if (logger.isDebugEnabled()) logger.debug("Attempting to read: "+resourceName);
            Object result = null;
            byte[] dstbytes = new byte[10240];
            try {
                //read in the file and store it in a byte array
                BufferedInputStream in = new BufferedInputStream(this.getClass().getResourceAsStream(resourceName));
                int offset = 0;
                int len = 1024;
                byte[] inbytes = new byte[len];
                while (true) {
                    int cnt = in.read(inbytes, 0, len);
                    if (cnt==-1) break;
                    if (offset+cnt>=dstbytes.length) {
                        byte[] newbytes = new byte[(int) (dstbytes.length+(len*5))];
                        System.arraycopy(dstbytes, 0, newbytes, 0, dstbytes.length);
                        dstbytes = newbytes;
                    }
                    System.arraycopy(inbytes, 0, dstbytes, offset, cnt);
                    offset+=cnt;
                }
                //merg_100201.1 - Added a close() statement after reading the resource 
                //from the inputstream. I noticed that I couldn't redeploy the app when 
                //it used the ResourceGateway. This simple .close() fixes it.
                in.close();
                
                //when we're done trim off the excess
                byte[] newbytes = new byte[offset];
                System.arraycopy(dstbytes, 0, newbytes, 0, offset);
                dstbytes = newbytes;
                                     
                //once we're done reading, create a local resource object
                if (logger.isDebugEnabled()) logger.debug("Success!");
                LocalResource lr = new LocalResource();
                lr.contentType = contentType;
                lr.contentData = dstbytes;
                result = lr;
            } catch (IOException e) {
                if (logger.isDebugEnabled()) logger.debug("Failure: "+e);
                result = RESOURCE_NOT_FOUND;
            }
            return new SoftReference(result);
        }
    }

    /**
     * The actual resource that gets cached
     */
    class LocalResource {
        public String contentType = null;
        public byte[] contentData = null;
    }

    //-------------------- HTTPServlet ---------------------------
    /**
     * <p>By default the GET request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the POST request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the OPTIONS request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doOptions(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the DELETE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the PUT request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPut(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the TRACE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doTrace(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {  
        handleDefault(req, resp);
    }


    //-------------------- Servlet -------------------------------
    /**
     * <p>Here's where we initialize the servlet.
     *
     * @param config the servlet config object this servlet is
     *        being initialized with
     */
    public void init(ServletConfig iconfig) throws ServletException {
        logger.info("initializing servlet");

        //save a reference to the config object
        config = iconfig;
    }

}
