/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultStateMap.java,v 1.3 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.io.*;
import java.util.*;

import org.enhydra.barracuda.core.util.logging.*;

/**
 * <p>A StateMap is an object that is capable of carrying state 
 * information along with it--you can put properties into the 
 * state and then get them back out.
 *
 * <p>This class provides the default implementation of a StateMap. 
 * The underlying storage structure is a HashMap, so it a) isn't 
 * threadsafe and b) accepts nulls. If you need more than the minimal 
 * functionality exposed in the StateMap interface, you should work 
 * with the underlying Map structure.
 *
 * <p>Key entities that implement StateMap:
 * <ul>
 *        <li>BaseEvent</li>
 *        <li>EventContext</li>
 *        <li>FormMap</li>
 * </ul>
 */
public class DefaultStateMap implements StateMap {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(DefaultStateMap.class.getName());

    //private vars
    protected Map props = null;                    //private property map

    /**
     * set a property in this StateMap
     *
     * @param key the key object
     * @param val the value object
     */    
    public void putState(Object key, Object val) {
        if (props==null) props = new HashMap();
        if (logger.isDebugEnabled()) logger.debug("Setting queue state: [key]:"+key+" [val]:"+val);
        props.put(key,val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        if (props==null) return null;
        else return props.get(key);
    }
    
    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        if (props==null) return null;
        else return props.remove(key);
    }
    
    /**
     * get a List of the keys for this StateMap (implementation
     * is an ArrayList)
     *
     * @return a List the keys for this StateMap
     */    
    public List getStateKeys() {
        if (props==null) return null;
        else return new ArrayList(props.keySet());
    }
    
    /**
     * get a copy of the underlying Map that holds 
     * the state values
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateValues() {
        if (props==null) props = new HashMap();
        return new HashMap(props);
    }

}
