/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultDOMLoader.java,v 1.6 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;

import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * Default implementation of a DOMLoader. 
 *
 * TODO - what about thread safety?
 */
public class DefaultDOMLoader implements DOMLoader {

    protected static BCategory logger = BCategory.getBInstance(DefaultDOMLoader.class.getName());

    protected static DefaultDOMLoader globalLoader = new DefaultDOMLoader();

    protected DOMFactory defaultDOMFactory = new DefaultDOMFactory();
    protected Map factories = new HashMap();
    protected Map classmap = new HashMap();
    protected Object sync = new Object();
    
    /**
     * Get a DOM associated with a given class of object.
     *
     * @param cl the class of DOM object
     * @return a DOM for the given class of object.
     * @throws IOException if the underlying DOMFactory that services this 
     *        request cannot handle this particular class
     */
    public Document getDOM(Class cl) throws IOException {
        return getDOM_private(cl, null, null);
    }
    
    /**
     * Get a DOM associated with a given class of object, based on
     * the specified locale.
     *
     * @param cl the class of DOM object
     * @param locale the target locale 
     * @return a DOM for the given class of object.
     * @throws IOException if the underlying DOMFactory that services this 
     *        request cannot handle this particular class
     */
    public Document getDOM(Class cl, Locale locale) throws IOException {
        return getDOM_private(cl, locale, null);
    }
    
    /**
     * Get a DOM associated with a given class of object, based on the
     * specified ViewCapabilities
     *
     * @param cl the class of DOM object
     * @param vc the clients ViewCapabilities (may be null)
     * @return a DOM for the given class of object.
     * @throws IOException if the underlying DOMFactory that services this 
     *        request cannot handle this particular class
     */
    public Document getDOM(Class cl, ViewCapabilities vc) throws IOException {
        return getDOM_private(cl, null, vc);
    }
    
    private Document getDOM_private(Class cl, Locale locale, ViewCapabilities vc) throws IOException {
        //eliminate the obvious
        if (cl==null) throw new IOException("Invalid class:"+cl);
        
        //if the locale is null, look to the ViewCapabilities
        if (locale==null && vc!=null) {
            locale = vc.getClientLocale();
            if (logger.isDebugEnabled()) logger.debug("Retrieving locale from ViewCapabilities: "+locale);
        }
        
        //if the locale is still null, just use the default
        if (locale==null) {
            locale = Locale.getDefault();
            if (logger.isDebugEnabled()) logger.debug("Using default locale: "+locale);
        }

        //first we need to figure out the real class name (taking into account
        //the locale). To do this, we must understand the naming convention. As
        //with resource bundles, we'll look for class name + locale. For example,
        //if we request HelloWorldHTML.class, but the locale is Spanish (es), 
        //then we should get back an instance of the DOM for HelloWorldHTML_es.class, 
        //unless of course that class does not exist, in which case it should return 
        //the DOM for HelloWorld.class. In this way, the DOM classes themselves act 
        //like resource bundles.
        String className = cl.getName();
//        if (cl.isInterface()) className = className+"Impl";    //added csc_081501 - this handles the situation where the target class name is actually an interface; in this case look for the implementation class instead.
        String baseName = className;
        Class targetClass = null;
        String targetName = null;
        String language = locale.getLanguage();
        String country = locale.getCountry();
        String variant = locale.getVariant();
        targetName = baseName+"_"+language+"_"+country+"_"+variant;
        targetClass = lookupClass(targetName);
        if (targetClass==null) {
            targetName = baseName+"_"+language+"_"+country;
            targetClass = lookupClass(targetName);
        }
        if (targetClass==null) {
            targetName = baseName+"_"+language;
            targetClass = lookupClass(targetName);
        }
        if (targetClass==null) {
            targetName = baseName;
            targetClass = lookupClass(targetName);
        }
        if (logger.isDebugEnabled()) logger.debug("Target class: "+targetClass);
        if (targetClass==null) return null;

        //at this point we should have a valid class; now we 
        //need to create a DOM for it
        synchronized (sync) {
            if (logger.isDebugEnabled()) logger.debug("Loading DOM");
            DOMFactory df = (DOMFactory) factories.get(targetClass);
            if (df!=null) return df.getInstance(targetClass);
            else return defaultDOMFactory.getInstance(targetClass);
        }
    }
    
    protected synchronized Class lookupClass(String className) {
        //first see if we have a reference to that class in our classmap
        Class cl = (Class) classmap.get(className);        
        
        //if not, try to instantiate it
        if (cl==null) {
            try {
                cl = Class.forName(className);
                classmap.put(className, cl);
            } catch (Exception e) {}         
        }
        return cl;
    }
    
    
    /**
     * Specify the default DOM factory.
     *
     * @param df the DOMFactory
     */
    public void setDefaultDOMFactory(DOMFactory df) {
        if (logger.isDebugEnabled()) logger.debug("Setting default DOM factory:"+df);
        synchronized (sync) {
            defaultDOMFactory = df;
        }
    }

    /**
     * Register a DOMFactory for a given class of DOM object.
     *
     * @param df the DOMFactory
     * @param cl the class of DOM object to be handled by this factory
     */
    public void registerDOMFactory(DOMFactory df, Class cl) {
        if (logger.isDebugEnabled()) logger.debug("Registering DOM factory:"+df+" for class:"+cl);
        synchronized (sync) {
            factories.put(cl, df);
        }
    }
    
    /**
     * Deregister a DOMFactory for a given class of DOM object.
     *
     * @param df the DOMFactory
     * @param cl the class of DOM object to be handled by this factory
     */
    public void deregisterDOMFactory(Class cl) {
        if (logger.isDebugEnabled()) logger.debug("Deregistering DOM factory for class:"+cl);
        synchronized (sync) {
            factories.remove(cl);
        }
    }
    
    /**
     * Get the global instance of the DOMLoader
     *
     * @return the global instance of the DOMLoader
     */
    public static DOMLoader getGlobalInstance() {
        return globalLoader;
    }
    
/*    
    public static void main(String args[]) {
        DOMLoader loader = DefaultDOMLoader.getGlobalInstance();
        Class targetCl = org.enhydra.barracuda.tutorials.xmlc.HelloWorld4HTML.class;
        Document dom = null;
        
        try {
            //load the default
            dom = loader.getDOM(targetCl);
            System.out.println ("DOM:"+dom);
            
            //load Spanish
            dom = loader.getDOM(targetCl, new Locale("es",""));
            System.out.println ("DOM:"+dom);
            
            //load Spanish (Chile)
            dom = loader.getDOM(targetCl, new Locale("es","CL"));
            System.out.println ("DOM:"+dom);
            
            //load French
            dom = loader.getDOM(targetCl, new Locale("fr",""));
            System.out.println ("DOM:"+dom);
            
            //load Arabic (should return default)
            dom = loader.getDOM(targetCl, new Locale("ar",""));
            System.out.println ("DOM:"+dom);
            
        } catch (Exception e) {
            System.out.println ("Err:"+e);
        }
    }    
*/    
}