/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultDOMWriter.java,v 1.7 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.w3c.tidy.Tidy;

import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.xml.dom.DOMOps;
import org.enhydra.xml.dom.DOMAccess;
import org.enhydra.barracuda.core.util.dom.io.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.logging.*;


/**
 * This class provides the default implementation for writing a DOM.
 * Note that it supports pretty printing (although the default is 
 * false).
 */
public class DefaultDOMWriter implements DOMWriter {

    protected static BCategory logger = BCategory.getBInstance(DefaultDOMWriter.class.getName());
    public static boolean defaultPrintPretty = false;
    public static boolean defaultPreventCaching = false;

    protected DOMFormatter dfm = null;
    protected OutputOptions doo = null;
    protected boolean printPretty = false;
    protected boolean preventCaching = false;

    private static byte[] sep = System.getProperty("line.separator").getBytes();
    private static String spaces = "                                                                              ";

    /**
     * Default constructor
     */
    public DefaultDOMWriter() {
        this(defaultPrintPretty);    
    }
    
    /**
     * Public constructor. Allows you to specify pretty printing.
     *
     * @param true if pretty print
     */
    public DefaultDOMWriter(boolean printPretty) {
        this(printPretty, defaultPreventCaching);
    }
    
    /**
     * Public constructor. Allows you to specify pretty printing and
     * whether or not the page should be cached.
     *
     * @param true if pretty print
     * @param true if we want to prevent client side caching of the page
     */
    public DefaultDOMWriter(boolean printPretty, boolean preventCaching) {
        setPrettyPrint(printPretty);
        setPreventCaching(preventCaching);
    }
    

    //--------------- DefaultDOMWriter ---------------------------
    /**
     * Write a DOM to a ServletResponse object. This method will 
     * automatically set the content type for you.
     *
     * @param node the DOM node to be written out
     * @param resp the HttpServletResponse object
     */
    public void write(Node node, HttpServletResponse resp) throws IOException {
        write(node, resp, null);
    }

    /**
     * Write a DOM to a ServletResponse object. This method will 
     * automatically set the content type for you.
     *
     * @param node the DOM node to be written out
     * @param resp the HttpServletResponse object
     * @param oo the target OutputOptions object (may be null)
     */
    public void write(Node node, HttpServletResponse resp, OutputOptions oo) throws IOException {
        //set the response type (TODO:need to support WML as well)
        String responseType = "text/xml";
        if (node instanceof HTMLDocument) responseType = "text/html";
        if (logger.isDebugEnabled()) logger.debug("Setting the response type:"+responseType);
        resp.setContentType(responseType); 

        //check to see if the node has the anti-cache marker
        Document doc = null;
        if (node instanceof Document) doc = (Document) node;
        else doc = node.getOwnerDocument();
        Element el = doc.getDocumentElement();
        String cache = el.getAttribute(BComponent.CACHING_MARKER);
        if (cache!=null) {
            if (cache.toLowerCase().equals("true")) preventCaching = true;
            el.removeAttribute(BComponent.CACHING_MARKER);
        }

        //if we need to prevent caching
        if (preventCaching) {
            //add the appropriate headers to the response
            resp.setHeader("Pragma","no-cache");
            resp.setHeader("Cache-Control","no-cache");
            resp.setDateHeader("Expires", System.currentTimeMillis());
        }

        //write the dom
        write(node, resp.getWriter(), oo);
    }

    /**
     * Write a DOM to an OutputStream.
     *
     * @param node the DOM node to be written out
     * @param out the OutputStream to be written to
     */
    public void write(Node node, OutputStream out) throws IOException {
        write(node, out, null);
    }
    
    /**
     * Write a DOM to an OutputStream.
     *
     * @param node the DOM node to be written out
     * @param out the OutputStream to be written to
     * @param oo the target OutputOptions object (may be null)
     */
    public void write(Node node, OutputStream out, OutputOptions oo) throws IOException {
        write(node, new PrintWriter(out), oo);
    }
    
    /**
     * Write a DOM to a Writer.
     *
     * @param node the DOM node to be written out
     * @param writer the writer to be written to
     */
    public void write(Node node, Writer writer) throws IOException {
        write(node, writer, null);
    }
    
    /**
     * Write a DOM to a Writer.
     *
     * @param node the DOM node to be written out
     * @param writer the writer to be written to
     * @param oo the target OutputOptions object (may be null)
     */
    public void write(Node node, Writer writer, OutputOptions oo) throws IOException {
/*
        if (dfm==null) {
            oo = DOMFormatter.getDefaultOutputOptions(node.getOwnerDocument());
            dfm = new DOMFormatter(oo);
        }
*/
        //build the default DOMFormatter if necessary
        if (dfm==null) dfm = new DOMFormatter();

        //set the default output options if necessary        
        if (doo==null) doo = oo;
        if (doo==null) doo = getDefaultOutputOptions(node.getOwnerDocument());
        
        //configure the formatter
        dfm.setOutputOptions(oo!=null ? oo : doo);
        
        //now print...        
        if (!printPretty) {
            //write the doc directly to the writer
            if (logger.isDebugEnabled()) logger.debug("Writing DOM directly to writer");
            dfm.write(node, writer);
            writer.close();
            
        } else {
            //write the doc to a byte stream
            byte[] bytes = dfm.toBytes(node);

            //filter the byte stream to tidy
            if (logger.isDebugEnabled()) logger.debug("Formatting for pretty print");
            Tidy tidy = new Tidy();
            tidy.setIndentContent(true);
            tidy.setSpaces(2);
            tidy.setQuiet(true);
            ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            tidy.parse(bais, baos);
                        
            //now write the file
            if (logger.isDebugEnabled()) logger.debug("Writing DOM to writer");
            writer.write(baos.toString());
            writer.close();
        }
    }







//    public OutputOptions getOutputOptions() {
//        return oo;    
//    }

    public void setPrettyPrint(boolean val) {
        printPretty = val;
    }

    public void setPreventCaching(boolean val) {
        preventCaching = val;
    }

    public static OutputOptions getDefaultOutputOptions(Document doc) {
        return DOMFormatter.getDefaultOutputOptions(doc);    
    }
    
/*
    public static void main(String args[]) {
        //setup
        DOMWriter dw = new DefaultDOMWriter();
        org.enhydra.xml.xmlc.XMLCFactory xmlcFactory = new org.enhydra.xml.xmlc.XMLCStdFactory(dw.getClass().getClassLoader(), new org.enhydra.xml.xmlc.StreamXMLCLogger());
        org.enhydra.barracuda.tutorials.xmlc.Visibility1HTML page1 = null;        
        org.enhydra.barracuda.tutorials.xmlc.Visibility2HTML page2 = null;        
        StringWriter sw = null;

        //load & print DOM page
System.out.println ("");        
System.out.println ("Here 0");        
        page1 = (org.enhydra.barracuda.tutorials.xmlc.Visibility1HTML) xmlcFactory.create(org.enhydra.barracuda.tutorials.xmlc.Visibility1HTML.class);
        page1.getElementItem2().setAttribute("visdom","false");
        page1.getElementItem4().setAttribute("visdom","false");
        sw = new StringWriter(5000);
        try {
            dw.write(page1, sw);
        } catch (IOException e){
            System.out.println("Error writing DOM:"+e);
        }
        System.out.println(sw.toString());

        //load & print DOM page
System.out.println ("");        
System.out.println ("Here 1");        
        page2 = (org.enhydra.barracuda.tutorials.xmlc.Visibility2HTML) xmlcFactory.create(org.enhydra.barracuda.tutorials.xmlc.Visibility2HTML.class);
        page2.getElementItem2().setAttribute("visdom","false");
        page2.getElementItem4().setAttribute("visdom","false");
        sw = new StringWriter(5000);
        try {
            dw.write(page2, sw);
        } catch (IOException e){
            System.out.println("Error writing DOM:"+e);
        }
        System.out.println(sw.toString());
    }

*/

}

