/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ContextServices.java,v 1.2 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.http;

import java.lang.ref.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.data.ReferenceFactory;
import org.enhydra.barracuda.core.util.logging.*;


/**
 * This class defines a convenience method to get the servlet
 * context. It also provides a mechanism to easily cache objects 
 * in the session by Reference (which allows them to automatically 
 * be removed from the session if the system starts running low on memory)
 */
public class ContextServices {

    //constants
    public static final String KEY = ContextServices.class.getName()+".Key";

    protected static BCategory logger = BCategory.getBInstance(ContextServices.class.getName());

    /**
     * get the servlet context from a ViewContext
     *
     * @param vc the ControlEventContext object
     * @return the users session
     */
    public static ServletContext getContext(ViewContext vc) {
        EventContext ec = vc.getEventContext();
        if (ec!=null && (ec instanceof ControlEventContext)) {
            return getContext((ControlEventContext) ec);
        } else {
            return null;
        }
    }
    
    /**
     * get the servlet context from a ControlEventContext
     *
     * @param ec the ControlEventContext object
     * @return the users session
     */
    public static ServletContext getContext(ControlEventContext ec) {
        return ec.getConfig().getServletContext();
    }
    
    /**
     * This method retrieves the servlet context from a ViewContext, 
     * and then looks for an object in the servlet context based on a given key. 
     * If the object is not present, it will be created using the 
     * ReferenceFactory and cached in servlet context for future use.
     *
     * @param vc the ViewContext
     * @param key the key that identifies this object
     * @param factory the ReferenceFactory used to create the object
     * @return the object from the cache 
     */
    public static Object getObjectFromCache(ViewContext vc, Object key, ReferenceFactory factory) {
        return getObjectFromCache(getContext(vc), key, factory);
    }
    
    /**
     * This method retrieves the servlet context from a ControlEventContext, 
     * and then looks for an object in the servlet context based on a given key. 
     * If the object is not present, it will be created using the 
     * ReferenceFactory and cached in servlet context for future use.
     *
     * @param ec the ControlEventContext
     * @param key the key that identifies this object
     * @param factory the ReferenceFactory used to create the object
     * @return the object from the cache 
     */
    public static Object getObjectFromCache(ControlEventContext ec, Object key, ReferenceFactory factory) {
        return getObjectFromCache(getContext(ec), key, factory);
    }
    
    /**
     * This method looks for an object in the servlet context based on a given key.
     * If the object is not present, it will be created using the ReferenceFactory
     * and cached in session for future use.
     *
     * @param context the ServletContext
     * @param key the key that identifies this object
     * @param factory the ReferenceFactory used to create the object
     * @return the object from the cache 
     */
    public static Object getObjectFromCache(ServletContext context, Object key, ReferenceFactory factory) {
        Reference r = (Reference) context.getAttribute(KEY+key);
        Object obj = null;
        if (r!=null) obj = r.get();
        if (r==null || obj==null) {
            r = factory.getObjectReference();
            obj = r.get();
            context.setAttribute(KEY+key, r);
            if (logger.isDebugEnabled()) logger.debug(0, "Created reference:"+r);
        }
        if (logger.isDebugEnabled()) logger.debug(0, "Returning object from cache:"+obj);
        return obj;    
    }
    

    
}
