package org.enhydra.barracuda.core.util.l10n;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.logging.*;


/**
 * Simple locale utilities. This class makes it easy to determine the target
 * locale from an event context or a servlet request. You can also set the locale
 * and ask the class to save the information for you (in a cookie and/or the
 * session) so that the locale info will persist across muliple requests
 */
public class Locales {

    //default param names
    public static String LANGAUGE_KEY = "$loc_lang";
    public static String COUNTRY_KEY = "$loc_cntry";
    public static String VARIANT_KEY = "$loc_var";

    //persistance options
    public static final int NONE = 0;
    public static final int SESSION = 1;
    public static final int COOKIES_AND_SESSION = 2;

    //default persistance option
    public static int PERSIST_DEFAULT = SESSION;

    protected static BCategory logger = BCategory.getBInstance(Locales.class.getName());
    private static final String LOCALE = Localize.class.getName()+".Locale";

    /**
     * Get the client locale from an EventContext using the default
     * param keys and persist option. This will probably be the method
     * you use most frequently.
     *
     * @param ec the EventContext from which we'd like to determine Locale
     * @return the target client Locale
     */
    public static Locale getClientLocale(EventContext ec) {
        return getClientLocale(ec, LANGAUGE_KEY, COUNTRY_KEY, VARIANT_KEY, PERSIST_DEFAULT);
    }

    /**
     * Get the client locale from an EventContext
     *
     * @param ec the EventContext from which we'd like to determine Locale
     * @param languageKey the key to be used to look in the request for a
     *        language paramter
     * @param countryKey the key to be used to look in the request for a
     *        country paramter
     * @param variantKey the key to be used to look in the request for a
     *        variant paramter
     * @param persistOption how we'd like to persist the Locale (by default, it will be stored
     *        in the SESSION)
     * @return the target client Locale
     */
    public static Locale getClientLocale(EventContext ec, String languageKey, String countryKey, String variantKey, int persistOption) {
        HttpServletRequest req = (HttpServletRequest) ec.getState(ControlEventContext.HTTP_SERVLET_REQUEST);
        HttpServletResponse resp = (HttpServletResponse) ec.getState(ViewEventContext.HTTP_SERVLET_RESPONSE);
        return getClientLocale(req, resp, languageKey, countryKey, variantKey, persistOption);
    }

    /**
     * Get the client locale from a ServletRequest using the default
     * param keys and persist option
     *
     * @param req the HttpServletRequest from which we'd like to determine Locale
     * @param resp the HttpServletResponse (needed if we want to save Locale in a cookie
     *        otherwise it may be null)
     * @return the target client Locale
     */
    public static Locale getClientLocale(HttpServletRequest req, HttpServletResponse resp) {
        return getClientLocale(req, resp, LANGAUGE_KEY, COUNTRY_KEY, VARIANT_KEY, PERSIST_DEFAULT);
    }

    /**
     * This method attempts to get the client locale.
     *
     * <p>First we look to see if we can determine the locale by the req's
     * form parameters (language, country, variant). If not, we look in
     * the session to see if we can get the information from there. If
     * that fails, we see if we can get the information from a client
     * cookie (where the value is a comma delimited string containing
     * language, country, and variant. If we still haven't got the locale
     * information, we try and retrieve it from the servlet request, and
     * if that fails, we use the default locale.
     *
     * <p>In each of these cases we save (depending on persistOption) the
     * locale info in both the session and in a client cookie (so as to
     * speed lookups on subsequent requests).
     *
     * @param req the HttpServletRequest from which we'd like to determine Locale
     * @param resp the HttpServletResponse (needed if we want to save Locale in a cookie
     *        otherwise it may be null)
     * @param languageKey the key to be used to look in the request for a
     *        language paramter
     * @param countryKey the key to be used to look in the request for a
     *        country paramter
     * @param variantKey the key to be used to look in the request for a
     *        variant paramter
     * @param persistOption how we'd like to persist the Locale (by default, it will be stored
     *        in the SESSION)
     * @return the target client Locale
     */
    public static Locale getClientLocale(HttpServletRequest req, HttpServletResponse resp, String languageKey, String countryKey, String variantKey, int persistOption) {
        Locale loc = null;
        if (req!=null) {
            //first look for manual form parameters; if we find it here
            //be sure to store the locale both as a cookie and in the session
            if (loc==null) {
                String language = req.getParameter(languageKey);
                String country = req.getParameter(countryKey);
                String variant = req.getParameter(variantKey);
                if (language!=null) {
                    if (logger.isDebugEnabled()) logger.debug("Found locale in req params");
                    loc = getLocale(language, country, variant);
                    saveClientLocale(req, resp, loc, persistOption);
                }
            }

            //next look in the session to see if we can find it there; if so,
            //make sure we save it as a cookie too
            if (loc==null && persistOption>=SESSION) {
                HttpSession session = SessionServices.getSession(req);
                if (session!=null) {
                    if (logger.isDebugEnabled()) logger.debug("Found locale in session");
                    loc = (Locale) session.getAttribute(LOCALE);
                    saveClientLocale(req, resp, loc, persistOption);
                }
            }

            //next see if there is a locale cookie; if there is,
            //make sure we set the locale in the session as well
            if (loc==null && persistOption>=COOKIES_AND_SESSION) {
                Cookie[] cookies = req.getCookies();
                if (cookies!=null) {
                    for (int i=0, max=cookies.length; i<max; i++) {
                        if (cookies[i].getName().equals(LOCALE)) {
                            if (logger.isDebugEnabled()) logger.debug("Found locale in cookie");
                            loc = readLocaleFromCookie(cookies[i]);
                            saveClientLocale(req, resp, loc, persistOption);
                        }
                    }
                }
            }

            //if we still haven't found a locale, look for it in the servlet
            //request; be sure to store the locale both as a cookie and in
            //the session
            if (loc==null) {
                if (logger.isDebugEnabled()) logger.debug("Getting it from servlet request");
                loc = req.getLocale();
                saveClientLocale(req, resp, loc, persistOption);
            }
        }
        //finally as a last resort, create a default locale; be sure to
        //store the locale both as a cookie and in the session
        if (loc==null) {
            if (logger.isDebugEnabled()) logger.debug("Finding default locale");
            loc = Locale.getDefault();
            saveClientLocale(req, resp, loc, persistOption);
        }
        if (logger.isDebugEnabled()) logger.debug("return locale "+loc);
        return loc;
    }

    private static Locale getLocale(String language, String country, String variant) {
        Locale locale = null;
        if (language!=null) {
            if (country==null) country = "";
            if (variant!=null) locale = new Locale(language, country, variant);
            else locale = new Locale(language, country);
        }
        return locale;
    }


    private static Locale readLocaleFromCookie(Cookie cookie) {
        String value = cookie.getValue();
        String language = null;
        String country = null;
        String variant = null;
        StringTokenizer st = new StringTokenizer(value,",");
        if (st.hasMoreTokens()) language = st.nextToken();
        if (st.hasMoreTokens()) country = st.nextToken();
        if (st.hasMoreTokens()) variant = st.nextToken();
        Locale locale = getLocale(language, country, variant);
        if (logger.isDebugEnabled()) logger.debug("Got locale "+locale+" from cookie! (value="+value+")");
        return locale;
    }

    private static void writeLocaleInSession(HttpServletRequest req, Locale locale) {
        if (req==null) return;
        HttpSession session = SessionServices.getSession(req);
        if (locale!=null) session.setAttribute(LOCALE, locale);
        else session.removeAttribute(LOCALE);
        if (logger.isDebugEnabled()) logger.debug("updated locale: "+locale+" in session!");
    }

    private static void writeLocaleInCookie(HttpServletResponse resp, Locale locale) {
        if (resp==null) return;
        String value = null;
        if (locale!=null) value = locale.getLanguage()+","+locale.getCountry()+","+locale.getVariant();
        Cookie cookie = new Cookie(LOCALE, value);
        cookie.setMaxAge(locale!=null ? Integer.MAX_VALUE : 0);
        resp.addCookie(cookie);
        if (logger.isDebugEnabled()) logger.debug("updated locale: "+locale+" in cookie!");
    }


    /**
     * Save the client locale using an EventContext using the default
     * persist option
     *
     * @param ec the EventContext in which we'd like to set Locale
     * @param loc the target client locale we'd like to set
     */
    public static void saveClientLocale(EventContext ec, Locale loc) {
        saveClientLocale(ec, loc, PERSIST_DEFAULT);
    }

    /**
     * Save the client locale using an EventContext
     *
     * @param ec the EventContext in which we'd like to set Locale
     * @param loc the target client locale we'd like to set
     * @param persistOption the specific persistOption to be used
     */
    public static void saveClientLocale(EventContext ec, Locale loc, int persistOption) {
        HttpServletRequest req = (HttpServletRequest) ec.getState(ControlEventContext.HTTP_SERVLET_REQUEST);
        HttpServletResponse resp = (HttpServletResponse) ec.getState(ViewEventContext.HTTP_SERVLET_RESPONSE);
        saveClientLocale(req, resp, loc, persistOption);
    }

    /**
     * Here we actually force a locale to be saved using the default
     * persist option. It will be stored in a cookie and/or the session.
     *
     * @param req the HttpServletRequest (needed to get the HttpSession)
     * @param resp the HttpServletResponse (needed if we want to save Locale
     *        in a cookie otherwise it may be null)
     * @param loc the target client locale we'd like to set
     */
    public static void saveClientLocale(HttpServletRequest req, HttpServletResponse resp, Locale loc) {
        saveClientLocale(req, resp, loc, PERSIST_DEFAULT);
    }

    /**
     * Here we actually force a locale to be saved. It will be stored in
     * a cookie and/or the session.
     *
     * @param req the HttpServletRequest (needed to get the HttpSession)
     * @param resp the HttpServletResponse (needed if we want to save Locale
     *        in a cookie otherwise it may be null)
     * @param loc the target client locale we'd like to set
     * @param persistOption the specific persistOption to be used
     */
    public static void saveClientLocale(HttpServletRequest req, HttpServletResponse resp, Locale loc, int persistOption) {
        //save it in the session
        if (persistOption>=SESSION) writeLocaleInSession(req, loc);

        //save it as a cookie
        if (persistOption>=COOKIES_AND_SESSION) writeLocaleInCookie(resp, loc);
    }

    /**
     * Release the client locale using an EventContext (this effectively
     * removes it from whereever it might have been persisted). This means that
     * on the next request the locale will be determined from scratch again.
     *
     * @param ec the EventContext in which we'd like to set Locale (this tells us
     *        where the locale info needs to be removed from)
     */
    public static void releaseClientLocale(EventContext ec) {
        releaseClientLocale(ec, PERSIST_DEFAULT);
    }

    /**
     * Release the client locale using an EventContext (this effectively
     * removes it from whereever it might have been persisted). This means that
     * on the next request the locale will be determined from scratch again.
     *
     * @param ec the EventContext in which we'd like to set Locale
     * @param persistOption the specific persistOption to be used (this tells us
     *        where the locale info needs to be removed from)
     */
    public static void releaseClientLocale(EventContext ec, int persistOption) {
        HttpServletRequest req = (HttpServletRequest) ec.getState(ControlEventContext.HTTP_SERVLET_REQUEST);
        HttpServletResponse resp = (HttpServletResponse) ec.getState(ViewEventContext.HTTP_SERVLET_RESPONSE);
        releaseClientLocale(req, resp);
    }

    /**
     * Release a client locale (this effectively removes it from whereever
     * it might have been persisted)
     *
     * @param req the HttpServletRequest (needed to get the HttpSession)
     * @param resp the HttpServletResponse (needed if we want to clear Locale
     *        from a cookie otherwise it may be null)
     * @param ec the EventContext in which we'd like to set Locale
     */
    public static void releaseClientLocale(HttpServletRequest req, HttpServletResponse resp) {
        releaseClientLocale(req, resp, PERSIST_DEFAULT);
    }

    /**
     * Release a client locale (this effectively removes it from whereever
     * it might have been persisted)
     *
     * @param req the HttpServletRequest (needed to get the HttpSession)
     * @param resp the HttpServletResponse (needed if we want to clear Locale
     *        from a cookie otherwise it may be null)
     * @param persistOption the specific persistOption to be used (this tells us
     *        where the locale info needs to be removed from)
     */
    public static void releaseClientLocale(HttpServletRequest req, HttpServletResponse resp, int persistOption) {
        //save it in the session
        if (persistOption>=SESSION) writeLocaleInSession(req, null);

        //save it as a cookie
        if (persistOption>=COOKIES_AND_SESSION) writeLocaleInCookie(resp, null);
    }

    /**
     * This utility function will run through a list of Locales
     * and return the index of the locale that matches most closely.
     * If none of them match whatsoever, return the defaultIndex
     *
     * @param locale the target locale
     * @param locales[] the array of locales to search
     * @param the default index (if no match is found)
     * @return the index of the closest matching locale
     */
    public static int findClosestLocale(Locale targetLocale, Locale[] locales, int defaultIndex) {
        int match2 = -1;
        int match1 = -1;

        for (int i=0, max=locales.length; i<max; i++) {
            //if we find a direct match (language, country, variant) return immediately
            if (targetLocale.equals(locales[i])) return i;

            //if we can match on 2, save a reference to the index
            if (targetLocale.getLanguage().equals(locales[i].getLanguage()) &&
                targetLocale.getCountry().equals(locales[i].getCountry())) match2 = i;

            //if we just match on language, save a reference to the index
            if (targetLocale.getLanguage().equals(locales[i].getLanguage())) match1 = i;
        }
        if (match2!=-1) return match2;
        else if (match1!=-1) return match1;
        else return defaultIndex;
    }

}
