/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DefaultXmlcUtilsImpl.java,v 1.3 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.taskdefs;

// import project specifics

// import logging specifics
import org.apache.log4j.Category;

// import java specifics
import java.io.File;
import java.util.List;
import java.util.ArrayList;

/**
 * Default implementation of the XmlcUtils class
 *
 * @author Robert Leftwich
 * @version 1.0
 */
public class DefaultXmlcUtilsImpl extends XmlcUtils {

  //===========================================================================
  // CONSTRUCTORS
  //===========================================================================

  /**
   * Construct default DefaultXmlcUtilsImpl (protected to force the use
   * of XmlcUtils.create() factory method).
   */
  protected DefaultXmlcUtilsImpl() {
    super();
  }

  //===========================================================================
  // QUERY METHODS
  //===========================================================================

  /**
   * Get a list of options files for the specified directory.
   * This method will search for the default options file name
   * (options.xmlc) in the specified directory and the user.dir
   * directory as well. It will return them in that order.
   *
   * @param theDirectory
   *               The directory to look in for the options file.
   * @return An array of options files with absolute paths.
   *         An empty array if none found.
   */
  public String[] getOptionFiles(File theDirectory) {
    return this.getOptionFiles(theDirectory,(String)null);
  }

  /**
   * Get a list of options files for the specified directory,
   * having the specified name. This method will search for the
   * specified options file name and the default options file name
   * (options.xmlc) in the specified directory and the user.dir
   * directory as well. It will return them in that order.
   *
   * @param theDirectory
   *               The directory to look in for the options file.
   * @param theOptionsFileName
   *               The name of the options file to look for.
   * @return An array of options files with absolute paths.
   *         An empty array if none found.
   */
  public String[] getOptionFiles(File theDirectory, String theOptionsFileName) {
    File[] dirs = { theDirectory };
    return this.getOptionFiles(dirs, theOptionsFileName);
  }

  /**
   * Get a list of options files for the specified directory,
   * having the specified names. This method will search for the
   * specified options file names and the default options file name
   * (options.xmlc) in the specified directory and the user.dir
   * directory as well. It will return them in that order.
   *
   * @param theDirectory
   *               The directory to look in for the options file.
   * @param theOptionsFileNames
   *               The list of names of the options file to look for.
   * @return An array of options files with absolute paths.
   *         An empty array if none found.
   */
  public String[] getOptionFiles(File theDirectory, String[] theOptionsFileNames) {
    File[] dirs = { theDirectory };
    return this.getOptionFiles(dirs, theOptionsFileNames);
  }

  /**
   * Get a list of options files for the specified directories,
   * having the specified name. This method will search for the
   * specified options file name and the default options file name
   * (options.xmlc) in the specified directories and the user.dir
   * directory as well. It will return them in that order.
   *
   * @param theDirectories
   *               The list of directories to look in for the options files.
   * @param theOptionsFileName
   *               The name of the options file to look for.
   * @return An array of options files with absolute paths.
   *         An empty array if none found.
   */
  public String[] getOptionFiles(File[] theDirectories, String theOptionsFileName) {
    String[] files = { theOptionsFileName };
    return this.getOptionFiles(theDirectories, files);
  }

  /**
   * Get a list of options files for the specified directories,
   * having the specified names. This method will search for the
   * specified options file names and the default options file name
   * (options.xmlc) in the specified directories and the user.dir
   * directory as well. It will return them in that order.
   *
   * @param theDirectories
   *               The list of directories to look in for the options files.
   * @param theOptionsFileNames
   *               The list of names of the options file to look for.
   * @return An array of options files with absolute paths.
   *         An empty array if none found.
   */
  public String[] getOptionFiles(File[] theDirectories, String[] theOptionsFileNames) {

    String[] optionsFileNames = theOptionsFileNames;

    if (null != optionsFileNames && optionsFileNames.length > 0) {
      for (int i = 0; i < optionsFileNames.length; i++) {
        String currFileName = optionsFileNames[i];
        if (null == currFileName || currFileName.length() == 0) {
          optionsFileNames[i] = DEFAULT_OPTIONS_FILENAME;
        }
        if (!optionsFileNames[i].endsWith(DEFAULT_OPTIONS_FILETYPE)) {
          File tmpFile = new File(optionsFileNames[i]);
          String tmpName = tmpFile.getName();
          int idx = tmpName.lastIndexOf(".");
          if (-1 == idx) {
            idx = tmpName.length();
          }
          StringBuffer newFileName = new StringBuffer(tmpName.substring(0,idx));
          newFileName.append(DEFAULT_OPTIONS_FILETYPE);
          optionsFileNames[i] = newFileName.toString();
        }
      }
    } else {
      String[] defaultOptionsFileNames = {DEFAULT_OPTIONS_FILENAME};
      optionsFileNames = defaultOptionsFileNames;
    }

    ArrayList allOptionsFiles = new ArrayList();

    for (int fileNameIdx = 0; fileNameIdx < optionsFileNames.length; fileNameIdx++) {
      String currFileName = optionsFileNames[fileNameIdx];
      for (int dirIdx = 0; dirIdx < theDirectories.length; dirIdx++) {
        File currDir = theDirectories[dirIdx];
        this.addFileIfExists(new File(currDir, currFileName),
                              allOptionsFiles);
        this.addFileIfExists(new File(currDir, DEFAULT_OPTIONS_FILENAME),
                              allOptionsFiles);
      }
      // look in user.dir as well
      this.addFileIfExists(new File(currFileName),
                            allOptionsFiles);
    }
    this.addFileIfExists(new File(DEFAULT_OPTIONS_FILENAME),
                          allOptionsFiles);

    String[] retVal = new String[ allOptionsFiles.size() ];
    for (int i = 0; i < allOptionsFiles.size(); i++) {
      retVal[i] = (String)allOptionsFiles.get(i);
      //theirLogger.debug("Found options file: " + retVal[i]);
    }

    return retVal;
  }

  /**
   * Build a full base file name (i.e. with no file type) out of
   * the specified components.
   *
   * Note that any directory prefix on the theBaseFileName file is stripped before
   * thePackageDir is applied. So that the following call :
   *
   *     buildFullBaseFileName("a.b.c", "foo/resources", "config/test.html");
   *
   * will return a string of "a/b/c/foo/resources/test"
   *
   * and this call :
   *
   *     buildFullBaseFileName("a.b.c", "", "config/test.html");
   *
   * will return a string of "a/b/c/config/test"
   *
   * @param thePackageName
   *               The name of the package to prepend to the file name.
   *               This should be specified using the Java naming convention
   *               i.e. org.enhydra.xmlc
   * @param thePackageDir
   *               The name of the directory to prepend to the file name.
   *               This is a directory path that can be specified using either '/' or '\'
   *               i.e. foo/resources
   * @param theBaseFileName
   *               The base file name that the preceding strings are prepended to.
   * @return A string representing the full path resulting from the specified components
   *         with all separators set to '/'
   */
  public String buildFullBaseFileName(String thePackageName,
                                       String thePackageDir,
                                       String theBaseFileName) {
    File baseFile;
    int dot = theBaseFileName.indexOf(".");
    if (dot != -1)
      baseFile = new File(theBaseFileName.substring(0, dot));
    else
      baseFile = new File(theBaseFileName);
    if (thePackageDir != null && thePackageDir.length() > 0)
    {
      baseFile = new File(thePackageDir,
                           baseFile.getName());
    }
    if (thePackageName != null && thePackageName.length() > 0)
    {
      baseFile = new File(thePackageName.replace('.', File.separatorChar),
                           baseFile.getPath());
    }
    return baseFile.getPath().replace(File.separatorChar, '/');
  }

  /**
   * Build a Java class name out of the specified components.
   *
   * @param theFullBaseFileName
   *               The full base file name (specified with '/' separators)
   *               and no file type. e.g. "org.enhydra.test"
   * @param theModifier
   *               The file modifier to append to the class name
   *               e.g. HTML
   * @return A Java class name string
   */
  public String buildClassName(String theFullBaseFileName,
                                String theModifier) {
    // Synthesise class name based on path and filename.
    String className = theFullBaseFileName.replace('/', '.');
    className = className.replace(File.separatorChar, '.') + theModifier;
    return className;
  }

  //===========================================================================
  // MUTATOR METHODS
  //===========================================================================

  //===========================================================================
  // HELPER METHODS
  //===========================================================================

  /**
   * Helper function to add a file to a list of files if the file
   * is a file, is readable and is not in the list already.
   *
   * @param theFile The file to check
   * @param theFileNameList
   *                 The list to add the file to
   */
  private void addFileIfExists(File theFile, List theFileNameList) {
    if (theFile.exists() && theFile.isFile()) {
      String path = theFile.getAbsolutePath();
      if (!theFileNameList.contains(path)) {
        theFileNameList.add (path);
      }
    }
  }

  //===========================================================================
  // DATA MEMBERS
  //===========================================================================

  //===========================================================================
  // STATIC DATA MEMBERS
  //===========================================================================

  /**
   * The loggger for this class
   */
  static Category theirLogger = Category.getInstance(DefaultXmlcUtilsImpl.class.getName());

}
