/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *  @author     christianc@lutris.com
 *
 * $Id: EventBuilder.java,v 1.5 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.taskdefs;

import java.util.*;
import java.io.*;
import org.apache.tools.ant.*;
import org.apache.tools.ant.taskdefs.*;
import org.apache.tools.ant.types.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * <p>This Ant taskdef reads in an xml file that conforms to 
 * BarracudaEventBuilder.dtd and generates a set of event classes 
 * according to the specified structure. The primary benefit of 
 * this is that it makes it very easy to declaratively define event 
 * hierarchies that still get compiled to real Java classes (thereby 
 * retaining the benefits of strong typing which you get through the 
 * manual approach)
 *
 * <p>This taskdef extends the Javac taskdef, so you should refer to
 * that documentation for paramters that affect compilation.
 *
 * <p>This taskdef provides one additional <strong>descriptor<strong>
 * parameter which should refer to the location of a valid xml file 
 * describing the event hierarchy.
 *
 * <p>Look at the Barracuda build.xml file for further usage examples.
 *
 * @author Christian Cryder <a href="mailto:christianc@enhydra.org">christianc@enhydra.org</a>
 */
public class EventBuilder extends Javac {

    //private constants
    private static final String DEFAULT_PARSER = "org.apache.xerces.parsers.SAXParser";

    //private vars
    protected String parserClass = null;
    protected File xmlFile = null; //the xml file

    /**
     * Sets the xml event descriptor file.
     */
    public void setDescriptor(File xmlFile) {
        this.xmlFile = xmlFile;
    }

    /**
     * Parse the specified event.xml file, generate event classes from 
     * it, and then compile the resulting classes.
     */
    public void execute() throws BuildException {
        //instantiate the handler, entity resolver
        LocalHandler handler = new LocalHandler();
        LocalEntityResolver resolver = new LocalEntityResolver();
        
        //now parse the event.xml file
        try {
            //instantiate the parser
            if (parserClass==null) parserClass = DEFAULT_PARSER;
            log("Instantiating parser "+parserClass, Project.MSG_DEBUG);
            XMLReader parser = (XMLReader) Class.forName(parserClass).newInstance();
            parser.setContentHandler(handler);
            parser.setErrorHandler(handler);
            parser.setEntityResolver(resolver);

            //get the source file
            log("Generating events from "+xmlFile, Project.MSG_DEBUG);
            InputSource source = new InputSource(new FileInputStream(xmlFile));
            
            //parse the file            
            parser.parse(source);
            
        } catch (org.xml.sax.SAXParseException spe) {
            spe.printStackTrace(System.err);
            throw new BuildException("SAX Error parsing event.xml!", spe);
        } catch (org.xml.sax.SAXException se) {
            if (se.getException() != null) {
                se.getException().printStackTrace(System.err);
                throw new BuildException("SAX Error processing event.xml!", se.getException());
            } else {
                se.printStackTrace(System.err);
                throw new BuildException("SAX Error processing event.xml!", se);
           }
        } catch (Exception e) {
            e.printStackTrace(System.err);
            throw new BuildException("Error processing event.xml!", e);
        } finally {
            //finally, make sure we clean up any accessed resources
            if (resolver!=null) resolver.cleanup();
        }
        
        //finally, compile the newly created files
        super.execute();
    }

    /**
     * helper class to keep track of the various settings for a 
     * given event node
     */
    class CurrentSettings {
        String template = null;
        String eventPackage = "";
        String eventName = null;
        String baseClass = null;
        CurrentSettings parent = null;
                
        public CurrentSettings createChild() {
            CurrentSettings cs = new CurrentSettings();
            cs.template = template;
            cs.baseClass = baseClass;
            cs.eventName = eventName;
            cs.eventPackage = eventPackage;
            cs.parent = CurrentSettings.this;
            return cs;
        }
        
        public CurrentSettings getRoot() {
            if (parent==null) return CurrentSettings.this;
            else return parent.getRoot();
        }

        public String getParentClassName() {
            return (parent.eventName==null ? baseClass : parent.eventPackage+"."+parent.eventName);
        }
        
        public String toString() {
            return "{"+eventName+" extends "+getParentClassName()+"}";
        }
    }

    /**
     * Helper class to actually parse the XML file (we use SAX)
     */
    class LocalHandler extends DefaultHandler {

        //tags
        String BUILD_EVENTS = "build-events";
        String REQ_EVENTS = "req-events";
        String RESP_EVENTS = "resp-events";
        String EVENT = "event";
        
        //attributes
        String PKG = "pkg";
        String TEMPLATE = "template";
        String BASE_CLASS = "base-class";
        String NAME = "name";
        
        //variables
        CurrentSettings cs = null;
        byte[] templateBytes = null;
        int fileCnt = 0;

        public void startDocument() {
            //set up the default settings
            cs = new CurrentSettings();
            cs.template = null;
            cs.baseClass = null;
            cs.eventName = null;
            cs.eventPackage = ".";
        }

        public void startElement(String uri, String local, String raw, Attributes attrs) throws SAXException {
            String curTag = local;

            //whenever we start a new element, the first thing to do is to push
            //a copy of the current settings onto the stack
            cs = cs.createChild();

            //unload the attributes
            for (int i=0, max=attrs.getLength(); i<max; i++) {
                String name = attrs.getLocalName(i);
                String value = attrs.getValue(i);
                if (name.equals(PKG)) {
                    cs.eventPackage = value;                
                } else if (name.equals(TEMPLATE)) {
                    cs.template = value;                
                } else if (name.equals(BASE_CLASS)) {
                    cs.baseClass = value;                
                } else if (name.equals(NAME)) {
                    cs.eventName = value;                
                }
            }

            //build-events (load the template)
            if (curTag.equals(BUILD_EVENTS)) {
                loadTemplate();
                
            //req-events
            } else if (curTag.equals(REQ_EVENTS)) {
                if (cs.baseClass==null) cs.baseClass = "org.enhydra.barracuda.core.event.HttpRequestEvent";

            //resp-events
            } else if (curTag.equals(RESP_EVENTS)) {
                if (cs.baseClass==null) cs.baseClass = "org.enhydra.barracuda.core.event.HttpResponseEvent";

            //event
            } else if (curTag.equals(EVENT)) {
                boolean created = buildEventFile();
                if (created) fileCnt++;
            }
        }
        
        public void endElement(String uri, String local, String raw) throws SAXException {
            cs = cs.parent;
        }

        public void endDocument() {
            if (fileCnt>0) log("Created "+fileCnt+" event files from "+xmlFile);
        }

        private String getLocationString(SAXParseException ex) {
            StringBuffer str = new StringBuffer();
            String systemId = ex.getSystemId();
            if(systemId!=null) {
                int index = systemId.lastIndexOf(47);
                if(index!=-1) systemId = systemId.substring(index + 1);
                str.append(systemId);
            }
            str.append(':');
            str.append(ex.getLineNumber());
            str.append(':');
            str.append(ex.getColumnNumber());
            return str.toString();
        }
        
        protected void loadTemplate() {
            if (templateBytes==null) {
                InputStream is = null;
                templateBytes = new byte[10240];
                try {
                    //if the specified a different template, load it from there
                    //otherwise use the default template from the classpath
                    if (cs.template!=null) is = new FileInputStream(cs.template);
                    else is = this.getClass().getResourceAsStream("EventBuilder.template");

                    //read in the file and store it in a byte array
                    BufferedInputStream in = new BufferedInputStream(is);
                    int offset = 0;
                    int len = 1024;
                    byte[] inbytes = new byte[len];
                    while (true) {
                        int cnt = in.read(inbytes, 0, len);
                        if (cnt==-1) break;
                        if (offset+cnt>=templateBytes.length) {
                            byte[] newbytes = new byte[(int) (templateBytes.length+(len*5))];
                            System.arraycopy(templateBytes, 0, newbytes, 0, templateBytes.length);
                            templateBytes = newbytes;
                        }
                        System.arraycopy(inbytes, 0, templateBytes, offset, cnt);
                        offset+=cnt;
                    }
            
                    //when we're done trim off the excess
                    byte[] newbytes = new byte[offset];
                    System.arraycopy(templateBytes, 0, newbytes, 0, offset);
                    templateBytes = newbytes;
                                         
                } catch (IOException e) {
                    throw new BuildException("Error reading template", e);
                }
            }        
        }
        
        protected boolean buildEventFile() {
            //first of all, see if we need to create a new event file
            String targetPackage = cs.eventPackage.replace('.','/');
            File targetPath = new File(getSrcdir()+"/"+targetPackage);
            File targetFile = new File(targetPath.toString(), cs.eventName+".java");
            
            //if the target path does not exist create it
            if (!targetPath.exists()) {
//csc_091801 - this doesn't compile in JDK 1.2.2...
//                if (!targetPath.mkdirs()) throw new BuildException("Error creating path: "+targetPath, location);
                if (!targetPath.mkdirs()) throw new BuildException("Error creating path: "+targetPath, getLocation());
    	    }

            //if the file exists and is newer than the events.xml timestamp,
            //skip it
            if (targetFile.exists() && 
                targetFile.lastModified()>xmlFile.lastModified()) return false;
        
            //create the file from the template
            try {
                targetFile.createNewFile();
                FileOutputStream out = new FileOutputStream(targetFile);
                out.write(templateBytes, 0, templateBytes.length);
                out.flush();
                out.close();
            } catch (IOException e) {
                throw new BuildException("Error writing "+targetFile, e);
            }
            
            //now perform the replace
            Replace replace = new Replace();
//csc_091801 - this doesn't compile in JDK 1.2.2...
//            replace.setProject(project);
            replace.setProject(getProject());
            replace.setFile(targetFile);
            Replace.Replacefilter rf = null;
            //...package name
            rf = replace.createReplacefilter();
            rf.setToken("@event.package@");
            rf.setValue(cs.eventPackage);
            //...class name
            rf = replace.createReplacefilter();
            rf.setToken("@event.name@");
            rf.setValue(cs.eventName);
            //...parent class name
            rf = replace.createReplacefilter();
            rf.setToken("@event.parent@");
            rf.setValue(cs.getParentClassName());
            //...execute it!
            replace.execute();
            return true;
        }
    }
    
    /**
     * local entity resolver to convert references to remote DTDs
     * to local resources that can be loaded from the classpath
     */    
    class LocalEntityResolver implements EntityResolver {
        String id = "http://barracuda.enhydra.org/Barracuda";
        List resourceList = new ArrayList();
    
        public InputSource resolveEntity (String publicId, String systemId) {
            if (systemId.startsWith(id)) {
                // return a special input source
                String altId = "/xlib"+systemId.substring(id.length());
                InputStream is = this.getClass().getResourceAsStream(altId);
                BufferedInputStream in = new BufferedInputStream(is);
                resourceList.add(in);
                return new InputSource(in);
            } else {
                // use the default behaviour
                return null;
            }
        }
        
        public void cleanup() {
            Iterator it = resourceList.iterator();
            while (it.hasNext()) {
                try {
                    ((InputStream) it.next()).close();
                } catch (Exception e) {
                    System.out.println("unexpected error: "+e);
                }
            }
        }
    }
}
