/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *  @author     christianc@lutris.com
 *
 * Changes
 * -----------------------
 *
 * rtl_20010610 - modified to use the forceRebuild setting from Xmlc base class
 *              - added code to save/restore setting of forceRebuild flag
 *                around second pass
 *
 * $Id: Localize.java,v 1.21 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.taskdefs;

import java.util.*;
import java.io.*;
//import java.lang.reflect.*;
import org.apache.tools.ant.*;
import org.apache.tools.ant.taskdefs.*;
import org.apache.tools.ant.types.*;
import org.w3c.dom.*;
import org.w3c.tidy.Tidy;


import org.enhydra.xml.io.*;
//RTL import org.enhydra.barracuda.core.util.dom.*;


/**
 * ANT task to invoke the localize HTML template files. Supports all the
 * attributes found in the Xmlc taskdef, plus the addition of a "tidy"
 * attr (defaults to true) that pretty prints the generated templates.
 * Also invokes javac compile, whereas the standalone xmlc taskdef doesn't
 * do that.
 *
 * @author christianc@lutris.com
 */
public class Localize extends Xmlc {

    static {
        // if not already has a xmlc utils class installed
        // then install the localize specific version
        if (null == System.getProperty(XmlcUtils.DEFAULT_UTILS_CLASS_KEY)) {
            System.setProperty(XmlcUtils.DEFAULT_UTILS_CLASS_KEY, LocalizeXmlcUtilsImpl.class.getName());
        }
    }

    protected static String logHdr1 = " [localize] ";
    protected String classpathRef = null;
    protected boolean tidy = true;
    
    public final static String[] countryCodes = new String[] {
        "aa", "ab", "af", "am", "ar", "as", "ay", "az", "ba", "be", "bg", "bh",
        "bi", "bn", "bo", "br", "ca", "co", "cs", "cy", "da", "de", "dz", "el",
        "en", "eo", "es", "et", "eu", "fa", "fi", "fj", "fo", "fr", "fy", "ga",
        "gd", "gl", "gn", "gu", "ha", "he", "hi", "hr", "hu", "hy", "ia", "id",
        "ie", "ik", "is", "it", "iu", "ja", "jw", "ka", "kk", "kl", "km", "kn",
        "ko", "ks", "ku", "ky", "la", "ln", "lo", "lt", "lv", "mg", "mi", "mk",
        "ml", "mn", "mo", "mr", "ms", "mt", "my", "na", "ne", "nl", "no", "oc",
        "om", "or", "pa", "pl", "ps", "pt", "qu", "rm", "rn", "ro", "ru", "rw",
        "sa", "sd", "sg", "sh", "si", "sk", "sl", "sm", "sn", "so", "sq", "sr",
        "ss", "st", "su", "sv", "sw", "ta", "te", "tg", "th", "ti", "tk", "tl",
        "tn", "to", "tr", "ts", "tt", "tw", "ug", "uk", "ur", "uz", "vi", "vo",
        "wo", "xh", "yi", "yo", "za", "zh", "zu"
        };

    /**
     * Set the classpath ref for javac.
     */
    public void setClasspathref(String iclasspathRef) {
        classpathRef = iclasspathRef;
    }

    /**
     * Run tidy on the localized templates
     * @param tidy Set "true" for tidy
     */
    public void setTidy(BooleanAttribute itidy) {
        tidy = (itidy.getValue().equals("yes") || itidy.getValue().equals("true"));
    }

    /**
     * Execute the task. The basic flow here is: invoke xmlc to generate
     * template classes, invoke the java compiler to compile those classes,
     * then use those compiled DOM structures to create localized versions of
     * the XML templates. Finally, reinvoke xmlc and javac to recompile the
     * newly created localized templates.
     */
    public void execute() throws BuildException {
        project.log(logHdr1+"Localizing "+srcDir+"...");

        //start by adjusting the log level
        if (verbose) {
            logLevel = project.MSG_INFO;
        } else {
            logLevel = project.MSG_DEBUG;
        }
    
        //start by allowing the default XMLC processing to occur
        //(this will create all the .java templates)
        project.log(logHdr1+"Invoking XMLC precompiler taskdef...", logLevel);
        int origLogLevel = logLevel;
        logLevel = project.MSG_DEBUG;
        super.execute();
        logLevel = origLogLevel;
        

        //now do a compile on the XMLC generated source
        project.log(logHdr1+"Invoking Javac compiler taskdef...", logLevel);
        InternalJavac compile = new InternalJavac();
        compile.setProject(project);
        compile.setSrcdir(new Path(project, srcOutDir.getPath()));
        compile.setDestdir(destDir);
        if (classpathRef!=null) compile.setClasspathRef(new Reference(classpathRef));
        compile.execute();

        //now we can create localized templates
        project.log(logHdr1+"Searching for XMLC Localization templates...", logLevel);

        // now scan source dir
        boolean wroteFiles = false;
        int fileCount = 0;
        int templateCount = 0;
        DirectoryScanner ds = this.getDirectoryScanner(srcDir);
        String[] files = ds.getIncludedFiles();
        //RTL CustomClassLoader ccloader = new CustomClassLoader();
        CustomClassLoader ccloader = new CustomClassLoader(project, compile.getClasspath());
        iterateFiles: for (int i=0; i<files.length; i++) {
            //read our file list
//            String file = files[i].toString();
            String file = files[i];
            String fileName = file;
            String fileExt = null;
            int epos = file.lastIndexOf(".");
            if (epos>-1) {
                fileName = file.substring(0,epos);
                if (epos+1<file.length()) fileExt = file.substring(epos+1);
            }

            //first and foremost, figure out if we're looking at an
            //existing localized template file. If so, skip it.
            if (fileName.indexOf("_")>0) {
                String tfile = fileName;
                if (!tfile.endsWith("_")) tfile += "_";
                StringTokenizer st = new StringTokenizer(tfile, "_");
                while (st.hasMoreTokens()) {
                    String token = st.nextToken();
                    int pos = Arrays.binarySearch(countryCodes, token);
                    if (pos>-1 && pos<countryCodes.length && countryCodes[pos].equals(token)) continue iterateFiles;
                }
            }

//csc_082401_start
            //see if the master template has a .xmlc file. If not, create it.
            File xmlcFile = new File(srcDir, fileName+".xmlc");
            createMasterXMLC: if (!xmlcFile.exists()) {
                //create the new .xmlc file
                BufferedWriter writer = null;
                try {
                    //write the contents out
                    writer = new BufferedWriter(new FileWriter(xmlcFile));
                    writer.write("-generate both");              
                    writer.newLine();
                    writer.flush();
                    
                    //touch the template so it'll get recompiled
                    File templateFile = new File(srcDir, files[i]);
                    templateFile.setLastModified(System.currentTimeMillis());
                    
                } catch (IOException e) {
                    if (writer!=null) try {writer.close();} catch (IOException ioe) {}
                }
            }
//csc_082401_end
            
            //figure out if the template has a properties file
            File propFile = new File(srcDir, fileName+".properties");
            if (!propFile.exists()) continue;
            long propLastTouched = propFile.lastModified();
            
            //now see if there are other properties files with different locale info
            DirectoryScanner dsLocales = new DirectoryScanner();
            String[] includes = {fileName+"*.properties"};
            dsLocales.setIncludes(includes);
            dsLocales.setBasedir(srcDir);
            dsLocales.scan();
            String[] localeFiles = dsLocales.getIncludedFiles();
            List locales = new ArrayList();
            HashMap propBundleMap = new HashMap();
            int spos = fileName.length();
            
//edk_082201            for (int j=0; j<localeFiles.length; j++) {
            iteratePropFiles:  for (int j=0; j<localeFiles.length; j++) {       //edk_082201
                //make sure we're loking at a localization prop file
                String localeFile = localeFiles[j];
                epos = localeFile.length()-11;
                String localeSubstr = localeFile.substring(spos,epos);
                if (!(localeSubstr.startsWith("_"))) continue;

//edk_082201_start
                // get the html-equivalent of the localization prop file
                int dotPos = localeFile.indexOf(".");
                String htmlFile = localeFile.substring(0, dotPos) + "." + fileExt;

                File localizedHtmlFile = new File(srcDir, htmlFile);
                File localizedPropFile = new File(srcDir, localeFile);
//edk_082301.1_start
//                File baseClassFile = new File(destDir, fileName + ML + ".class");
                String baseClassFileName = "";
                if (packageDir!=null) baseClassFileName = packageDir + "/";
                baseClassFileName += fileName + ML + ".class";


//csc_082401_start
                //see if the generated template has an .xmlc file. If not, create it.
                String xmlcOptionsFileName = localeFile.substring(0, dotPos)+".xmlc";
                File xmlcOptionsFile = new File(srcDir, xmlcOptionsFileName);
                boolean autogeneratedOptionsFile = false;
                createMasterXMLC: if (!xmlcOptionsFile.exists()) {

                    //figure out parent class name
                    String clStr = baseClassFileName;
                    int epos1 = clStr.indexOf(".class");
                    clStr = clStr.substring(0,epos1); 
                    clStr = clStr.replace('\\','.');
                    clStr = clStr.replace('/','.');

                    //create the new .xmlc file
                    BufferedWriter writer = null;
                    try {
                        //write the contents out
                        writer = new BufferedWriter(new FileWriter(xmlcOptionsFile));
                        writer.write("-implements "+clStr);              
                        writer.newLine();
                        writer.flush();
                        autogeneratedOptionsFile = true;
                    } catch (IOException e) {
                        if (writer!=null) try {writer.close();} catch (IOException ioe) {}
                    }
                }
//csc_082401_end

                //File baseClassFile = new File(destDir, fileName + ML + ".class");
                File baseClassFile = new File(destDir, baseClassFileName);
//edk_082301.1_end
                if (localizedHtmlFile.exists()) {
                    File baseHtmlFile = new File(srcDir, fileName + "." + fileExt);

                    long localizedHtmlLastTouched = localizedHtmlFile.lastModified();
                    long outputPropLastTouched = localizedPropFile.lastModified();
                    long baseClassLastTouched = -1;
                    if (baseClassFile!=null) baseClassLastTouched = baseClassFile.lastModified();

//edk_082301.1_start
//                    if (!forceBuild &&
//                         localizedHtmlLastTouched>=propLastTouched &&
//                         localizedHtmlLastTouched>=baseClassLastTouched &&
//                         localizedHtmlLastTouched>=outputPropLastTouched) {
//                        project.log(logHdr1+"        Template up-to-date: "+localizedHtmlFile.getName(), logLevel);
//                        continue iteratePropFiles;
//                    }

                    // rebuild if:
                    // 1) forceBuild is true, or
                    // 2) the base properties file is more recent than the locale html file, or
                    // 3) the base html class file is more recent than the locale html file, or
                    // 4) the locale properties file is more recent than the locale html file
                    boolean rebuild =
                        (forceBuild ||
                         autogeneratedOptionsFile ||
                         propLastTouched>=localizedHtmlLastTouched ||
                         baseClassLastTouched>=localizedHtmlLastTouched ||
                         outputPropLastTouched>=localizedHtmlLastTouched);

                    // if the base html file has a .xmlc file associated with it, factor
                    // that into the rebuild logic
                    String baseXmlcFileName = fileName + ".xmlc";
                    File baseXmlcFile = new File(srcDir, baseXmlcFileName);
                    if (baseXmlcFile.exists() && baseXmlcFile.canRead())  {
                        // rebuild if
                        // 5) the base .xmlc file is more recent than the locale html file
                        long baseXmlcLastTouched = baseXmlcFile.lastModified();
                        rebuild = rebuild || (baseXmlcLastTouched>=localizedHtmlLastTouched);
                    }

                    // if the locale html file has a .xmlc file associated with it, factor that
                    // into the rebuild logic
                    String localeXmlcFileName = localeFile.substring(0, dotPos) + ".xmlc";
                    File localeXmlcFile = new File(srcDir, localeXmlcFileName);
                    if (localeXmlcFile.exists() && localeXmlcFile.canRead())  {
                        // rebuild if
                        // 6) the locale-specific
                        long localeXmlcLastTouched = localeXmlcFile.lastModified();
                        rebuild = rebuild || (localeXmlcLastTouched>=localizedHtmlLastTouched);
                    }

                    if (!rebuild)  {
                        project.log(logHdr1+"        Template up-to-date: "+localizedHtmlFile.getName(), logLevel);
                        continue iteratePropFiles;
                    }
//edk_082301.1_end
                }
//edk_082201_end

                //figure out the language, country, and variant                
                StringTokenizer st = new StringTokenizer(localeSubstr, "_");
                String language = "";
                String country = "";
                String variant = null;
                if (st.hasMoreTokens()) language = st.nextToken();
                if (st.hasMoreTokens()) country = st.nextToken();
                if (st.hasMoreTokens()) {
                    String sep = "";
                    while (st.hasMoreTokens()) {
//081201_edk_start
//Ed Keen [edk@interactiveportal.com] submitted the following fix: 
//If variant is null (as it is in the first pass), then you will 
//get a string like "nullfoo."  This causes problems later in execution.
//                        variant += st.nextToken();
                        if (variant==null) variant = st.nextToken();
                        else variant += st.nextToken();
//081201_edk_end
                        sep = "_";
                    }
                }
                
                //figure out the targetLocale
                Locale targetLocale = null;
                if (variant!=null) targetLocale = new Locale(language, country, variant);
                else targetLocale = new Locale(language, country);
                locales.add(targetLocale);

                //find the actual prop file and place it in the propFileMap
                File localePropFile = new File(srcDir, fileName+localeSubstr+".properties");
                if (!localePropFile.exists()) {                                
                    project.log(logHdr1+"        Unable to read localization properties file "+localePropFile, project.MSG_WARN);
                    continue;
                }
                PropertyResourceBundle rb = null;
                try {rb = new PropertyResourceBundle(new FileInputStream(localePropFile));}
                catch (Exception e) {
                    project.log(logHdr1+"        Unable to create resource bundle for file "+localePropFile, project.MSG_WARN);
                    continue;
                }
                propBundleMap.put(localeSubstr, rb);
            }

//081201_edk_start
            // if there are no locales to work with, continue to the next html file
            if (locales.size()==0) continue;
//081201_edk_end

            //see if we can instantiate the template using xmlc
            Class cl = ccloader.findClass(destDir, packageDir, fileName);
            //RTL long sourceLastTouched = ccloader.lastModified();
            if (cl==null) {
                project.log(logHdr1+"        Unable to find class!", project.MSG_WARN);
                continue;
            }

//081201_edk_start_remove
/*
            //RTL-start
            File classFile = ccloader.getClassFile(destDir, packageDir, fileName);
            long sourceLastTouched = -1;
            if (null != classFile) {
                sourceLastTouched = classFile.lastModified();
            }
            //RTL-end
*/            
//081201_edk_end_remove

            Object o = null;
            try {o = cl.newInstance();}
            catch (Exception e) {
                project.log(logHdr1+"        Error instantiating class "+cl, project.MSG_WARN);
                continue;
            }
            if (!(o instanceof Document)) {
//csc_032701                project.log("        Template "+fileName+" does not implement Document interface", project.MSG_WARN);
                project.log(logHdr1+"        Template "+cl.getName()+" does not implement Document interface", project.MSG_WARN);    //csc_032701
                continue;
            }
            Document templateDoc = (Document) o;
            project.log(logHdr1+"    Loaded template source "+cl, logLevel);
            fileCount++;

            //load the master properties file (what we're doing here is creating
            //a map where we can look up keys by value)
            HashMap valueMap = new HashMap();
            Properties masterProps = new Properties();
            try {masterProps.load(new FileInputStream(propFile));}
            catch (Exception e) {
                project.log(logHdr1+"        Unable to read master properties file "+propFile, project.MSG_WARN);
                continue;
            }
            Enumeration enum = masterProps.propertyNames();
            while (enum.hasMoreElements()) {
                String key = (String) enum.nextElement();
                String value = masterProps.getProperty(key);
                valueMap.put(value, key);            
            }
    
             //now process each target locale
            Object locArray[] = locales.toArray();
            Document docArray[] = new Document[locArray.length];
            for (int j=0; j<locArray.length; j++) {
                //figure out the locale extension
                List locList = new ArrayList();
                Locale loc = (Locale) locArray[j];
                String locExt = "";
                String buf = loc.getLanguage();
                if (buf!=null && buf.length()>0) {
                    locExt+="_"+buf;
                    locList.add(0,propBundleMap.get(locExt));
                    buf = loc.getCountry();
                    if (buf!=null && buf.length()>0) {
                        locExt+="_"+buf;
                        locList.add(0,propBundleMap.get(locExt));
                        buf = loc.getVariant();
                        if (buf!=null && buf.length()>0) {
                            locExt+="_"+buf;
                            locList.add(0,propBundleMap.get(locExt));
                        }
                    }
                }

                //create the output file
                File outputFile = new File(srcDir, fileName+locExt+"."+fileExt);
                File outputProps = new File(srcDir, fileName+locExt+".properties");
                
//081201_edk_start_remove
/*
                //make sure it actually needs to be written. If force rebuild set or
                // if the template date is older than
                //  a) the default properties file OR
                //  b) the default java template OR
                //  c) the localized properties file
                //then it will be rewritten. Otherwise, just continue on...
                if (outputFile.exists()) {
                    long templateLastTouched = outputFile.lastModified();
                    long outputPropLastTouched = outputProps.lastModified();
                    // rtl_20010610 if (
                    if (!forceBuild &&                     // rtl_20010610
                            templateLastTouched>=propLastTouched &&
                            templateLastTouched>=sourceLastTouched &&
                            templateLastTouched>=outputPropLastTouched) {
                        project.log(logHdr1+"        Template up-to-date: "+outputFile.getName(), logLevel);
                        continue;
                    }
                }
*/            
//081201_edk_end_remove

                //start by instantiating a doc that will represent each locale
                Document doc = null;
                try {doc = (Document) cl.newInstance();}
                catch (Exception e) {
                    project.log(logHdr1+"        Unexpected error:"+e, project.MSG_WARN);
                    continue;
                }
                wroteFiles = true;
                templateCount++;

                //now walk the dom and make substitutions
                localizeNode(doc, valueMap, locList);
            
                //write the doc
                try {
                    //if we're tidying
//right now there is a bug in a) this code or b) the tidy formatter...it's causing
//a blank space to be inserted into <td> elements that only contain images, not text
//This majorly screws up formatting...until I can figure out a workaround, tidy
//will just be turned off. [NOTE: its a bug in Tidy, or in how we're invoking Tidy]
//                    if (tidy) {
                    if (false) {
                        //write the doc to a byte stream
                        DOMFormatter df = new DOMFormatter();
                        byte[] bytes = df.toBytes(doc);

                        //filter the byte stream to tidy
                        Tidy tidy = new Tidy();
                        tidy.setIndentContent(true);
                        tidy.setSpaces(2);
                        tidy.setQuiet(true);
                        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        tidy.parse(bais, baos);
                        
                        //now write the file
                        FileWriter fw = new FileWriter(outputFile);
                        fw.write(baos.toString());
                        fw.close();
                    //compact                    
                    } else {
                        //create a file writer and let the formatter write directly to it
                        FileWriter fw = new FileWriter(outputFile);
                        DOMFormatter df = new DOMFormatter();
                        df.write(doc, fw);
                        fw.close();
                    }                    
                    project.log(logHdr1+"        Template created: "+outputFile.getName(), logLevel);
                } catch (Exception e) {
                    project.log(logHdr1+"        Unexpected error writing template "+outputFile.getName()+":"+e, project.MSG_WARN);
                }
            }     
        }    

        //now, if we actually wrote out new files we need to
        //do phase 2...
        if (wroteFiles) {
            //now invoke XMLC processing again. This will cause the newly created
            //templates to get compiled via XMLC
            project.log(logHdr1+"Invoking XMLC precompiler taskdef (pass 2)...", logLevel);
            origLogLevel = logLevel;
            logLevel = project.MSG_VERBOSE;
            boolean origforceBuild = forceBuild;    // rtl_20010610 - save current setting
            forceBuild = false;    //never force on the 2nd pass
            super.execute();
            logLevel = origLogLevel;

            //finally, invoke javac one more time to ensure that the new created
            //templates get compiled as well
            project.log(logHdr1+"Invoking Javac compiler taskdef (pass 2)...", logLevel);
            compile.execute();
            forceBuild = origforceBuild;                    // rtl_20010610 - reset
        }
        project.log(logHdr1+"Finished!", logLevel);
        project.log(logHdr1+"Processed "+files.length+" source files, found "+fileCount+" master templates, created "+templateCount+" new localized templates"+(fileCount>0 && templateCount==0 ? " (all files up-to-date)" : ""));
    }


    /**
     * The idea here is to look up a value in the value map to match
     * it to the corresponding id. Then we look at each prop file contained
     * in the loc list successively until we find a match. We then substitute
     * that value into the node.
     */
    protected void localizeNode(Node node, HashMap valueMap, List locList) {
    
/*
There's an interesting opportunity for improvement here. Consider the following examples:

Opt 1
-----
Config.html --> <input ... value="Test" ...>
Config.properties --> Config.General.Reset = Test
Config_en.properties --> Config.General.Reset = Hey, it worked!

Opt 2
-----
Config.html --> <input ... value="Test&uuml;" ...>
Config.properties --> Config.General.Reset = Test
Config_en.properties --> Config.General.Reset = Hey, it worked!

Opt 3
-----
Config.html --> <input ... value="Test&uuml;" ...>
Config.properties --> Config.General.Reset = Test&uuml;
Config_en.properties --> Config.General.Reset = Hey, it worked2!

Options 1 and 2 both work. Option 3 does not. The reason is that even though the value
"Test&uuml;" is physically embedded in the markup template, the DOM getValue() method as
used below returns it as "Test", which consequently doesn't have a match in the
properties file.

A good improvement here would be to do something where if we don't find a localization
match, convert any special characters in the string to their correspond markup
equivalents and then recheck the localization files for that value. So! if you get a
piece of text like "Test" and you don't find a match, try looking for a match on
Test&uuml;" as well...

This would make the localization process a little easier to use, because it would then
work in all cases.
*/    
        //look first for text
        if (node instanceof Text) {
            Text txtNode = (Text) node;
            String txt = txtNode.getData();
            String id = (String) valueMap.get(txt);
            if (id!=null) {
                Iterator it = locList.iterator();
                iterate_rbs: while (it.hasNext()) {
                    ResourceBundle rb = (ResourceBundle) it.next();
                    String val = null;
                    try {val = rb.getString(id);}
                    catch (MissingResourceException e) {}
                    if (val!=null) {
                        CDATASection cdsNode = node.getOwnerDocument().createCDATASection(val);
                        Node parent    = txtNode.getParentNode();
                        parent.replaceChild(cdsNode, txtNode);
//                        txtNode.setData(val);
                        break iterate_rbs;
                    }
                }
            }
        }

        //next look for attributes
        NamedNodeMap nnm = node.getAttributes();
        if (nnm!=null) for (int i=0; i<nnm.getLength(); i++) {
            Attr attr = (Attr) nnm.item(i);
            String txt = attr.getValue();
//if (txt.startsWith("Test")) System.out.println ("HERE!!!! txt={"+txt+"}");
            String id = (String) valueMap.get(txt);
            if (id!=null) {
                Iterator it = locList.iterator();
                iterate_rbs: while (it.hasNext()) {
                    ResourceBundle rb = (ResourceBundle) it.next();
                    String val = null;
                    try {val = rb.getString(id);}
                    catch (MissingResourceException e) {}
                    if (val!=null) {
                        attr.setValue(val);
                        break iterate_rbs;
                    }
                }
            }
        }
        
        //localize any children
        if (node.hasChildNodes()) {        
            NodeList nl = node.getChildNodes();
            for (int i=0; i<nl.getLength(); i++) {
                localizeNode(nl.item(i), valueMap, locList);
            }
        }
    
    }

    /**
     * In order to use the Javac taskdef from inside our localize taskdef,
     * we have to be able to set the project name
     */
    class InternalJavac extends Javac {
        
        public InternalJavac() {
      // RTL - remove Jikes warning by adding this.
            this.setTaskName("Localize");
        }
        
        public void setProject(Project iproject) {
      // RTL - remove Jikes warning by adding this.
            this.project = iproject;
        }
    }

    /**
     * This class loader is used to try and load the newly compiled
     * classes from the output classpath
     */    
    class CustomClassLoader extends AntClassLoader {

        public CustomClassLoader(Project project, Path classpath) {
            super(project, classpath);
        }

        public Class findClass(File classDir, String outputPackage, String fileName) {
            //figure out the file name        
            String targetFileName = getFileName(outputPackage, fileName);
            Class cl = null;
//edk_082301_start
//            try {
//                cl = findClass(targetFileName+ML);
//            } catch (ClassNotFoundException ex) {
//                // just return null
//            }
            try {
                // look for the implementation first
                cl = findClass(targetFileName+ML+"Impl");
            } catch (ClassNotFoundException ex) {
                // implementation not found -- look for the regular class
                try  {
                    cl = findClass(targetFileName+ML);
                } catch (ClassNotFoundException ex2) {
                    // just return null
                }
            }
//edk_082301_end

            return cl;
        }

        private String getFileName(String pkg, String name) {
            String fileName = name.replace('\\','/');
            //note: we have to strip off the package part in the name here,
            //since xmlc compiles templates in subdirectories into the same
            //package
//081301_rs_start
//Roland Schulz [rschulz@f-451.de] submitted the following patch to 
//handle the situation where package name is null...
            if (pkg==null) return fileName;
//081301_rs_end
            int lastSlash = fileName.lastIndexOf('/');
            if (lastSlash>-1) fileName = fileName.substring(lastSlash+1);
            return pkg+'/'+fileName;
        }

        public File getClassFile(File classDir, String outputPackage, String fileName) {
//edk_082301_start
//            String targetFileName = getFileName(outputPackage, fileName);
//            File file = new File(classDir, targetFileName+ML+".class");    //csc
//            if (!file.exists() || !file.canRead()) file = null;
//            return file;
            String targetFileName = getFileName(outputPackage, fileName);

            // look for the implementation first
            File file = new File(classDir, targetFileName+ML+"Impl.class");

            // if it doesn't exist look for the regular class
            if (!file.exists()) file = new File(classDir, targetFileName+ML+".class");
            
            //make sure it exists and we can read it
            if (!file.exists() || !file.canRead()) file = null;
            return file;
//edk_082301_end
        }

    }
    /*class CustomClassLoader extends ClassLoader {
        long lastModified = -1;

        public Class findClass(File classDir, String outputPackage, String fileName) {
        
            //figure out the file name        
            String targetFileName = getFileName(outputPackage, fileName);
            String fqFileName = getFQFileName(targetFileName);

            File targetFile = null;
//csc_032701            Class cl = loadClassData(classDir, targetFileName+"HTML");
//csc_032701            if (cl==null) cl = loadClassData(classDir, targetFileName+"WML");
//csc_032701            if (cl==null) cl = loadClassData(classDir, targetFileName+"XML");
            Class cl = loadClassData(classDir, targetFileName+ML);    //csc_032701
            return cl;
        }
        
        public long lastModified() {
            return lastModified;
        }
        
        private String getFileName(String pkg, String name) {
            String fileName = name.replace('\\','/');
            //note: we have to strip off the package part in the name here,
            //since xmlc compiles templates in subdirectories into the same
            //package
            int lastSlash = fileName.lastIndexOf('/');
            if (lastSlash>-1) fileName = fileName.substring(lastSlash+1);
            return pkg+'/'+fileName;
        }
        
        private String getFQFileName(String fileName) {
            return fileName.replace('/','.');
        }
        

        private Class loadClassData(File classDir, String fileName) {
            //first see if the file has already been loaded
            Class cl = findLoadedClass(fileName);
            if (cl!=null) return cl;
            
            //if not go ahead and try to get it
            File file = new File(classDir, fileName+".class");
            if (!file.exists() || !file.canRead()) return null;
            lastModified = file.lastModified();
            byte[] bytes = new byte[(int) file.length()];
            try {
                FileInputStream fis = new FileInputStream(file);
                fis.read(bytes);
            } catch (IOException e) {
                System.out.println ("Error reading file "+file+":"+e);
            }
            cl = defineClass(null, bytes, 0, bytes.length);
            return cl;
        }
    }*/
    
}
