/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestEqualsValidator.java,v 1.5 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.logging.*;
import tests.util.*;


/**
 * This test verifies that the EqualsValidator works correctly.
 */
public class TestEqualsValidator extends ValidatorTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestEqualsValidator.class.getName();
    private static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestEqualsValidator(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Test String types
     */
    public void testString() {
        //create a form validator and excercise it
        String s1 = "foo";
        EqualsValidator v1 = new EqualsValidator(s1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

        //v1
        assertValid("Error validating foo==foo", v1, el, s1);
        assertValid("Error validating foo.equals(foo)", v1, el, "foo");
        assertInvalid("Error invalidating foo.equals(foo2)", v1, el, "foo2");
        assertInvalid("Error invalidating foo.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null==foo2", v2, el, "foo2");
    }
    
    /**
     * Test Boolean types
     */
    public void testBoolean() {
        //create a form validator and excercise it
        Boolean bT = new Boolean(true);
        Boolean bF = new Boolean(false);
        EqualsValidator v1 = new EqualsValidator(bT);
        EqualsValidator v2 = new EqualsValidator(bF);
        EqualsValidator v3 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);

        //v1
        assertValid("Error validating true==true", v1, el, bT);                        
        assertValid("Error validating true.equals(true)", v1, el, new Boolean(true));
        assertInvalid("Error invalidating true.equals(false)", v1, el, bF);            
        assertInvalid("Error invalidating true.equals(null)", v1, el, null);            

        //v2
        assertValid("Error validating false==false", v2, el, bF);                    
        assertValid("Error validating false.equals(false)", v2, el, new Boolean(false));
        assertInvalid("Error invalidating false.equals(true)", v2, el, bT);    
        assertInvalid("Error invalidating false.equals(null)", v2, el, null);    

        //v3
        assertValid("Error validating null==null", v3, el, null);            
        assertInvalid("Error invalidating null==true", v3, el, bT);            
        assertInvalid("Error invalidating null==false", v3, el, bF);            
    }
    
    /**
     * Test Integer types
     */
    public void testInteger() {
        //create a form validator and excercise it
        Integer i1 = new Integer(1000);
        EqualsValidator v1 = new EqualsValidator(i1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);

        //v1
        assertValid("Error validating i1==i1", v1, el, i1);
        assertValid("Error validating i1.equals(1000)", v1, el, new Integer(1000));
        assertInvalid("Error invalidating i1.equals(2000)", v1, el, new Integer(2000));
        assertInvalid("Error invalidating i1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Integer(2000));
    }

    /**
     * Test Date types
     */
    public void testDate() {
        //create a form validator and excercise it
        Date d1 = new Date();
        Date d2 = new Date(d1.getTime());
        Date d3 = new Date(65000);
        EqualsValidator v1 = new EqualsValidator(d1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);

        //v1
        assertValid("Error validating d1==d1", v1, el, d1);
        assertValid("Error validating d1.equals(d2)", v1, el, d2);
        assertInvalid("Error invalidating d1.equals(d3)", v1, el, d3);
        assertInvalid("Error invalidating d1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(d3)", v2, el, d3);
    }

    /**
     * Test Long types
     */
    public void testLong() {
        //create a form validator and excercise it
        Long l1 = new Long(1000);
        EqualsValidator v1 = new EqualsValidator(l1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);

        //v1
        assertValid("Error validating l1==l1", v1, el, l1);
        assertValid("Error validating l1.equals(1000)", v1, el, new Long(1000));
        assertInvalid("Error invalidating l1.equals(2000)", v1, el, new Long(2000));
        assertInvalid("Error invalidating l1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Long(2000));
    }

    /**
     * Test Short types
     */
    public void testShort() {
        //create a form validator and excercise it
        Short s1 = new Short((short) 1000);
        EqualsValidator v1 = new EqualsValidator(s1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);

        //v1
        assertValid("Error validating s1==s1", v1, el, s1);
        assertValid("Error validating s1.equals(1000)", v1, el, new Short((short) 1000));
        assertInvalid("Error invalidating s1.equals(2000)", v1, el, new Short((short) 2000));
        assertInvalid("Error invalidating s1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Short((short) 2000));
    }

    /**
     * Test Double types
     */
    public void testDouble() {
        //create a form validator and excercise it
        Double d1 = new Double(1000);
        EqualsValidator v1 = new EqualsValidator(d1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);

        //v1
        assertValid("Error validating d1==d1", v1, el, d1);
        assertValid("Error validating d1.equals(1000)", v1, el, new Double(1000));
        assertInvalid("Error invalidating d1.equals(2000)", v1, el, new Double(2000));
        assertInvalid("Error invalidating d1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Double(2000));
    }

    /**
     * Test Float types
     */
    public void testFloat() {
        //create a form validator and excercise it
        Float f1 = new Float(1000);
        EqualsValidator v1 = new EqualsValidator(f1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);

        //v1
        assertValid("Error validating f1==f1", v1, el, f1);
        assertValid("Error validating f1.equals(1000)", v1, el, new Float(1000));
        assertInvalid("Error invalidating f1.equals(2000)", v1, el, new Float(2000));
        assertInvalid("Error invalidating f1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Float(2000));
    }
    
}
