/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestMaxLengthValidator.java,v 1.3 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.logging.*;
import tests.util.*;


/**
 * This test verifies that MaxLengthValidator works correctly.
 */
public class TestMaxLengthValidator extends ValidatorTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestMaxLengthValidator.class.getName();
    private static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestMaxLengthValidator(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Test String types
     */
    public void testString() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating ''.length()<=-1", v1, el, "");
        assertInvalid("Error invalidating foo.length()<=-1", v1, el, "foo");
        assertInvalid("Error invalidating foofoo.length()<=-1", v1, el, "foofoo");
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertValid("Error validating ''.length()<=0", v2, el, "");
        assertInvalid("Error invalidating foo.length()<=0", v2, el, "foo");
        assertInvalid("Error invalidating foofoo.length()<=0", v2, el, "foofoo");
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating ''.length()<=5", v3, el, "");
        assertValid("Error validating foo.length()<=5", v3, el, "foo");
        assertInvalid("Error invalidating foofoo.length()<=5", v3, el, "foofoo");
    }
    
    /**
     * Test Boolean types - this test ensures that if we pass in a 
     * null FormElement or try to validate the length on a Boolean
     * it will always generate a ValidationException
     */
    public void testBoolean() {
        //create a form validator and excercise it
        MaxLengthValidator v = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);
        Boolean bT = new Boolean(true);
        Boolean bF = new Boolean(false);

        //v
        assertInvalid("Error invalidating null element", v, null, "foo");
        assertInvalid("Error invalidating Boolean(true)", v, el, bT);
        assertInvalid("Error invalidating Boolean(false)", v, el, bF);
    }
    
    /**
     * Test Integer types
     */
    public void testInteger() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);
        Integer i = new Integer(123);

        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating i.length()<=-1", v1, el, i);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating i.length()<=0", v2, el, i);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating i.length()<=5", v3, el, i);
    }

    /**
     * Test Date types - this test ensures that if we try to validate 
     * the length on a Date it will always generate a ValidationException
     */
    public void testDate() {
        //create a form validator and excercise it
        MaxLengthValidator v = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);
        Date d = new Date();

        //v
        assertValid("Error validating null date", v, el, null);
        assertInvalid("Error invalidating Date", v, el, d);
    }

    /**
     * Test Long types
     */
    public void testLong() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);
        Long l = new Long(123);

        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating l.length()<=-1", v1, el, l);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating l.length()<=0", v2, el, l);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating l.length()<=5", v3, el, l);
    }

    /**
     * Test Short types
     */
    public void testShort() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);
        Short s = new Short((short)123);

        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating s.length()<=-1", v1, el, s);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating s.length()<=0", v2, el, s);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating s.length()<=5", v3, el, s);
    }

    /**
     * Test Double types
     */
    public void testDouble() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);
        Double d = new Double(123);

        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating d.length()<=-1", v1, el, d);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating d.length()<=0", v2, el, d);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating d.length()<=5", v3, el, d);
    }

    /**
     * Test Float types
     */
    public void testFloat() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);
        Float f = new Float(1.23);

        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating f.length()<=-1", v1, el, f);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating f.length()<=0", v2, el, f);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating f.length()<=5", v3, el, f);
    }
    
}
