/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestNotNullValidator.java,v 1.3 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.logging.*;
import tests.util.*;


/**
 * This test verifies that NotNullValidator works correctly.
 */
public class TestNotNullValidator extends ValidatorTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestNotNullValidator.class.getName();
    private static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestNotNullValidator(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Test String types
     */
    public void testString() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertInvalid("Error invalidating ''==null", v, el, "");
        assertInvalid("Error invalidating '  '==null", v, el, "  ");
        assertValid("Error validating foo!=null", v, el, "foo");
    }
    
    /**
     * Test Boolean types - this test ensures that if we pass in a 
     * null FormElement or try to validate the length on a Boolean
     * it will always generate a ValidationException
     */
    public void testBoolean() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);

        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating true!=null", v, el, new Boolean(true));
        assertValid("Error validating false!=null", v, el, new Boolean(false));
    }
    
    /**
     * Test Integer types
     */
    public void testInteger() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);
        Integer i1 = new Integer(123);

        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating i1!=null", v, el, i1);
    }

    /**
     * Test Date types - this test ensures that if we try to validate 
     * the length on a Date it will always generate a ValidationException
     */
    public void testDate() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);
        Date d = new Date();

        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating d!=null", v, el, d);
    }

    /**
     * Test Long types
     */
    public void testLong() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);
        Long l1 = new Long(123);

        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating l1!=null", v, el, l1);
    }

    /**
     * Test Short types
     */
    public void testShort() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);
        Short s1 = new Short((short)123);

        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating s1!=null", v, el, s1);
    }

    /**
     * Test Double types
     */
    public void testDouble() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);
        Double d1 = new Double(123);

        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating d1!=null", v, el, d1);
    }

    /**
     * Test Float types
     */
    public void testFloat() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);
        Float f1 = new Float(123);

        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating f1!=null", v, el, f1);
    }
    
}
