/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestOptionsFile.java,v 1.5 2001/09/25 19:05:31 cryd0221 Exp $
 */
package org.enhydra.barracuda.taskdefs;

// import junit specifics
import junit.framework.TestCase;

// import logging specifics
import org.apache.log4j.Category;

// import java specifics
import java.io.File;
import java.io.IOException;

/**
 * Unit tests for getOptionFiles() method of DefaultXmlcUtilsImpl
 *
 */
public class TestOptionsFile extends TestCase {

    //===========================================================================
    // CONSTRUCTORS
    //===========================================================================

    /**
     * Default constructor
     *
     * @param theName   The name of the test
     */
    public TestOptionsFile( String theName) {
        super( theName );
    }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     */
    protected void setUp() throws Exception {
        super.setUp();
        // create the test directory if required
        if ( !this.myBaseTestDir.exists() ) {
          assert("Cannot create base test directory:" + this.myBaseTestDir.getAbsolutePath(),
                 this.myBaseTestDir.mkdir() );
        }
    }

    /**
     * Tears down the fixture, for example, close a network connection.
     * This method is called after a test is executed.
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        // delete any xmlc in parent directory of base
        File[] files = this.myBaseTestDir.getAbsoluteFile().getParentFile().listFiles();
        for (int i = 0; i < files.length; i++) {
          File currFile = files[i];
          if ( currFile.getName().endsWith(XmlcUtils.DEFAULT_OPTIONS_FILETYPE) ) {
            assert("Cannot delete: " + currFile.getAbsolutePath(), currFile.delete() );
          }
        }
        this.recursiveDelete( this.myBaseTestDir );
    }

    //===========================================================================
    // TEST METHODS
    //===========================================================================
    public void testDefaultNoOptionsFile() {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        String[] foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir );
        assertEquals("Incorrect no. of option files in empty directory", 0, foundOptionFiles.length );
    }

    public void testOptionsFileNulls() {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        String[] foundOptionFiles = xmlcUtils.getOptionFiles( null );
        assertEquals("Incorrect no. of option files in empty directory", 0, foundOptionFiles.length );

        foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir, (String)null );
        assertEquals("Incorrect no. of option files in empty directory", 0, foundOptionFiles.length );

        foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir, (String[])null );
        assertEquals("Incorrect no. of option files in empty directory", 0, foundOptionFiles.length );

        String[] files = { "test1", null, "test2" };
        foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir, files );
        assertEquals("Incorrect no. of option files in empty directory", 0, foundOptionFiles.length );

        File[] dirs = { null, new File("test1"), null, new File("test2") };
        foundOptionFiles = xmlcUtils.getOptionFiles( dirs, (String)null );
        assertEquals("Incorrect no. of option files in empty directory", 0, foundOptionFiles.length );

        foundOptionFiles = xmlcUtils.getOptionFiles( dirs, (String[])null );
        assertEquals("Incorrect no. of option files in empty directory", 0, foundOptionFiles.length );

        foundOptionFiles = xmlcUtils.getOptionFiles( dirs, files );
        assertEquals("Incorrect no. of option files in empty directory", 0, foundOptionFiles.length );
    }

    public void testDefaultOneOptionsFile() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        File expectedOptionFile = new File( this.myBaseTestDir, XmlcUtils.DEFAULT_OPTIONS_FILENAME );
        assert("Cannot create option file:" + expectedOptionFile.getAbsolutePath(),
               expectedOptionFile.createNewFile() );
        String[] foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir );
        assertEquals("Incorrect no. of option files", 1, foundOptionFiles.length );
    }

    public void testDefaultOptionsFileIgnoresDirectories() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        File expectedOptionFile = new File( this.myBaseTestDir, XmlcUtils.DEFAULT_OPTIONS_FILENAME );
        assert("Cannot create option directory:" + expectedOptionFile.getAbsolutePath(),
               expectedOptionFile.mkdir() );

        String[] foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir );
        assertEquals("Incorrect no. of option files", 0, foundOptionFiles.length );
    }

    public void testDefaultOptionsFileUserDir() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        File expectedOptionFile = new File( XmlcUtils.DEFAULT_OPTIONS_FILENAME );
        assert("Cannot create option file:" + expectedOptionFile.getAbsolutePath(),
               expectedOptionFile.createNewFile() );
        String[] foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir );
        assertEquals("Incorrect no. of option files", 1, foundOptionFiles.length );
    }

    public void testDefaultOptionsFileUserAndSpecified() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        File userDirOptionFile = new File( XmlcUtils.DEFAULT_OPTIONS_FILENAME );
        assert("Cannot create user dir option file:" + userDirOptionFile.getAbsolutePath(),
               userDirOptionFile.createNewFile() );

        File expectedOptionFile = new File( this.myBaseTestDir, XmlcUtils.DEFAULT_OPTIONS_FILENAME );
        assert("Cannot create base option file:" + expectedOptionFile.getAbsolutePath(),
               expectedOptionFile.createNewFile() );
        String[] foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir );

        assertEquals("Incorrect no. of option files", 2, foundOptionFiles.length );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFile.getAbsolutePath(), foundOptionFiles[0] );
        assertEquals( "Incorrect user dir filename: ",
                      userDirOptionFile.getAbsolutePath(), foundOptionFiles[1] );
    }

    public void testOverrideOptionsFileUserAndSpecified() throws IOException {
        String overrideOptionsFilename = "override.xmlc";
        XmlcUtils xmlcUtils = XmlcUtils.create();

        File userDirOptionFile = new File( overrideOptionsFilename );
        assert("Cannot create user dir option file:" + userDirOptionFile.getAbsolutePath(),
               userDirOptionFile.createNewFile() );

        File expectedOptionFile = new File( this.myBaseTestDir, overrideOptionsFilename );
        assert("Cannot create base option file:" + expectedOptionFile.getAbsolutePath(),
               expectedOptionFile.createNewFile() );
        String[] foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir, overrideOptionsFilename );

        assertEquals("Incorrect no. of option files", 2, foundOptionFiles.length );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFile.getAbsolutePath(), foundOptionFiles[0] );
        assertEquals( "Incorrect user dir filename: ",
                      userDirOptionFile.getAbsolutePath(), foundOptionFiles[1] );
    }

    public void testOverrideOptionsFileUserAndSpecifiedUsingDefaultName() throws IOException {
        String overrideOptionsFilename = XmlcUtils.DEFAULT_OPTIONS_FILENAME;
        XmlcUtils xmlcUtils = XmlcUtils.create();

        File userDirOptionFile = new File( overrideOptionsFilename );
        assert("Cannot create user dir option file:" + userDirOptionFile.getAbsolutePath(),
               userDirOptionFile.createNewFile() );

        File expectedOptionFile = new File( this.myBaseTestDir, overrideOptionsFilename );
        assert("Cannot create base option file:" + expectedOptionFile.getAbsolutePath(),
               expectedOptionFile.createNewFile() );
        String[] foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir, overrideOptionsFilename );

        assertEquals("Incorrect no. of option files", 2, foundOptionFiles.length );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFile.getAbsolutePath(), foundOptionFiles[0] );
        assertEquals( "Incorrect user dir filename: ",
                      userDirOptionFile.getAbsolutePath(), foundOptionFiles[1] );
    }

    public void testOptionsFilename() throws IOException {
        String currFile = "test";
        XmlcUtils xmlcUtils = XmlcUtils.create();
        File expectedOptionFile = new File( this.myBaseTestDir, currFile + xmlcUtils.DEFAULT_OPTIONS_FILETYPE );
        assert("Cannot create option file:" + expectedOptionFile.getAbsolutePath(),
               expectedOptionFile.createNewFile() );

        String[] foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir, currFile );
        assertEquals("Incorrect no. of option files for " + currFile, 1, foundOptionFiles.length );
        assertEquals("Found option filename should match expected for " + currFile,
                     expectedOptionFile.getAbsolutePath(),
                     foundOptionFiles[0] );

        foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir, expectedOptionFile.getAbsolutePath() );
        assertEquals("Incorrect no. of option files for " + currFile, 1, foundOptionFiles.length );
        assertEquals("Found option filename should match expected for " + currFile,
                     expectedOptionFile.getAbsolutePath(),
                     foundOptionFiles[0] );
    }

    public void testOverrideAndDefaultOptionsFiles() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        File defaultUserDirOptionFile = new File( XmlcUtils.DEFAULT_OPTIONS_FILENAME );
        assert("Cannot create default user dir option file:" + defaultUserDirOptionFile.getAbsolutePath(),
               defaultUserDirOptionFile.createNewFile() );

        File defaultExpectedOptionFile = new File( this.myBaseTestDir, XmlcUtils.DEFAULT_OPTIONS_FILENAME );
        assert("Cannot create default base option file:" + defaultExpectedOptionFile.getAbsolutePath(),
               defaultExpectedOptionFile.createNewFile() );

        String overrideOptionsFilename = "override.xmlc";
        File userDirOptionFile = new File( overrideOptionsFilename );
        assert("Cannot create user dir option file:" + userDirOptionFile.getAbsolutePath(),
               userDirOptionFile.createNewFile() );

        File expectedOptionFile = new File( this.myBaseTestDir, overrideOptionsFilename );
        assert("Cannot create base option file:" + expectedOptionFile.getAbsolutePath(),
               expectedOptionFile.createNewFile() );

        String[] foundOptionFiles = xmlcUtils.getOptionFiles( this.myBaseTestDir, overrideOptionsFilename );

        assertEquals("Incorrect no. of option files", 4, foundOptionFiles.length );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFile.getAbsolutePath(), foundOptionFiles[0] );
        assertEquals( "Incorrect default expected filename: ",
                      defaultExpectedOptionFile.getAbsolutePath(), foundOptionFiles[1] );
        assertEquals( "Incorrect user dir filename: ",
                      userDirOptionFile.getAbsolutePath(), foundOptionFiles[2] );
        assertEquals( "Incorrect defaultuser dir filename: ",
                      defaultUserDirOptionFile.getAbsolutePath(), foundOptionFiles[3] );
    }

    public void testMultipleOptionsFiles() throws IOException {
        File expectedOptionFile1 = new File( this.myBaseTestDir, "test1.xmlc" );
        assert("Cannot create option file:" + expectedOptionFile1.getAbsolutePath(),
               expectedOptionFile1.createNewFile() );

        File expectedOptionFile2 = new File( this.myBaseTestDir, "test2.xmlc" );
        assert("Cannot create option file:" + expectedOptionFile2.getAbsolutePath(),
               expectedOptionFile2.createNewFile() );

        File expectedOptionFile3 = new File( this.myBaseTestDir, "test3.xmlc" );
        assert("Cannot create option file:" + expectedOptionFile3.getAbsolutePath(),
               expectedOptionFile3.createNewFile() );

        String[] expectedOptionFiles = new String[3];
        expectedOptionFiles[0] = expectedOptionFile3.getName();
        expectedOptionFiles[1] = expectedOptionFile1.getName();
        expectedOptionFiles[2] = expectedOptionFile2.getName();

        XmlcUtils xmlcUtils = XmlcUtils.create();
        String[] foundOptionFiles =
          xmlcUtils.getOptionFiles( this.myBaseTestDir, expectedOptionFiles );
        assertEquals("Incorrect no. of option files",
                     expectedOptionFiles.length, foundOptionFiles.length );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFile3.getAbsolutePath(), foundOptionFiles[0] );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFile1.getAbsolutePath(), foundOptionFiles[1] );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFile2.getAbsolutePath(), foundOptionFiles[2] );
    }

    public void testMultipleDirOptionsFiles() throws IOException {
        File dir1 = new File( this.myBaseTestDir, "Dir1" );
        assert("Cannot create test directory:" + dir1.getAbsolutePath(), dir1.mkdir() );

        File dir2 = new File( this.myBaseTestDir, "Dir2" );
        assert("Cannot create test directory:" + dir2.getAbsolutePath(), dir2.mkdir() );

        File dir3 = new File( this.myBaseTestDir, "Dir1/Dir3" );
        assert("Cannot create test directory:" + dir3.getAbsolutePath(), dir3.mkdir() );

        String optionsFilename = "test.xmlc";
        File expectedOptionFileDir1 = new File( dir1, optionsFilename );
        assert("Cannot create option file:" + expectedOptionFileDir1.getAbsolutePath(),
               expectedOptionFileDir1.createNewFile() );

        File expectedOptionFileDir2 = new File( dir2, optionsFilename );
        assert("Cannot create option file:" + expectedOptionFileDir2.getAbsolutePath(),
               expectedOptionFileDir2.createNewFile() );

        File expectedOptionFileDir3 = new File( dir3, optionsFilename );
        assert("Cannot create option file:" + expectedOptionFileDir3.getAbsolutePath(),
               expectedOptionFileDir3.createNewFile() );

        File[] dirs = new File[3];
        dirs[0] = dir1;
        dirs[1] = dir2;
        dirs[2] = dir3;

        XmlcUtils xmlcUtils = XmlcUtils.create();
        String[] foundOptionFiles = xmlcUtils.getOptionFiles( dirs, optionsFilename );
        assertEquals("Incorrect no. of option files", dirs.length, foundOptionFiles.length );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFileDir1.getAbsolutePath(), foundOptionFiles[0] );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFileDir2.getAbsolutePath(), foundOptionFiles[1] );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFileDir3.getAbsolutePath(), foundOptionFiles[2] );

        // make sure filters out duplicates and uses specified order
        dirs = new File[6];
        dirs[0] = dir3;
        dirs[1] = dir2;
        dirs[2] = dir2;
        dirs[3] = dir1;
        dirs[4] = dir3;
        dirs[5] = dir1;

        foundOptionFiles = xmlcUtils.getOptionFiles( dirs, optionsFilename );
        assertEquals("Incorrect no. of option files", 3, foundOptionFiles.length );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFileDir3.getAbsolutePath(), foundOptionFiles[0] );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFileDir2.getAbsolutePath(), foundOptionFiles[1] );
        assertEquals( "Incorrect expected filename: ",
                      expectedOptionFileDir1.getAbsolutePath(), foundOptionFiles[2] );
    }

    //===========================================================================
    // HELPER  METHODS
    //===========================================================================

    private void recursiveDelete( File theFile ) {
        if ( theFile.isDirectory() ) {
            File[] files = theFile.listFiles();
            for (int i = 0; i < files.length; i++) {
                this.recursiveDelete( files[i] );
            }
        }
        assert("Cannot delete: " + theFile.getAbsolutePath(),
               theFile.delete() );
    }

    //===========================================================================
    // DATA MEMBERS
    //===========================================================================
    private File myBaseTestDir = new File( TEST_DIR );

    //===========================================================================
    // STATIC DATA MEMBERS
    //===========================================================================
    private static final String TEST_DIR="_TestOptionsFile";

    // setup the loggger for this class
    static Category theirLogger = Category.getInstance( TestOptionsFile.class.getName() );
}
